@@ -51,10 +51,8 @@ static const char *find_used_table(STMT *stmt)
     char *table_name;
     MYSQL_RES *result= stmt->result;
 
-    MYODBCDbgEnter;
-
     if ( stmt->table_name && stmt->table_name[0] )
-        MYODBCDbgReturn( stmt->table_name );
+        return stmt->table_name;
 
     table_name= 0;
     for ( field= result->fields, end= field+ result->field_count;
@@ -70,7 +68,7 @@ static const char *find_used_table(STMT *stmt)
             {
                 set_error(stmt,MYERR_S1000,
                           "Can't modify a row from a statement that uses more than one table",0);
-                MYODBCDbgReturn( NULL );
+                return NULL;
             }
         }
 #else
@@ -82,7 +80,7 @@ static const char *find_used_table(STMT *stmt)
             {
                 set_error(stmt,MYERR_S1000,
                           "Can't modify a row from a statement that uses more than one table",0);
-                MYODBCDbgReturn( NULL );
+                return NULL;
             }
         }
 #endif
@@ -92,7 +90,7 @@ static const char *find_used_table(STMT *stmt)
       while using cursors.
     */
     stmt->table_name= dupp_str(table_name,SQL_NTS);
-    MYODBCDbgReturn( stmt->table_name );
+    return stmt->table_name;
 }
 
 
@@ -185,61 +183,113 @@ char *check_if_positioned_cursor_exists(STMT *pStmt, STMT **pStmtCursor)
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : checks whether the Primary Key column exists in the table
-  if it exists, returns the PK column name
+/**
+  Check if a field exists in a result set.
+
+  @param[in]  name    Name of the field
+  @param[in]  result  Result set to check
 */
-
-static SQLRETURN check_if_pk_exists(STMT FAR *stmt)
+static my_bool have_field_in_result(const char *name, MYSQL_RES *result)
 {
-    char buff[NAME_LEN+18];
-    MYSQL_ROW row;
-    MYSQL_RES *presult;
+  MYSQL_FIELD  *field;
+  unsigned int ncol;
 
-    if ( stmt->cursor.pk_validated )
-        return(stmt->cursor.pk_count);
-
-    /*
-      Check for the existence of keys in the table
-      We quote the table name to allow weird table names.
-      TODO: Write a table-name-quote function and use this instead.
-    */
+  for (ncol= 0; ncol < result->field_count; ncol++)
+  {
+    field= result->fields + ncol;
+    if (myodbc_strcasecmp(name,
 #if MYSQL_VERSION_ID >= 40100
-    strxmov(buff,"show keys from `",stmt->result->fields->org_table,"`",NullS);
+                          field->org_name
 #else
-    strxmov(buff,"show keys from `",stmt->result->fields->table,"`",NullS);
+                          field->name
 #endif
-    MYLOG_QUERY(stmt, buff);
-    pthread_mutex_lock(&stmt->dbc->lock);
-    if ( mysql_query(&stmt->dbc->mysql,buff) ||
-         !(presult= mysql_store_result(&stmt->dbc->mysql)) )
-    {
-        set_error(stmt,MYERR_S1000,mysql_error(&stmt->dbc->mysql),
-                  mysql_errno(&stmt->dbc->mysql));
-        pthread_mutex_unlock(&stmt->dbc->lock);
-        return(0);
-    }
+                          ) == 0)
+      return TRUE;
+  }
 
-    /*
-      TODO: Fix this loop to only return columns that are part of the
-      primary key.
-    */
-    while ( (row= mysql_fetch_row(presult)) &&
-            (stmt->cursor.pk_count < MY_MAX_PK_PARTS) )
-    {
-        /*
-          Collect all keys, it may be
-          - PRIMARY or
-          - UNIQUE NOT NULL
-          TODO: Fix this seperately and use the priority..
-        */
-        strmov(stmt->cursor.pkcol[stmt->cursor.pk_count++].name,row[4]);
-    }
-    mysql_free_result(presult);
+  return FALSE;
+}
+
+
+/**
+  Check if a primary or unique key exists in the table referred to by
+  the statement for which all of the component fields are in the result
+  set. If such a key exists, the field names are stored in the cursor.
+
+  @param[in]  stmt  Statement
+
+  @return  Whether a usable unique keys exists
+*/
+static my_bool check_if_usable_unique_key_exists(STMT *stmt)
+{
+  char buff[NAME_LEN * 2 + 18], /* Possibly escaped name, plus text for query */
+       *pos, *table;
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  int seq_in_index= 0;
+
+  if (stmt->cursor.pk_validated)
+    return stmt->cursor.pk_count;
+
+#if MYSQL_VERSION_ID >= 40100
+  if (stmt->result->fields->org_table)
+    table= stmt->result->fields->org_table;
+  else
+#endif
+    table= stmt->result->fields->table;
+
+  /* Use SHOW KEYS FROM table to check for keys. */
+  pos= strmov(buff, "SHOW KEYS FROM `");
+  pos+= mysql_real_escape_string(&stmt->dbc->mysql, pos, table, strlen(table));
+  pos= strmov(pos, "`");
+
+  MYLOG_QUERY(stmt, buff);
+
+  pthread_mutex_lock(&stmt->dbc->lock);
+  if (mysql_query(&stmt->dbc->mysql, buff) ||
+      !(res= mysql_store_result(&stmt->dbc->mysql)))
+  {
+    set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+              mysql_errno(&stmt->dbc->mysql));
     pthread_mutex_unlock(&stmt->dbc->lock);
-    stmt->cursor.pk_validated= 1;
-    return(stmt->cursor.pk_count);
+    return FALSE;
+  }
+
+  while ((row= mysql_fetch_row(res)) &&
+         stmt->cursor.pk_count < MY_MAX_PK_PARTS)
+  {
+    int seq= atoi(row[3]);
+
+    /* If this is a new key, we're done! */
+    if (seq <= seq_in_index)
+      break;
+
+    /* Unless it is non_unique, it does us no good. */
+    if (row[1][0] == '1')
+      continue;
+
+    /* If this isn't the next part, this key is no good. */
+    if (seq != seq_in_index + 1)
+      continue;
+
+    /* Check that we have the key field in our result set. */
+    if (have_field_in_result(row[4], stmt->result))
+    {
+      /* We have a unique key field -- copy it, and increment our count. */
+      strmov(stmt->cursor.pkcol[stmt->cursor.pk_count++].name, row[4]);
+      seq_in_index= seq;
+    }
+    else
+      /* Forget about any key we had in progress, we didn't have it all. */
+      stmt->cursor.pk_count= seq_in_index= 0;
+  }
+  mysql_free_result(res);
+  pthread_mutex_unlock(&stmt->dbc->lock);
+
+  /* Remember that we've figured this out already. */
+  stmt->cursor.pk_validated= 1;
+
+  return stmt->cursor.pk_count > 0;
 }
 
 
@@ -362,14 +412,13 @@ static SQLRETURN copy_rowdata(STMT FAR *stmt, PARAM_BIND  param,
                               NET **net, SQLCHAR **to)
 {
     SQLCHAR *orig_to= *to;
-    MYSQL mysql= stmt->dbc->mysql;
     /* Negative length means either NULL or DEFAULT, so we need 7 chars. */
     SQLUINTEGER length= (*param.actual_len > 0 ? *param.actual_len + 1 : 7);
 
     if ( !(*to= (SQLCHAR *) extend_buffer(*net,(char*) *to,length)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
-    if ( !(*to= (SQLCHAR*) insert_param(&mysql, (char*) *to, &param)) )
+    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc, (char*) *to, &param)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
     /* We have to remove zero bytes or we have problems! */
@@ -381,7 +430,7 @@ static SQLRETURN copy_rowdata(STMT FAR *stmt, PARAM_BIND  param,
     param.buffer= (gptr) ",";
     *param.actual_len= 1;
 
-    if ( !(*to= (SQLCHAR*) insert_param(&mysql,(char*) *to, &param)) )
+    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc,(char*) *to, &param)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
     return(SQL_SUCCESS);
@@ -422,13 +471,12 @@ static SQLRETURN copy_field_data(STMT FAR *stmt, PARAM_BIND *param,
 {
     PARAM_BIND dummy;
     SQLLEN     dummy_len= 5; /* sizeof(" AND ") */
-    MYSQL mysql= stmt->dbc->mysql;
     SQLUINTEGER length= *(param->actual_len)+5;
 
     if ( !(*to= (SQLCHAR*) extend_buffer(*net, (char*) *to,length)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
-    if ( !(*to= (SQLCHAR*) insert_param(&mysql, (char*) *to, param)) )
+    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc, (char*) *to, param)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
     /* Insert " AND ", where clause with multiple search */
@@ -437,7 +485,7 @@ static SQLRETURN copy_field_data(STMT FAR *stmt, PARAM_BIND *param,
     dummy.buffer= (gptr) " AND ";
     dummy.actual_len= &dummy_len;
 
-    if ( !(*to= (SQLCHAR*) insert_param(&mysql, (char*) *to, &dummy)) )
+    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc, (char*) *to, &dummy)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
     return SQL_SUCCESS;
@@ -454,7 +502,6 @@ static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
                             SQLUSMALLINT nSrcCol)
 {
     PARAM_BIND  param;
-    ulong       transfer_length,precision,display_size;
     MYSQL_FIELD *field= mysql_fetch_field_direct(result,nSrcCol);
     MYSQL_ROW   row_data= result->data_cursor->data + nSrcCol;
     NET         *net=&stmt->dbc->mysql.net;
@@ -463,12 +510,7 @@ static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
 
     /* Copy row buffer data to statement */
     param.used= 1;
-    param.SqlType= unireg_to_sql_datatype( stmt,
-                                           field,
-                                           0,
-                                           &transfer_length,
-                                           &precision,
-                                           &display_size );
+    param.SqlType= get_sql_data_type(stmt, field, 0);
     param.CType= SQL_C_CHAR;
 
     if ( row_data && *row_data )
@@ -513,8 +555,7 @@ static my_bool if_float_field(STMT FAR *stmt, MYSQL_FIELD *field)
 /*
   @type    : myodbc3 internal
   @purpose : checks for the existance of pk columns in the resultset,
-  if it is, copy that data to query, else get the data by
-  building a temporary resultset
+  if it is, copy that data to query, else we can't find the right row
 */
 
 static SQLRETURN insert_pk_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
@@ -526,98 +567,36 @@ static SQLRETURN insert_pk_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
     MYCURSOR     *cursor= &stmt->cursor;
     SQLUINTEGER  pk_count= 0;
 
+    /* Look for primary key columns in the current result set, */
+    for (ncol= 0; ncol < result->field_count; ncol++)
+    {
+      field= result->fields+ncol;
+      for (index= 0; index < cursor->pk_count; index++)
+      {
+        if (!myodbc_strcasecmp(cursor->pkcol[index].name, field->org_name))
+        {
+          /* PK data exists...*/
+          dynstr_append_quoted_name(dynQuery, field->org_name);
+          dynstr_append_mem(dynQuery, "=", 1);
+          if (insert_field(stmt, result, dynQuery, ncol))
+            return SQL_ERROR;
+          cursor->pkcol[index].bind_done= TRUE;
+          pk_count++;
+          break;
+        }
+      }
+    }
+
     /*
-      Look for primary key columns in the current result set,
-      if it exists, take that data else query new resultset
+     If we didn't have data for all the components of the primary key,
+     we can't build a correct WHERE clause.
     */
-    for ( ncol= 0; ncol < result->field_count; ncol++ )
-    {
-        field= result->fields+ncol;
-        for ( index= 0; index < cursor->pk_count; index++ )
-        {
-            if ( !myodbc_strcasecmp(cursor->pkcol[index].name,field->name) )
-            {
-                /* PK data exists...*/
-                dynstr_append_quoted_name(dynQuery,field->name);
-                dynstr_append_mem(dynQuery,"=",1);
-                if ( insert_field(stmt,result,dynQuery,ncol) )
-                    return(SQL_ERROR);
-                cursor->pkcol[index].bind_done= TRUE;
-                pk_count++;
-                break;
-            }
-        }
-    }
-    if ( pk_count != cursor->pk_count )
-    {
-        /*
-          Primary key column doesn't exists in the opened rs, so
-          get the data by executing a query
-        */
-        DYNAMIC_STRING query;
-        MYSQL_RES *presult;
-        SQLUSMALLINT field_count= 0;
+    if (pk_count != cursor->pk_count)
+      return set_stmt_error(stmt, "HY000",
+                            "Not all components of primary key are available, "
+                            "so row to modify cannot be identified", 0);
 
-        if ( init_dynamic_string(&query, "SELECT ", 1024,1024) )
-            return set_error(stmt,MYERR_S1001,NULL,4001);
-
-        for ( index= 0; index < cursor->pk_count; index++ )
-        {
-            if ( !cursor->pkcol[index].bind_done )
-            {
-                dynstr_append_quoted_name(&query,stmt->cursor.pkcol[index].name);
-                dynstr_append_mem(&query,",",1);
-            }
-        }
-        query.length-= 1;
-        dynstr_append_mem(&query," FROM ",6);
-
-        if ( !find_used_table(stmt) )
-        {
-            dynstr_free(&query);
-            return(SQL_ERROR);
-        }
-
-        dynstr_append_quoted_name(&query,stmt->table_name);
-        MYLOG_QUERY(stmt, query.str);
-        pthread_mutex_lock(&stmt->dbc->lock);
-        if ( mysql_query(&stmt->dbc->mysql,query.str) ||
-             !(presult= mysql_store_result(&stmt->dbc->mysql)) )
-        {
-            set_error(stmt,MYERR_S1000,mysql_error(&stmt->dbc->mysql),
-                      mysql_errno(&stmt->dbc->mysql));
-            pthread_mutex_unlock(&stmt->dbc->lock);
-            dynstr_free(&query);
-            return(SQL_ERROR);
-        }
-        pthread_mutex_unlock(&stmt->dbc->lock);
-
-        for ( index= 0;index< (uint) stmt->current_row;index++ )
-            presult->data_cursor= presult->data_cursor->next;
-
-        for ( index= 0; index < cursor->pk_count; index++ )
-        {
-            if ( !cursor->pkcol[index].bind_done )
-            {
-                dynstr_append_quoted_name(dynQuery,cursor->pkcol[index].name);
-                dynstr_append_mem(dynQuery,"=",1);
-
-                /*
-                  Might have multiple pk fields in the missing list ..
-                  so avoid the wrong query by having internal field_count..
-                */
-                if ( insert_field(stmt,presult,dynQuery,field_count++) )
-                {
-                    mysql_free_result(presult);
-                    dynstr_free(&query);
-                    return(SQL_ERROR);
-                }
-            }
-        }
-        mysql_free_result(presult);
-        dynstr_free(&query);
-    }
-    return(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -635,15 +614,13 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
   unsigned int  i,j;
   BOOL          found_field;
 
-  MYODBCDbgEnter;
-
   /*
     Get the base table name. If there was more than one table underlying
     the result set, this will fail, and we couldn't build a suitable
     list of fields.
   */
   if (!(find_used_table(stmt)))
-    MYODBCDbgReturn(SQL_ERROR);
+    return SQL_ERROR;
 
   /*
     Get the list of all of the columns of the underlying table by using
@@ -658,7 +635,7 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
     set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
               mysql_errno(&stmt->dbc->mysql));
     pthread_mutex_unlock(&stmt->dbc->lock);
-    MYODBCDbgReturn(SQL_ERROR);
+    return SQL_ERROR;
   }
   pthread_mutex_unlock(&stmt->dbc->lock);
 
@@ -669,7 +646,7 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
   if (mysql_num_fields(presultAllColumns) != mysql_num_fields(result))
   {
     mysql_free_result(presultAllColumns);
-    MYODBCDbgReturn(SQL_ERROR);
+    return SQL_ERROR;
   }
 
   /*
@@ -687,10 +664,8 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
     */
     if (if_float_field(stmt, table_field))
     {
-      MYODBCDbgInfo("field '%s' is a floating-point field",
-                    table_field->name);
       mysql_free_result(presultAllColumns);
-      MYODBCDbgReturn(SQL_ERROR);
+      return SQL_ERROR;
     }
 
     found_field= FALSE;
@@ -704,9 +679,8 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
         dynstr_append_mem(dynQuery, "=", 1);
         if (insert_field(stmt, result, dynQuery, j))
         {
-          MYODBCDbgInfo("failed to append value for '%s'", table_field->name);
           mysql_free_result(presultAllColumns);
-          MYODBCDbgReturn(SQL_ERROR);
+          return SQL_ERROR;
         }
         found_field= TRUE;
         break;
@@ -718,15 +692,13 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
     */
     if (!found_field)
     {
-      MYODBCDbgInfo("could not find field '%s' in result set",
-                    table_field->name);
       mysql_free_result(presultAllColumns);
-      MYODBCDbgReturn(SQL_ERROR);
+      return SQL_ERROR;
     }
   }
 
   mysql_free_result(presultAllColumns);
-  MYODBCDbgReturn(SQL_SUCCESS);
+  return SQL_SUCCESS;
 }
 
 /*
@@ -744,16 +716,21 @@ static SQLRETURN build_where_clause( STMT FAR *       pStmt,
     /* simply append WHERE to our statement */
     dynstr_append_mem( dynQuery, " WHERE ", 7 );
 
-    /* IF pk exists THEN use pk cols for where ELSE use all cols */
-    if ( check_if_pk_exists( pStmt ) )
+    /*
+      If a suitable key exists, then we'll use those columns, otherwise
+      we'll try to use all of the columns.
+    */
+    if (check_if_usable_unique_key_exists(pStmt))
     {
-        if ( insert_pk_fields( pStmt, dynQuery ) != SQL_SUCCESS )
-            return set_stmt_error( pStmt, "HY000", "Build WHERE -> insert_pk_fields() failed.", 0 );
+      if (insert_pk_fields(pStmt, dynQuery) != SQL_SUCCESS)
+        return SQL_ERROR;
     }
     else
     {
-        if ( append_all_fields( pStmt, dynQuery ) != SQL_SUCCESS )
-            return set_stmt_error( pStmt, "HY000", "Build WHERE -> insert_fields() failed.", 0 );
+      if (append_all_fields(pStmt, dynQuery) != SQL_SUCCESS)
+        return set_stmt_error(pStmt, "HY000",
+                              "Build WHERE -> insert_fields() failed.",
+                              0);
     }
     /* Remove the trailing ' AND ' */
     dynQuery->length -= 5;
@@ -804,7 +781,6 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLUINTEGER irow,
                                   DYNAMIC_STRING *dynQuery)
 {
     PARAM_BIND    param;
-    ulong         transfer_length,precision,display_size;
     SQLLEN        length;
     uint          ncol, ignore_count= 0;
     MYSQL_FIELD *field;
@@ -826,7 +802,7 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLUINTEGER irow,
         field= mysql_fetch_field_direct(result,ncol);
         bind= stmt->bind+ncol;
 
-        if ( bind && !bind->field )
+        if (!stmt->bind || (bind && !bind->field))
         {
           ignore_count++;
           continue;
@@ -860,11 +836,7 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLUINTEGER irow,
         dynstr_append_mem(dynQuery,"=",1);
 
         param.used= 1;
-        param.SqlType= unireg_to_sql_datatype( stmt,field,
-                                               0,
-                                               &transfer_length,
-                                               &precision,
-                                               &display_size );
+        param.SqlType= get_sql_data_type(stmt, field, NULL);
         param.CType= bind->fCType;
         param.buffer= (gptr) bind->rgbValue+irow*bind->cbValueMax;
         param.ValueMax= bind->cbValueMax;
@@ -909,7 +881,6 @@ SQLRETURN my_pos_delete(STMT FAR *stmt, STMT FAR *stmtParam,
         return nReturn;
 
     /* DELETE the row(s) */
-    MYODBCDbgInfo( "SQL_DELETE: %s", dynQuery->str );
     nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length);
     if ( nReturn == SQL_SUCCESS || nReturn == SQL_SUCCESS_WITH_INFO )
     {
@@ -1022,7 +993,6 @@ static SQLRETURN setpos_delete(STMT FAR *stmt, SQLUSMALLINT irow,
             return nReturn;
 
         /* execute our DELETE statement */
-        MYODBCDbgInfo( "SQLPOS_DELETE: %s", dynQuery->str );
         if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
             affected_rows+= stmt->dbc->mysql.affected_rows;
 
@@ -1096,7 +1066,6 @@ static SQLRETURN setpos_update(STMT FAR *stmt, SQLUSMALLINT irow,
         if (!SQL_SUCCEEDED(nReturn))
           return nReturn;
 
-        MYODBCDbgInfo( "SQLPOS_UPDATE: %s", dynQuery->str );
         if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
             affected_rows+= stmt->dbc->mysql.affected_rows;
 
@@ -1137,8 +1106,7 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
     SQLUINTEGER  insert_count= 1;           /* num rows to insert - will be real value when row is 0 (all)  */
     SQLUINTEGER  count= 0;                  /* current row                                                  */
     SQLLEN       length;
-    MYSQL        mysql= stmt->dbc->mysql;
-    NET         *net= &mysql.net;
+    NET         *net= &stmt->dbc->mysql.net;
     SQLUSMALLINT ncol;
     SQLCHAR      *to;
     ulong        query_length= 0;           /* our original query len so we can reset pos if break_insert   */
@@ -1172,10 +1140,10 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
             dynstr_append_mem(ext_query,"(", 1);
             for ( ncol= 0; ncol < result->field_count; ncol++ )
             {
-                ulong        transfer_length,precision,display_size;
                 MYSQL_FIELD *field= mysql_fetch_field_direct(result,ncol);
                 BIND        *bind= stmt->bind+ncol;
-                SQLINTEGER   binding_offset= 0, element_size= 0;
+                SQLINTEGER   binding_offset= 0;
+                SQLUINTEGER  element_size= 0;
                 SQLLEN       ind_or_len;
 
                 if (stmt->stmt_options.bind_type != SQL_BIND_BY_COLUMN &&
@@ -1194,12 +1162,7 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
                 else
                   ind_or_len= bind->cbValueMax;
 
-                param.SqlType= unireg_to_sql_datatype(stmt,
-                                                      field,
-                                                      0,
-                                                      &transfer_length,
-                                                      &precision,
-                                                      &display_size);
+                param.SqlType= get_sql_data_type(stmt, field, NULL);
                 param.CType = bind->fCType;
                 param.buffer= ((gptr)bind->rgbValue +
                                binding_offset +
@@ -1249,7 +1212,6 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
         }  /* END OF while(count < insert_count) */
 
         ext_query->str[--ext_query->length]= '\0';
-        MYODBCDbgInfo( "batch_insert: %s", ext_query->str );
         if ( exec_stmt_query(stmt, ext_query->str, ext_query->length) !=
              SQL_SUCCESS )
             return(SQL_ERROR);
@@ -1301,39 +1263,33 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
     SQLRETURN sqlRet= SQL_SUCCESS;
     MYSQL_RES *result= stmt->result;
 
-    MYODBCDbgEnter;
-
-    MYODBCDbgInfo( "irow: %d", irow );
-    MYODBCDbgInfo( "fOption: %s", MYODBCDbgPosTypeString( fOption ) );
-    MYODBCDbgInfo( "Lock: %d", fLock );
-
     CLEAR_STMT_ERROR(stmt);
 
     if ( !result )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1010,NULL,0) );
+        return set_error(stmt,MYERR_S1010,NULL,0);
 
     /* If irow > maximum rows in the resultset */
     if ( fOption != SQL_ADD && irow > result->row_count )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1107,NULL,0) );
+        return set_error(stmt,MYERR_S1107,NULL,0);
 
     /* Not a valid lock type ..*/
     if ( fLock != SQL_LOCK_NO_CHANGE )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1C00,NULL,0) );
+        return set_error(stmt,MYERR_S1C00,NULL,0);
 
     switch ( fOption )
     {
         case SQL_POSITION:
             {
                 if ( irow == 0 )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1109,NULL,0) );
+                    return set_error(stmt,MYERR_S1109,NULL,0);
 
                 if ( irow > stmt->rows_found_in_set )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1107,NULL,0) );
+                    return set_error(stmt,MYERR_S1107,NULL,0);
 
                 /* If Dynamic cursor, fetch the latest resultset */
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
                 {
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1000, alloc_error, 0) );
+                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
                 }
 
                 pthread_mutex_lock(&stmt->dbc->lock);
@@ -1341,12 +1297,18 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 sqlRet= SQL_SUCCESS;
                 stmt->cursor_row= (long)(stmt->current_row+irow);
                 mysql_data_seek(stmt->result,(my_ulonglong)stmt->cursor_row);
-                stmt->current_values= stmt->result->data_cursor->data;
+                stmt->current_values= mysql_fetch_row(stmt->result);
                 stmt->last_getdata_col= (uint)  ~0;; /* reset */
                 if ( stmt->fix_fields )
                     stmt->current_values= (*stmt->fix_fields)(stmt,stmt->current_values);
                 else
                     stmt->result_lengths= mysql_fetch_lengths(stmt->result);
+                /*
+                 The call to mysql_fetch_row() moved stmt->result's internal
+                 cursor, but we don't want that. We seek back to this row
+                 so the MYSQL_RES is in the state we expect.
+                */
+                mysql_data_seek(stmt->result,(my_ulonglong)stmt->cursor_row);
                 pthread_mutex_unlock(&stmt->dbc->lock);
                 break;
             }
@@ -1356,15 +1318,15 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 DYNAMIC_STRING dynQuery;
 
                 if ( irow > stmt->rows_found_in_set )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1107,NULL,0) );
+                    return set_error(stmt,MYERR_S1107,NULL,0);
 
                 /* IF dynamic cursor THEN rerun query to refresh resultset */
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1000, alloc_error, 0) );
+                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
 
                 /* start building our DELETE statement */
                 if ( init_dynamic_string(&dynQuery, "DELETE FROM ", 1024, 1024) )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1001,NULL,4001) );
+                    return set_error(stmt,MYERR_S1001,NULL,4001);
 
                 sqlRet = setpos_delete( stmt, irow, &dynQuery );
                 dynstr_free(&dynQuery);
@@ -1376,14 +1338,14 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 DYNAMIC_STRING dynQuery;
 
                 if ( irow > stmt->rows_found_in_set )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1107,NULL,0) );
+                    return set_error(stmt,MYERR_S1107,NULL,0);
 
                 /* IF dynamic cursor THEN rerun query to refresh resultset */
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1000, alloc_error, 0) );
+                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
 
                 if ( init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1001,NULL,4001) );
+                    return set_error(stmt,MYERR_S1001,NULL,4001);
 
                 sqlRet= setpos_update(stmt,irow,&dynQuery);
                 dynstr_free(&dynQuery);
@@ -1397,14 +1359,15 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 SQLUSMALLINT    nCol        = 0;
 
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1000, alloc_error, 0) );
+                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
                 result= stmt->result;
 
                 if ( !(table_name= find_used_table(stmt)) )
-                    MYODBCDbgReturnReturn( SQL_ERROR );
+                    return SQL_ERROR;
 
                 if ( init_dynamic_string(&dynQuery, "INSERT INTO ", 1024,1024) )
-                    MYODBCDbgReturnReturn( set_stmt_error(stmt,"S1001","Not enough memory",4001) );
+                    return set_stmt_error(stmt, "S1001", "Not enough memory",
+                                          4001);
 
                 dynstr_append_quoted_name(&dynQuery,table_name);
                 dynstr_append_mem(&dynQuery,"(",1);
@@ -1442,9 +1405,9 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
             }
 
         default:
-            MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1009,NULL,0) );
+            return set_error(stmt,MYERR_S1009,NULL,0);
     }
-    MYODBCDbgReturnReturn( sqlRet );
+    return sqlRet;
 }
 
 
@@ -1460,28 +1423,26 @@ SQLRETURN SQL_API SQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *szCursor,
 {
     STMT FAR *stmt= (STMT FAR*) hstmt;
 
-    MYODBCDbgEnter;
-
     CLEAR_STMT_ERROR(stmt);
 
     if ( !szCursor )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1009,NULL,0) );
+        return set_error(stmt,MYERR_S1009,NULL,0);
 
     if ( cbCursor == SQL_NTS )
         cbCursor= (SQLSMALLINT) strlen((char*) szCursor);
 
     if ( cbCursor < 0 )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1090,NULL,0) );
+        return set_error(stmt,MYERR_S1090,NULL,0);
 
     if ( (cbCursor == 0) ||
          (cbCursor > MYSQL_MAX_CURSOR_LEN) ||
          (myodbc_casecmp((char*) szCursor, "SQLCUR", 6) == 0)  ||
          (myodbc_casecmp((char*) szCursor, "SQL_CUR", 7) == 0) )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_34000,NULL,0) );
+        return set_error(stmt,MYERR_34000,NULL,0);
 
     x_free((gptr) stmt->cursor.name);
     stmt->cursor.name= dupp_str((char*) szCursor,cbCursor);
-    MYODBCDbgReturnReturn( SQL_SUCCESS );
+    return SQL_SUCCESS;
 }
 
 
@@ -1498,12 +1459,10 @@ SQLRETURN SQL_API SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR FAR *szCursor,
     SQLINTEGER  nLength;
     SQLSMALLINT nDummyLength;
 
-    MYODBCDbgEnter;
-
     CLEAR_STMT_ERROR(stmt);
 
     if ( cbCursorMax < 0 )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_S1090,NULL,0) );
+        return set_error(stmt,MYERR_S1090,NULL,0);
 
     if ( !pcbCursor )
         pcbCursor= &nDummyLength;
@@ -1521,9 +1480,9 @@ SQLRETURN SQL_API SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR FAR *szCursor,
     nLength= min(*pcbCursor , cbCursorMax);
 
     if ( nLength != *pcbCursor )
-        MYODBCDbgReturnReturn( set_error(stmt,MYERR_01004,NULL,0) );
+        return set_error(stmt,MYERR_01004,NULL,0);
 
-    MYODBCDbgReturnReturn( SQL_SUCCESS );
+    return SQL_SUCCESS;
 }
 
 
@@ -1548,12 +1507,10 @@ SQLRETURN SQL_API SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
 
 SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
 {
-    MYODBCDbgEnter;
-
     if ( Operation == SQL_ADD )
-        MYODBCDbgReturnReturn( my_SQLSetPos(Handle, 0, SQL_ADD, SQL_LOCK_NO_CHANGE) );
+        return my_SQLSetPos(Handle, 0, SQL_ADD, SQL_LOCK_NO_CHANGE);
 
-    MYODBCDbgReturnReturn( set_error(Handle,MYERR_S1C00,NULL,0) );
+    return set_error(Handle,MYERR_S1C00,NULL,0);
 }
 
 
@@ -1565,6 +1522,5 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
 
 SQLRETURN SQL_API SQLCloseCursor(SQLHSTMT Handle)
 {
-    MYODBCDbgEnter;
-    MYODBCDbgReturnReturn( my_SQLFreeStmt(Handle, SQL_CLOSE) );
+    return  my_SQLFreeStmt(Handle, SQL_CLOSE);
 }
