@@ -61,18 +61,18 @@ char allowed_chars[]=
       *pcbInfoValue= 0;                                                     \
       strmake((char*)rgbInfoValue,"",cbInfoValueMax);                       \
   }                                                                         \
-  MYODBCDbgReturnReturn(*pcbInfoValue >= cbInfoValueMax ?                      \
-		     SQL_SUCCESS_WITH_INFO:                                         \
-		     SQL_SUCCESS);                                                  \
+  return (*pcbInfoValue >= cbInfoValueMax ?                                 \
+          SQL_SUCCESS_WITH_INFO:                                            \
+          SQL_SUCCESS);                                                      \
 }
 
 #define MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,val,len) \
 { \
   *pcbInfoValue= len; \
   strmake((char*)rgbInfoValue,val,cbInfoValueMax); \
-  MYODBCDbgReturnReturn(len >= cbInfoValueMax ? \
-		     SQL_SUCCESS_WITH_INFO: \
-		     SQL_SUCCESS); \
+  return (len >= cbInfoValueMax ? \
+          SQL_SUCCESS_WITH_INFO: \
+          SQL_SUCCESS); \
 }
 
 static my_bool myodbc_ov2_inited= 0;
@@ -102,14 +102,6 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   SQLSMALLINT dummy;
   my_bool using_322;
 
-  MYODBCDbgEnter;
-
-  MYODBCDbgInfo("fInfoType: %s", MYODBCDbgInfoTypeString(fInfoType));
-  MYODBCDbgInfo("fInfoType: %d", fInfoType);
-  MYODBCDbgInfo("rgbInfoValue: 0x%lx", (long)rgbInfoValue);
-  MYODBCDbgInfo("cbInfoValueMax :%d", cbInfoValueMax);
-  MYODBCDbgInfo("pcbInfoValue: 0x%lx", (long)pcbInfoValue);
-
   if (cbInfoValueMax)
     cbInfoValueMax--;
 
@@ -182,7 +174,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_OUTER_JOINS:
   case SQL_ORDER_BY_COLUMNS_IN_SELECT:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"Y",1);
-    break;
+    /* break; will never reach */
 
   case SQL_CATALOG_NAME_SEPARATOR:
     if (dbc->flag & FLAG_NO_CATALOG)
@@ -193,7 +185,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     {
       MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,".",1);
     }
-    break;
+    /* break; will never reach */
 
   case SQL_CATALOG_TERM:
     if (dbc->flag & FLAG_NO_CATALOG)
@@ -204,7 +196,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     {
       MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"database",8);
     }
-    break;
+    /* break; will never reach */
 
   case SQL_CATALOG_USAGE:
     *((SQLUINTEGER *)rgbInfoValue)= (SQL_CU_DML_STATEMENTS |
@@ -222,7 +214,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
       collation sequences / user.
     */
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"",0);
-    break;
+    /* break; will never reach */
 
   case SQL_CONCAT_NULL_BEHAVIOR:
     *((SQLUSMALLINT *)rgbInfoValue)= SQL_CB_NULL;
@@ -358,7 +350,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_DATA_SOURCE_NAME:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,dbc->dsn);
-    break;
+    /* break; will never reach */
 
   case SQL_ACCESSIBLE_PROCEDURES:
   case SQL_DATA_SOURCE_READ_ONLY:
@@ -368,11 +360,11 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_PROCEDURES:
   case SQL_ROW_UPDATES:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"N",1);
-    break;
+    /* break; will never reach */
 
   case SQL_DATABASE_NAME:
     MYINFO_SET_STR(rgbInfoValue, cbInfoValueMax, pcbInfoValue, dbc->database);
-    break;
+    /* break; will never reach */
 
   case SQL_DATETIME_LITERALS:
     *((SQLUINTEGER *)rgbInfoValue)= SQL_DL_SQL92_DATE |
@@ -383,12 +375,12 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_DBMS_NAME:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"MySQL");
-    break;
+    /* break; will never reach */
 
   case SQL_DBMS_VER:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,
                    dbc->mysql.server_version);
-    break;
+    /* break; will never reach */
 
   case SQL_DDL_INDEX:
     *((SQLUINTEGER *)rgbInfoValue)= SQL_DI_CREATE_INDEX | SQL_DI_DROP_INDEX;
@@ -402,15 +394,15 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_DRIVER_NAME:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,DRIVER_DLL_NAME);
-    break;
+    /* break; will never reach */
 
   case SQL_DRIVER_ODBC_VER:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,SQL_SPEC_STRING);
-    break;
+    /* break; will never reach */
 
   case SQL_DRIVER_VER:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,DRIVER_VERSION);
-    break;
+    /* break; will never reach */
 
   case SQL_DROP_TABLE:
     *((SQLUINTEGER *)rgbInfoValue)= (SQL_DT_DROP_TABLE |
@@ -531,7 +523,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     using_322= is_prefix(mysql_get_server_info(&dbc->mysql),"3.22");
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,
                    !using_322 ? "`" : " ");
-    break;
+    /* break; will never reach */
 
   case SQL_INDEX_KEYWORDS:
     *((SQLUINTEGER *)rgbInfoValue)= SQL_IK_NONE;
@@ -548,7 +540,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_KEYWORDS:
     /* @todo Need to return all keywords..*/
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,mysql_keywords);
-    break;
+    /* break; will never reach */
 
   case SQL_MAX_CATALOG_NAME_LEN:
   case SQL_MAX_COLUMN_NAME_LEN:
@@ -652,7 +644,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_PROCEDURE_TERM:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"",0);
-    break;
+    /* break; will never reach */
 
   case SQL_POS_OPERATIONS:
     *((SQLINTEGER *)rgbInfoValue)= (SQL_POS_POSITION |
@@ -680,7 +672,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_SCHEMA_TERM:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"",0);
-    break;
+    /* break; will never reach */
 
   case SQL_SCROLL_OPTIONS:
     *((SQLUINTEGER *)rgbInfoValue)= (SQL_SO_FORWARD_ONLY | SQL_SO_STATIC);
@@ -700,16 +692,16 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_SEARCH_PATTERN_ESCAPE:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"\\",1);
-    break;
+    /* break; will never reach */
 
   case SQL_SERVER_NAME:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,
                    dbc->mysql.host_info);
-    break;
+    /* break; will never reach */
 
   case SQL_SPECIAL_CHARACTERS:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue, allowed_chars);
-    break;
+    /* break; will never reach */
 
   case SQL_SQL_CONFORMANCE:
     *((SQLUINTEGER *)rgbInfoValue)= SQL_SC_SQL92_INTERMEDIATE;
@@ -822,7 +814,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_TABLE_TERM:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"table",5);
-    break;
+    /* break; will never reach */
 
   case SQL_TXN_CAPABLE:
     if (trans_supported(dbc) && !(dbc->flag & FLAG_NO_TRANSACTIONS))
@@ -845,11 +837,11 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_USER_NAME:
     MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,dbc->user);
-    break;
+    /* break; will never reach */
 
   case SQL_XOPEN_CLI_YEAR:
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"1992",4);
-    break;
+    /* break; will never reach */
 
   case SQL_FETCH_DIRECTION:
     *((SQLINTEGER *)rgbInfoValue)= (SQL_FD_FETCH_NEXT | SQL_FD_FETCH_FIRST |
@@ -872,10 +864,10 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     {
       char buff[80];
       sprintf(buff, "Unsupported option: %d to SQLGetInfo", fInfoType);
-      MYODBCDbgReturnReturn(set_conn_error(hdbc, MYERR_S1C00, buff, 4000));
+      return set_conn_error(hdbc, MYERR_S1C00, buff, 4000);
     }
   }
-  MYODBCDbgReturnReturn( SQL_SUCCESS );
+  return SQL_SUCCESS;
 }
 
 
@@ -939,7 +931,7 @@ char sql_searchable[6], sql_unsearchable[6], sql_nullable[6], sql_no_nulls[6],
      sql_float[6], sql_real[6], sql_double[6], sql_char[6], sql_varchar[6],
      sql_longvarchar[6], sql_timestamp[6], sql_decimal[6], sql_numeric[6],
      sql_varbinary[6], sql_time[6], sql_date[6], sql_binary[6],
-     sql_longvarbinary[6];
+     sql_longvarbinary[6], sql_datetime[6];
 
 char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][19]=
 {
@@ -1022,17 +1014,17 @@ char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][19]=
   {"double auto_increment",sql_double,"15",NULL,NULL,NULL,sql_no_nulls,"0",sql_searchable,"0","0","1","double auto_increment","0","4",sql_double,NULL,"10",NULL},
 
   /* SQL_TYPE_DATE= 91 */
-  {"date",sql_date,"10","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","date",NULL,NULL,sql_date,NULL,NULL,NULL},
+  {"date",sql_date,"10","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","date",NULL,NULL,sql_datetime,sql_date,NULL,NULL},
 
   /* SQL_TYPE_TIME= 92 */
-  {"time",sql_time,"6","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","time",NULL,NULL,sql_time,NULL,NULL,NULL},
+  {"time",sql_time,"6","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","time",NULL,NULL,sql_datetime,sql_time,NULL,NULL},
 
   /* YEAR - SQL_SMALLINT */
   {"year",sql_smallint,"4",NULL,NULL,NULL,sql_nullable,"0",sql_searchable,"0","0","0","year",NULL,NULL,sql_smallint,NULL,"10",NULL},
 
   /* SQL_TYPE_TIMESTAMP= 93 */
-  {"datetime",sql_timestamp,"21","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","datetime","0","0",sql_timestamp,NULL,NULL,NULL},
-  {"timestamp",sql_timestamp,"14","'","'",NULL,sql_no_nulls,"0",sql_searchable,"0","0","0","timestamp","0","0",sql_timestamp,NULL,NULL,NULL},
+  {"datetime",sql_timestamp,"21","'","'",NULL,sql_nullable,"0",sql_searchable,"0","0","0","datetime","0","0",sql_datetime,sql_timestamp,NULL,NULL},
+  {"timestamp",sql_timestamp,"14","'","'",NULL,sql_no_nulls,"0",sql_searchable,"0","0","0","timestamp","0","0",sql_datetime,sql_timestamp,NULL,NULL},
 
   /* SQL_VARCHAR= 12 */
 #if TO_BE_DELETED /* NO NEED OF THIS, causes problems for DM */
@@ -1060,13 +1052,11 @@ SQLRETURN SQL_API SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   STMT *stmt= (STMT *)hstmt;
   uint i;
 
-  MYODBCDbgEnter;
-  MYODBCDbgInfo("fSqlType: %d", fSqlType);
-
   my_SQLFreeStmt(hstmt, MYSQL_RESET);
 
   /* Set up result Data dictionary. */
   stmt->result= (MYSQL_RES *)my_malloc(sizeof(MYSQL_RES), MYF(MY_ZEROFILL));
+  stmt->fake_result= 1;
   stmt->result_array= (char **)my_malloc(sizeof(SQL_GET_TYPE_INFO_values),
                                          MYF(MY_FAE | MY_ZEROFILL));
 
@@ -1081,7 +1071,8 @@ SQLRETURN SQL_API SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   {
     for (i= 0 ; i < MYSQL_DATA_TYPES ; i++)
     {
-      if (atoi(SQL_GET_TYPE_INFO_values[i][1]) == fSqlType)
+      if (atoi(SQL_GET_TYPE_INFO_values[i][1]) == fSqlType ||
+          atoi(SQL_GET_TYPE_INFO_values[i][15]) == fSqlType)
       {
         memcpy((gptr)&stmt->result_array[stmt->result->row_count++ *
                                          SQL_GET_TYPE_INFO_FIELDS],
@@ -1092,7 +1083,7 @@ SQLRETURN SQL_API SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   }
   mysql_link_fields(stmt, SQL_GET_TYPE_INFO_fields, SQL_GET_TYPE_INFO_FIELDS);
 
-  MYODBCDbgReturnReturn(SQL_SUCCESS);
+  return SQL_SUCCESS;
 }
 
 
@@ -1103,63 +1094,34 @@ SQLRETURN SQL_API SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
 */
 void init_getfunctions(void)
 {
-#if MYSQL_VERSION_ID >= 40100
-  int2str(SQL_SEARCHABLE,sql_searchable,-10,0);
-  int2str(SQL_UNSEARCHABLE,sql_unsearchable,-10,0);
-  int2str(SQL_NULLABLE,sql_nullable,-10,0);
-  int2str(SQL_NO_NULLS,sql_no_nulls,-10,0);
-  int2str(SQL_BIT,sql_bit,-10,0);
-  int2str(SQL_TINYINT,sql_tinyint,-10,0);
-  int2str(SQL_SMALLINT,sql_smallint,-10,0);
-  int2str(SQL_INTEGER,sql_integer,-10,0);
-  int2str(SQL_BIGINT,sql_bigint,-10,0);
-  int2str(SQL_DECIMAL,sql_decimal,-10,0);
-  int2str(SQL_NUMERIC,sql_numeric,-10,0);
-  int2str(SQL_REAL,sql_real,-10,0);
-  int2str(SQL_FLOAT,sql_float,-10,0);
-  int2str(SQL_DOUBLE,sql_double,-10,0);
-  int2str(SQL_CHAR,sql_char,-10,0);
-  int2str(SQL_VARCHAR,sql_varchar,-10,0);
-  int2str(SQL_LONGVARCHAR,sql_longvarchar,-10,0);
-  int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10,0);
-  int2str(SQL_VARBINARY,sql_varbinary,-10,0);
-  int2str(SQL_BINARY,sql_binary,-10,0);
-  int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
-  int2str(SQL_TYPE_DATE,sql_date,-10,0);
-  int2str(SQL_TYPE_TIME,sql_time,-10,0);
+  my_int2str(SQL_SEARCHABLE,sql_searchable,-10,0);
+  my_int2str(SQL_UNSEARCHABLE,sql_unsearchable,-10,0);
+  my_int2str(SQL_NULLABLE,sql_nullable,-10,0);
+  my_int2str(SQL_NO_NULLS,sql_no_nulls,-10,0);
+  my_int2str(SQL_BIT,sql_bit,-10,0);
+  my_int2str(SQL_TINYINT,sql_tinyint,-10,0);
+  my_int2str(SQL_SMALLINT,sql_smallint,-10,0);
+  my_int2str(SQL_INTEGER,sql_integer,-10,0);
+  my_int2str(SQL_BIGINT,sql_bigint,-10,0);
+  my_int2str(SQL_DECIMAL,sql_decimal,-10,0);
+  my_int2str(SQL_NUMERIC,sql_numeric,-10,0);
+  my_int2str(SQL_REAL,sql_real,-10,0);
+  my_int2str(SQL_FLOAT,sql_float,-10,0);
+  my_int2str(SQL_DOUBLE,sql_double,-10,0);
+  my_int2str(SQL_CHAR,sql_char,-10,0);
+  my_int2str(SQL_VARCHAR,sql_varchar,-10,0);
+  my_int2str(SQL_LONGVARCHAR,sql_longvarchar,-10,0);
+  my_int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10,0);
+  my_int2str(SQL_VARBINARY,sql_varbinary,-10,0);
+  my_int2str(SQL_BINARY,sql_binary,-10,0);
+  my_int2str(SQL_DATETIME,sql_datetime,-10,0);
+  my_int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
+  my_int2str(SQL_TYPE_DATE,sql_date,-10,0);
+  my_int2str(SQL_TYPE_TIME,sql_time,-10,0);
 # if (ODBCVER < 0x0300)
   myodbc_sqlstate2_init();
   myodbc_ov2_inited= 1;
 # endif
-#else
-  int2str(SQL_SEARCHABLE,sql_searchable,-10);
-  int2str(SQL_UNSEARCHABLE,sql_unsearchable,-10);
-  int2str(SQL_NULLABLE,sql_nullable,-10);
-  int2str(SQL_NO_NULLS,sql_no_nulls,-10);
-  int2str(SQL_BIT,sql_bit,-10);
-  int2str(SQL_TINYINT,sql_tinyint,-10);
-  int2str(SQL_SMALLINT,sql_smallint,-10);
-  int2str(SQL_INTEGER,sql_integer,-10);
-  int2str(SQL_BIGINT,sql_bigint,-10);
-  int2str(SQL_DECIMAL,sql_decimal,-10);
-  int2str(SQL_NUMERIC,sql_numeric,-10);
-  int2str(SQL_REAL,sql_real,-10);
-  int2str(SQL_FLOAT,sql_float,-10);
-  int2str(SQL_DOUBLE,sql_double,-10);
-  int2str(SQL_CHAR,sql_char,-10);
-  int2str(SQL_VARCHAR,sql_varchar,-10);
-  int2str(SQL_LONGVARCHAR,sql_longvarchar,-10);
-  int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10);
-  int2str(SQL_VARBINARY,sql_varbinary,-10);
-  int2str(SQL_BINARY,sql_binary,-10);
-  int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10);
-  int2str(SQL_TYPE_DATE,sql_date,-10);
-  int2str(SQL_TYPE_TIME,sql_time,-10);
-# if (ODBCVER < 0x0300)
-  myodbc_sqlstate2_init();
-  myodbc_ov2_inited= 1;
-# endif
-#endif
 }
 
 /**
@@ -1169,15 +1131,9 @@ void myodbc_ov_init(SQLINTEGER odbc_version)
 {
   if (odbc_version == SQL_OV_ODBC2)
   {
-#if MYSQL_VERSION_ID >= 40100
-    int2str(SQL_TIMESTAMP,sql_timestamp,-10,0);
-    int2str(SQL_DATE,sql_date,-10,0);
-    int2str(SQL_TIME,sql_time,-10,0);
-#else
-    int2str(SQL_TIMESTAMP,sql_timestamp,-10);
-    int2str(SQL_DATE,sql_date,-10);
-    int2str(SQL_TIME,sql_time,-10);
-#endif
+    my_int2str(SQL_TIMESTAMP,sql_timestamp,-10,0);
+    my_int2str(SQL_DATE,sql_date,-10,0);
+    my_int2str(SQL_TIME,sql_time,-10,0);
     myodbc_sqlstate2_init();
     myodbc_ov2_inited= 1;
   }
@@ -1187,15 +1143,9 @@ void myodbc_ov_init(SQLINTEGER odbc_version)
       return;
     myodbc_ov2_inited= 0;
 
-#if MYSQL_VERSION_ID >= 40100
-    int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
-    int2str(SQL_TYPE_DATE,sql_date,-10,0);
-    int2str(SQL_TYPE_TIME,sql_time,-10,0);
-#else
-    int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10);
-    int2str(SQL_TYPE_DATE,sql_date,-10);
-    int2str(SQL_TYPE_TIME,sql_time,-10);
-#endif
+    my_int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
+    my_int2str(SQL_TYPE_DATE,sql_date,-10,0);
+    my_int2str(SQL_TYPE_TIME,sql_time,-10,0);
     myodbc_sqlstate3_init();
   }
 }
@@ -1204,7 +1154,7 @@ void myodbc_ov_init(SQLINTEGER odbc_version)
 /**
   List of functions supported in the driver.
 */
-SQLINTEGER myodbc3_functions[]=
+SQLUSMALLINT myodbc3_functions[]=
 {
     SQL_API_SQLALLOCCONNECT,
     SQL_API_SQLALLOCENV,
@@ -1303,10 +1253,7 @@ SQLRETURN SQL_API SQLGetFunctions(SQLHDBC hdbc __attribute__((unused)),
                                   SQLUSMALLINT fFunction,
                                   SQLUSMALLINT *pfExists)
 {
-  SQLINTEGER index, myodbc_func_size;
-
-  MYODBCDbgEnter;
-  MYODBCDbgInfo("fFunction: %d",fFunction);
+  SQLUSMALLINT index, myodbc_func_size;
 
   myodbc_func_size= sizeof(myodbc3_functions) / sizeof(myodbc3_functions[0]);
 
@@ -1314,10 +1261,10 @@ SQLRETURN SQL_API SQLGetFunctions(SQLHDBC hdbc __attribute__((unused)),
   {
     for (index= 0; index < myodbc_func_size; index++)
     {
-      int id= myodbc3_functions[index];
+      SQLUSMALLINT id= myodbc3_functions[index];
       pfExists[id >> 4]|= (1 << (id & 0x000F));
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
   }
 
   if (fFunction == SQL_API_ALL_FUNCTIONS)
@@ -1327,7 +1274,7 @@ SQLRETURN SQL_API SQLGetFunctions(SQLHDBC hdbc __attribute__((unused)),
       if (myodbc3_functions[index] < 100)
         pfExists[myodbc3_functions[index]]= SQL_TRUE;
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
   }
 
   *pfExists= SQL_FALSE;
@@ -1340,5 +1287,5 @@ SQLRETURN SQL_API SQLGetFunctions(SQLHDBC hdbc __attribute__((unused)),
     }
   }
 
-  MYODBCDbgReturnReturn(SQL_SUCCESS);
+  return SQL_SUCCESS;
 }
