@@ -55,21 +55,19 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
                                   SQLINTEGER   Attribute,
                                   SQLPOINTER   ValuePtr)
 {
-    MYODBCDbgEnter;
-
     switch (Attribute)
     {
         case SQL_ATTR_ASYNC_ENABLE:
             if (ValuePtr == (SQLPOINTER) SQL_ASYNC_ENABLE_ON)
-                MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                 "Doesn't support asynchronous, changed to default",0));
+                return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                        "Doesn't support asynchronous, changed to default",0);
             break;
 
         case SQL_ATTR_CURSOR_SENSITIVITY:
             if (ValuePtr != (SQLPOINTER) SQL_UNSPECIFIED)
             {
-                MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                 "Option value changed to default cursor sensitivity(unspecified)",0));
+                return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                        "Option value changed to default cursor sensitivity(unspecified)",0);
             }
             break;
 
@@ -78,8 +76,8 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
             {
                 options->cursor_type= SQL_CURSOR_FORWARD_ONLY;
                 if (ValuePtr != (SQLPOINTER)SQL_CURSOR_FORWARD_ONLY)
-                    MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                     "Forcing the use of forward-only cursor)",0));
+                    return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                            "Forcing the use of forward-only cursor)",0);
             }
             else if (((STMT FAR*)Handle)->dbc->flag & FLAG_DYNAMIC_CURSOR)
             {
@@ -89,8 +87,8 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
                 else
                 {
                     options->cursor_type= SQL_CURSOR_STATIC;
-                    MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                     "Option value changed to default static cursor",0));
+                    return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                            "Option value changed to default static cursor",0);
                 }
             }
             else
@@ -102,28 +100,27 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
                 else
                 {
                     options->cursor_type= SQL_CURSOR_STATIC;
-                    MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                     "Option value changed to default static cursor",0));
+                    return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                            "Option value changed to default static cursor",0);
                 }
             }
             break;
 
         case SQL_ATTR_MAX_LENGTH:
-            options->max_length= (SQLUINTEGER) ValuePtr;
+            options->max_length= (SQLULEN) ValuePtr;
             break;
 
         case SQL_ATTR_MAX_ROWS:
-            options->max_rows= (SQLUINTEGER) ValuePtr;
+            options->max_rows= (SQLULEN) ValuePtr;
             break;
 
         case SQL_ATTR_METADATA_ID:
             if (ValuePtr == (SQLPOINTER) SQL_TRUE)
-                MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                 "Doesn't support SQL_ATTR_METADATA_ID to true, changed to default",0));
+                return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                        "Doesn't support SQL_ATTR_METADATA_ID to true, changed to default",0);
             break;
 
         case SQL_ATTR_RETRIEVE_DATA:
-            MYODBCDbgInfo( "%s", "SQL_ATTR_RETRIEVE_DATA value is ignored" );
             break;
 
         case SQL_ATTR_ROW_BIND_TYPE:
@@ -132,12 +129,12 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
 
         case SQL_ATTR_SIMULATE_CURSOR:
             if (ValuePtr != (SQLPOINTER) SQL_SC_TRY_UNIQUE)
-                MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_01S02,
-                                                 "Option value changed to default cursor simulation",0));
+                return set_handle_error(HandleType,Handle,MYERR_01S02,
+                                        "Option value changed to default cursor simulation",0);
             break;
 
         case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-            options->bind_offset= (SQLINTEGER *)ValuePtr;
+            options->bind_offset= (SQLLEN *)ValuePtr;
             break;
 
         case 1226:/* MS SQL Server Extension */
@@ -150,19 +147,17 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
         case SQL_ATTR_CONCURRENCY:
         case SQL_ATTR_NOSCAN:
         case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-            MYODBCDbgInfo( "Attribute, %d is ignored", (int)Attribute );
             break;
 
         case SQL_ATTR_FETCH_BOOKMARK_PTR:
         case SQL_ATTR_USE_BOOKMARKS:
-            MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0));
+            return set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0);
             break;
 
         default:
-            MYODBCDbgInfo( "Attribute, %d is ignored", (int)Attribute );
             break;
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -179,11 +174,8 @@ get_constmt_attr(SQLSMALLINT  HandleType,
                  SQLPOINTER   ValuePtr,
                  SQLINTEGER   *StringLengthPtr __attribute__((unused)))
 {
-    MYODBCDbgEnter;
-
     switch (Attribute)
     {
-        
         case SQL_ATTR_ASYNC_ENABLE:
             *((SQLUINTEGER *) ValuePtr)= SQL_ASYNC_ENABLE_OFF;
             break;
@@ -197,11 +189,11 @@ get_constmt_attr(SQLSMALLINT  HandleType,
             break;
 
         case SQL_ATTR_MAX_LENGTH:
-            *((SQLUINTEGER *) ValuePtr)= options->max_length;
+            *((SQLULEN *) ValuePtr)= options->max_length;
             break;
 
         case SQL_ATTR_MAX_ROWS:
-            *((SQLUINTEGER *) ValuePtr)= options->max_rows;
+            *((SQLULEN *) ValuePtr)= options->max_rows;
             break;
 
         case SQL_ATTR_METADATA_ID:
@@ -241,18 +233,15 @@ get_constmt_attr(SQLSMALLINT  HandleType,
             break;
 
         case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-            *((SQLINTEGER *) ValuePtr)= options->bind_offset ?
-                                        *(options->bind_offset):
-                                        0;
+            *((SQLLEN *) ValuePtr)= options->bind_offset;
             break;
 
         case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-            MYODBCDbgInfo( "Attribute, %d is ignored", (int)Attribute );
-            MYODBCDbgReturnReturn(SQL_SUCCESS_WITH_INFO);
+            return SQL_SUCCESS_WITH_INFO;
 
         case SQL_ATTR_FETCH_BOOKMARK_PTR:
         case SQL_ATTR_USE_BOOKMARKS:
-            MYODBCDbgReturnReturn(set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0));
+            return set_handle_error(HandleType,Handle,MYERR_S1C00,NULL,0);
 
         case 1226:/* MS SQL Server Extension */
         case 1227:
@@ -260,9 +249,9 @@ get_constmt_attr(SQLSMALLINT  HandleType,
             break;
 
         default:
-          MYODBCDbgError( "Invalid attribute/option identifier:%d", (int)Attribute );
+            break;
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -279,54 +268,45 @@ set_con_attr(SQLHDBC    hdbc,
 {
     DBC FAR *dbc= (DBC FAR*) hdbc;
 
-    MYODBCDbgEnter;
-
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-    MYODBCDbgInfo( "ValuePtr: 0x%lx", (ulong)ValuePtr );
-    MYODBCDbgInfo( "StrLenPtr: %d", (int)StringLengthPtr );
-
     switch (Attribute)
     {
-        
         case SQL_ATTR_ACCESS_MODE:
-            MYODBCDbgInfo( "SQL_ATTR_ACCESS_MODE %ld ignored", (SQLUINTEGER)ValuePtr );
             break;
 
         case SQL_ATTR_AUTOCOMMIT:
             if (ValuePtr != (SQLPOINTER) SQL_AUTOCOMMIT_ON)
             {
-                if (!dbc->server) /* no connection yet */
+                if (!is_connected(dbc))
                 {
                     dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
-                    MYODBCDbgReturnReturn(SQL_SUCCESS);
+                    return SQL_SUCCESS;
                 }
                 if (!(trans_supported(dbc)) || (dbc->flag & FLAG_NO_TRANSACTIONS))
-                    MYODBCDbgReturnReturn(set_conn_error(dbc,MYERR_S1C00,
-                                                   "Transactions are not enabled", 4000));
+                    return set_conn_error(dbc,MYERR_S1C00,
+                                          "Transactions are not enabled", 4000);
 
                 if (autocommit_on(dbc))
-                    MYODBCDbgReturnReturn(odbc_stmt(dbc,"SET AUTOCOMMIT=0"));
+                    return odbc_stmt(dbc,"SET AUTOCOMMIT=0");
             }
-            else if (!dbc->server) /* no connection yet */
+            else if (!is_connected(dbc))
             {
                 dbc->commit_flag= CHECK_AUTOCOMMIT_ON;
-                MYODBCDbgReturnReturn(SQL_SUCCESS);
+                return SQL_SUCCESS;
             }
             else if (trans_supported(dbc) && !(autocommit_on(dbc)))
-                MYODBCDbgReturnReturn(odbc_stmt(dbc,"SET AUTOCOMMIT=1"));
+                return odbc_stmt(dbc,"SET AUTOCOMMIT=1");
             break;
 
         case SQL_ATTR_LOGIN_TIMEOUT:
             {
                 /* we can't change timeout values in post connect state */
-               if (dbc->server) {
-                  MYODBCDbgReturnReturn(
-                       set_conn_error(dbc, MYERR_S1011, NULL, 0));
-               } 
+               if (is_connected(dbc)) {
+                  return set_conn_error(dbc, MYERR_S1011, NULL, 0);
+               }
                else
                {
                   dbc->login_timeout= (SQLUINTEGER) ValuePtr;
-                  MYODBCDbgReturnReturn(SQL_SUCCESS);
+                  return SQL_SUCCESS;
                }
             }
             break;
@@ -337,7 +317,7 @@ set_con_attr(SQLHDBC    hdbc,
                 We don't do anything with this, but we pretend that we did
                 to be consistent with Microsoft SQL Server.
               */
-              MYODBCDbgReturnReturn(SQL_SUCCESS);
+              return SQL_SUCCESS;
             }
             break;
 
@@ -352,16 +332,16 @@ set_con_attr(SQLHDBC    hdbc,
                 char ldb[NAME_LEN+1], *db;
 
                 if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
-                    MYODBCDbgReturnReturn(set_conn_error(hdbc,MYERR_S1009,NULL, 0));
+                    return set_conn_error(hdbc,MYERR_S1009,NULL, 0);
 
                 pthread_mutex_lock(&dbc->lock);
-                if ( dbc->mysql.net.vio )
+                if (is_connected(dbc))
                 {
                     if (mysql_select_db(&dbc->mysql,(char*) db))
                     {
                         set_conn_error(dbc,MYERR_S1000,mysql_error(&dbc->mysql),mysql_errno(&dbc->mysql));
                         pthread_mutex_unlock(&dbc->lock);
-                        MYODBCDbgReturnReturn(SQL_ERROR);
+                        return SQL_ERROR;
                     }
                 }
                 my_free((gptr) dbc->database,MYF(0));
@@ -374,8 +354,8 @@ set_con_attr(SQLHDBC    hdbc,
         case SQL_ATTR_ODBC_CURSORS:
             if ((dbc->flag & FLAG_FORWARD_CURSOR) &&
                 ValuePtr != (SQLPOINTER) SQL_CUR_USE_ODBC)
-                MYODBCDbgReturnReturn(set_conn_error(hdbc,MYERR_01S02,
-                                               "Forcing the Driver Manager to use ODBC cursor library",0));
+                return set_conn_error(hdbc,MYERR_01S02,
+                                      "Forcing the Driver Manager to use ODBC cursor library",0);
             break;
 
         case SQL_OPT_TRACE:
@@ -386,18 +366,17 @@ set_con_attr(SQLHDBC    hdbc,
             {
                 char buff[100];
                 sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",(int) Attribute);
-                MYODBCDbgReturnReturn(set_conn_error(hdbc,MYERR_01S02,buff,0));
+                return set_conn_error(hdbc,MYERR_01S02,buff,0);
             }
 
         case SQL_ATTR_PACKET_SIZE:
-            MYODBCDbgInfo( "SQL_ATTR_PACKET_SIZE %ld ignored", (SQLUINTEGER)ValuePtr );
             break;
 
         case SQL_ATTR_TXN_ISOLATION:
-            if (!dbc->server)  /* no connection yet */
+            if (!is_connected(dbc))  /* no connection yet */
             {
                 dbc->txn_isolation= (SQLINTEGER)ValuePtr;
-                MYODBCDbgReturnReturn(SQL_SUCCESS);
+                return SQL_SUCCESS;
             }
             if (trans_supported(dbc))
             {
@@ -416,8 +395,6 @@ set_con_attr(SQLHDBC    hdbc,
                 if (odbc_stmt(dbc,buff) == SQL_SUCCESS)
                     dbc->txn_isolation= (SQLINTEGER)ValuePtr;
             }
-            else
-                MYODBCDbgInfo( "SQL_ATTR_TXN_ISOLATION %ld ignored", (SQLINTEGER)ValuePtr );
             break;
 
             /*
@@ -426,9 +403,10 @@ set_con_attr(SQLHDBC    hdbc,
               works fine...lets support it..nothing to loose..
             */
         default:
-            MYODBCDbgReturnReturn(set_constmt_attr(2,dbc,&dbc->stmt_options,Attribute,ValuePtr));
+            return set_constmt_attr(2, dbc, &dbc->stmt_options, Attribute,
+                                    ValuePtr);
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -448,13 +426,6 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
     SQLINTEGER len;
     SQLPOINTER vparam= 0;
 
-    MYODBCDbgEnter;
-
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-    MYODBCDbgInfo( "ValuePtr: 0x%lx", (ulong)ValuePtr );
-    MYODBCDbgInfo( "BufLen: %d", (int)BufferLength );
-    MYODBCDbgInfo( "StrLenPtr: 0x%lx", (ulong)StringLengthPtr );
-
     if (!ValuePtr)
         ValuePtr= vparam;
 
@@ -463,7 +434,6 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
 
     switch (Attribute)
     {
-        
         case SQL_ATTR_ACCESS_MODE:
             *((SQLUINTEGER *) ValuePtr)= SQL_MODE_READ_WRITE;
             break;
@@ -548,7 +518,7 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
                 Unless we're not connected yet, then we just assume it will
                 be REPEATABLE READ, which is the server default.
               */
-              if (!dbc->server)
+              if (!is_connected(dbc))
               {
                 *((SQLINTEGER *) ValuePtr)= SQL_TRANSACTION_REPEATABLE_READ;
                 break;
@@ -556,10 +526,9 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
 
               if (odbc_stmt(dbc, "SELECT @@tx_isolation"))
               {
-                MYODBCDbgReturnReturn(set_handle_error(SQL_HANDLE_DBC,hdbc,
-                                                       MYERR_S1000,
-                                                       "Failed to get "
-                                                       "isolation level", 0));
+                return set_handle_error(SQL_HANDLE_DBC,hdbc,
+                                        MYERR_S1000,
+                                        "Failed to get isolation level", 0);
               }
               else
               {
@@ -599,7 +568,7 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
                                      Attribute,ValuePtr,
                                      StringLengthPtr);
     }
-    MYODBCDbgReturnReturn(result);
+    return result;
 }
 
 
@@ -618,15 +587,8 @@ set_stmt_attr(SQLHSTMT   hstmt,
     SQLRETURN result= SQL_SUCCESS;
     STMT_OPTIONS *options= &stmt->stmt_options;
 
-    MYODBCDbgEnter;
-
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-    MYODBCDbgInfo( "ValuePtr: 0x%lx", (ulong)ValuePtr );
-    MYODBCDbgInfo( "StrLenPtr: %d", (int)StringLengthPtr );
-
     switch (Attribute)
     {
-        
         case SQL_ATTR_CURSOR_SCROLLABLE:
             if (ValuePtr == (SQLPOINTER)SQL_NONSCROLLABLE &&
                 options->cursor_type != SQL_CURSOR_FORWARD_ONLY)
@@ -639,8 +601,8 @@ set_stmt_attr(SQLHSTMT   hstmt,
 
         case SQL_ATTR_AUTO_IPD:
             if (ValuePtr != (SQLPOINTER)SQL_FALSE)
-                MYODBCDbgReturnReturn(set_error(hstmt,MYERR_01S02,
-                                          "Option value changed to default auto ipd",0));
+                return set_error(hstmt,MYERR_01S02,
+                                 "Option value changed to default auto ipd",0);
             break;
 
         case SQL_ATTR_PARAM_STATUS_PTR: /* need to support this ....*/
@@ -653,9 +615,9 @@ set_stmt_attr(SQLHSTMT   hstmt,
 
         case SQL_ATTR_PARAMSET_SIZE:
             if (ValuePtr != (SQLPOINTER)1)
-                MYODBCDbgReturnReturn(set_error(hstmt,MYERR_01S02,
-                                          "Option value changed to default parameter size",
-                                          0));
+                return set_error(hstmt,MYERR_01S02,
+                                 "Option value changed to default parameter size",
+                                 0);
             break;
 
         case SQL_ATTR_ROW_ARRAY_SIZE:
@@ -664,8 +626,8 @@ set_stmt_attr(SQLHSTMT   hstmt,
             break;
 
         case SQL_ATTR_ROW_NUMBER:
-            MYODBCDbgReturnReturn(set_error(hstmt,MYERR_S1000,
-                                      "Trying to set read-only attribute",0));
+            return set_error(hstmt,MYERR_S1000,
+                             "Trying to set read-only attribute",0);
             break;
 
         case SQL_ATTR_ROW_OPERATION_PTR:
@@ -677,7 +639,7 @@ set_stmt_attr(SQLHSTMT   hstmt,
             break;
 
         case SQL_ATTR_ROWS_FETCHED_PTR:
-            options->rowsFetchedPtr= (SQLUINTEGER *)ValuePtr;
+            options->rowsFetchedPtr= (SQLULEN *)ValuePtr;
             break;
 
         case SQL_ATTR_SIMULATE_CURSOR:
@@ -693,7 +655,7 @@ set_stmt_attr(SQLHSTMT   hstmt,
             result= set_constmt_attr(3,hstmt,options,
                                      Attribute,ValuePtr);
     }
-    MYODBCDbgReturnReturn(result);
+    return result;
 }
 
 
@@ -714,12 +676,6 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
     SQLPOINTER vparam;
     SQLINTEGER len;
 
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-    MYODBCDbgInfo( "ValuePtr: 0x%lx", (ulong)ValuePtr );
-    MYODBCDbgInfo( "BufLen: %d", (int)BufferLength );
-    MYODBCDbgInfo( "StrLenPtr: 0x%lx", (ulong)StringLengthPtr );
-
     if (!ValuePtr)
         ValuePtr= &vparam;
 
@@ -728,7 +684,6 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
 
     switch (Attribute)
     {
-        
         case SQL_ATTR_CURSOR_SCROLLABLE:
             if (options->cursor_type == SQL_CURSOR_FORWARD_ONLY)
                 *(SQLUINTEGER*)ValuePtr= SQL_NONSCROLLABLE;
@@ -769,7 +724,7 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
             break;
 
         case SQL_ATTR_ROWS_FETCHED_PTR:
-            ValuePtr= (SQLUINTEGER *)options->rowsFetchedPtr;
+            ValuePtr= (SQLULEN *)options->rowsFetchedPtr;
             break;
 
         case SQL_ATTR_SIMULATE_CURSOR:
@@ -813,7 +768,7 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
                                      StringLengthPtr);
     }
 
-    MYODBCDbgReturnReturn(result);
+    return result;
 }
 
 
@@ -826,16 +781,7 @@ SQLRETURN SQL_API SQLSetConnectOption( SQLHDBC      hdbc,
                                        SQLUSMALLINT fOption,
                                        SQLULEN      vParam )
 {
-    SQLRETURN result= SQL_SUCCESS;
-
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hdbc: %p", hdbc );
-    MYODBCDbgInfo( "fOption: %s", MYODBCDbgConnectOptionString( fOption ) );
-    MYODBCDbgInfo( "fOption: %d", fOption );
-
-    result= set_con_attr(hdbc,fOption,(SQLPOINTER)vParam,SQL_NTS);
-
-    MYODBCDbgReturnReturn(result);
+  return set_con_attr(hdbc, fOption, (SQLPOINTER)vParam, SQL_NTS);
 }
 
 
@@ -847,16 +793,7 @@ SQLRETURN SQL_API SQLSetConnectOption( SQLHDBC      hdbc,
 SQLRETURN SQL_API SQLGetConnectOption(SQLHDBC hdbc,SQLUSMALLINT fOption,
                                       SQLPOINTER vParam)
 {
-    SQLRETURN result= SQL_SUCCESS;
-
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hdbc: %p", hdbc );
-    MYODBCDbgInfo( "fOption: %s", MYODBCDbgConnectOptionString( fOption ) );
-    MYODBCDbgInfo( "fOption: %d", fOption );
-
-    result= get_con_attr(hdbc,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
-
-    MYODBCDbgReturnReturn(result);
+  return get_con_attr(hdbc, fOption, vParam, SQL_NTS, (SQLINTEGER *)NULL);
 }
 
 
@@ -869,16 +806,7 @@ SQLRETURN SQL_API SQLSetStmtOption( SQLHSTMT        hstmt,
                                     SQLUSMALLINT    fOption,
                                     SQLULEN         vParam )
 {
-    SQLRETURN result= SQL_SUCCESS;
-
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hstmt: %p", hstmt );
-    MYODBCDbgInfo( "fOption: %s", MYODBCDbgStmtOptionString( fOption ) );
-    MYODBCDbgInfo( "fOption: %d", fOption );
-
-    result= set_stmt_attr(hstmt,fOption,(SQLPOINTER)vParam,SQL_NTS);
-
-    MYODBCDbgReturnReturn(result);
+  return set_stmt_attr(hstmt,fOption,(SQLPOINTER)vParam,SQL_NTS);
 }
 
 
@@ -890,16 +818,7 @@ SQLRETURN SQL_API SQLSetStmtOption( SQLHSTMT        hstmt,
 SQLRETURN SQL_API SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT fOption,
                                    SQLPOINTER vParam)
 {
-    SQLRETURN result= SQL_SUCCESS;
-
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hstmt: %p", hstmt );
-    MYODBCDbgInfo( "fOption: %s", MYODBCDbgStmtOptionString( fOption ) );
-    MYODBCDbgInfo( "fOption: %d", fOption );
-
-    result= get_stmt_attr(hstmt,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
-
-    MYODBCDbgReturnReturn(result);
+  return get_stmt_attr(hstmt, fOption, vParam, SQL_NTS, (SQLINTEGER *)NULL);
 }
 
 
@@ -914,18 +833,12 @@ SQLSetEnvAttr(SQLHENV    henv,
               SQLPOINTER ValuePtr,
               SQLINTEGER StringLength __attribute__((unused)))
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "henv: %p", henv );
-    MYODBCDbgInfo( "Atrr: %s", MYODBCDbgEnvAttrString( Attribute ) );
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-    MYODBCDbgInfo( "ValuePtr: 0x%lx", (ulong)ValuePtr );
 
     if (((ENV FAR *)henv)->connections)
-        MYODBCDbgReturnReturn( set_env_error(henv,MYERR_S1010,NULL,0) );
+        return set_env_error(henv, MYERR_S1010, NULL, 0);
 
     switch (Attribute)
     {
-        
         case SQL_ATTR_ODBC_VERSION:
             ((ENV FAR *)henv)->odbc_ver= (SQLINTEGER)ValuePtr;
             break;
@@ -935,9 +848,9 @@ SQLSetEnvAttr(SQLHENV    henv,
                 break;
 
         default:
-            MYODBCDbgReturnReturn(set_env_error(henv,MYERR_S1C00,NULL,0));
+            return set_env_error(henv,MYERR_S1C00,NULL,0);
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -953,11 +866,6 @@ SQLGetEnvAttr(SQLHENV    henv,
               SQLINTEGER BufferLength __attribute__((unused)),
               SQLINTEGER *StringLengthPtr __attribute__((unused)))
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "henv: %p", henv );
-    MYODBCDbgInfo( "Atrr: %s", MYODBCDbgEnvAttrString( Attribute ) );
-    MYODBCDbgInfo( "Atrr: %d", (int)Attribute );
-
     switch ( Attribute )
     {
         case SQL_ATTR_CONNECTION_POOLING:
@@ -973,9 +881,9 @@ SQLGetEnvAttr(SQLHENV    henv,
             break;
 
         default:
-            MYODBCDbgReturnReturn(set_env_error(henv,MYERR_S1C00,NULL,0));
+            return set_env_error(henv,MYERR_S1C00,NULL,0);
     }
-    MYODBCDbgReturnReturn(SQL_SUCCESS);
+    return SQL_SUCCESS;
 }
 
 
@@ -989,11 +897,7 @@ SQLRETURN SQL_API SQLSetConnectAttr(SQLHDBC hdbc,
                                     SQLPOINTER ValuePtr,
                                     SQLINTEGER StringLength)
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hdbc: %p", hdbc );
-    MYODBCDbgInfo( "Attribute: %s", MYODBCDbgConnectAttrString( Attribute ) );
-    MYODBCDbgInfo( "Attribute: %d", (int)Attribute );
-    MYODBCDbgReturnReturn( set_con_attr(hdbc,Attribute, ValuePtr,StringLength) );
+  return set_con_attr(hdbc, Attribute, ValuePtr, StringLength);
 }
 
 /*
@@ -1007,11 +911,7 @@ SQLRETURN SQL_API SQLGetConnectAttr(SQLHDBC hdbc,
                                     SQLINTEGER BufferLength,
                                     SQLINTEGER *StringLengthPtr)
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hdbc: %p", hdbc );
-    MYODBCDbgInfo( "Attribute: %s", MYODBCDbgConnectAttrString( Attribute ) );
-    MYODBCDbgInfo( "Attribute: %d", (int)Attribute );
-    MYODBCDbgReturnReturn( get_con_attr( hdbc,Attribute, ValuePtr,BufferLength, StringLengthPtr ) );
+  return get_con_attr(hdbc, Attribute, ValuePtr, BufferLength, StringLengthPtr);
 }
 
 /*
@@ -1024,11 +924,7 @@ SQLRETURN SQL_API SQLSetStmtAttr(SQLHSTMT   hstmt,
                                  SQLPOINTER ValuePtr,
                                  SQLINTEGER StringLength)
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hstmt: %p", hstmt );
-    MYODBCDbgInfo( "Attribute: %s", MYODBCDbgStmtAttrString( Attribute ) );
-    MYODBCDbgInfo( "Attribute: %d", (int)Attribute );
-    MYODBCDbgReturnReturn( set_stmt_attr( hstmt, Attribute, ValuePtr, StringLength ) );
+  return set_stmt_attr(hstmt, Attribute, ValuePtr, StringLength );
 }
 
 /*
@@ -1041,9 +937,6 @@ SQLRETURN SQL_API SQLGetStmtAttr(SQLHSTMT   hstmt,
                                  SQLINTEGER BufferLength,
                                  SQLINTEGER *StringLengthPtr)
 {
-    MYODBCDbgEnter;
-    MYODBCDbgInfo( "hstmt: %p", hstmt );
-    MYODBCDbgInfo( "Attribute: %s", MYODBCDbgStmtAttrString( Attribute ) );
-    MYODBCDbgInfo( "Attribute: %d", (int)Attribute );
-    MYODBCDbgReturnReturn( get_stmt_attr( hstmt,Attribute, ValuePtr,BufferLength, StringLengthPtr ) );
+  return get_stmt_attr(hstmt ,Attribute, ValuePtr, BufferLength,
+                       StringLengthPtr);
 }
