@@ -60,6 +60,8 @@
 #include <odbcinst.h>
 #endif
 
+typedef BOOL (INSTAPI *ConfigDSNFunctor)(HWND, WORD, LPCSTR, LPCSTR);
+
 /*! Our syntax. This is likely to expand over time. */
 char *szSyntax =
 "\n" \
@@ -123,20 +125,20 @@ char *szSyntax =
 "  List data source attributes for MyODBC;\n" \
 "    $ myodbc3i -q -s -n\"MySQL ODBC 3.51 Driver\"\n" \
 "  Register a driver;\n" \
-"    $ myodbc3i -a -d -t\"MySQL ODBC 3.51 Driver;DRIVER=/usr/lib/libmyodbc3.so;SETUP=/usr/lib/libmyodbc3S.so\"\n" \
+"    $ myodbc3i -a -d -t\"MySQL ODBC 3.51 Driver;Driver=/usr/lib/libmyodbc3.so;Setup=/usr/lib/libmyodbc3S.so\"\n" \
 "  Create a user data source name;\n" \
-"    $ myodbc3i -a -su -t\"DSN=MyDSN;DRIVER=MySQL ODBC 3.51 Driver;SERVER=localhost;UID=pharvey\"\n";
+"    $ myodbc3i -a -su -t\"DSN=MyDSN;Driver=MySQL ODBC 3.51 Driver;Server=localhost;UID=pharvey\"\n";
 
 /*
     Register Driver for Various Platforms
     
     XP
     
-        $ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=myodbc3.dll;SETUP=myodbc3S.dll"
+        $ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;Driver=myodbc3.dll;Setup=myodbc3S.dll"
         
         OR for old GUI...
         
-        $ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=myodbc3.dll;SETUP=myodbc3.dll"
+        $ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;Driver=myodbc3.dll;Setup=myodbc3.dll"
         
     OSX
 
@@ -444,20 +446,8 @@ int doQueryDataSourceName( UWORD nScope )
     MYODBCUTIL_DATASOURCE *pDataSource = MYODBCUtilAllocDataSource( MYODBCUTIL_DATASOURCE_MODE_DSN_VIEW );
 
     /* set scope */
-    switch ( nScope )
-    {
-        case ODBC_BOTH_DSN:
-            if ( !SQLSetConfigMode( nScope ) )
-                return FALSE;
-            break;
-        case ODBC_USER_DSN:
-        case ODBC_SYSTEM_DSN:
-            if ( !SQLSetConfigMode( nScope ) )
-                return FALSE;
-            break;
-        default:
-            return FALSE;
-    }
+    if ( !SQLSetConfigMode( nScope ) )
+        return FALSE;
 
     if ( !MYODBCUtilReadDataSource( pDataSource, pszName ) )
     {
@@ -563,6 +553,15 @@ int doAddDriver()
         return 0;
     }
 
+    switch (cObjectSub) {
+    case 'u':
+      if (!SQLSetConfigMode(ODBC_USER_DSN))
+        return FALSE;
+    case 's':
+      if (!SQLSetConfigMode(ODBC_SYSTEM_DSN))
+        return FALSE;
+    }
+
     /*
         Create a oopy of pszAttributes where the ';' are replaced
         with '\0' and ensure that at least 2 '\0' are at the end.
@@ -578,8 +577,8 @@ int doAddDriver()
             pszDriverInfo[nChar] = pszAttributes[nChar];
     }
 
-    pszDriverInfo[nChar]    = '\0';
     pszDriverInfo[nChar++]  = '\0';
+    pszDriverInfo[nChar]    = '\0';
 
     /*
         Call ODBC installer to do the work.
@@ -812,7 +811,7 @@ int doConfigDataSource( WORD nRequest )
     MYODBCUTIL_DRIVER *     pDriver             = MYODBCUtilAllocDriver();
     MYODBCUTIL_DATASOURCE * pDataSourceGiven    = MYODBCUtilAllocDataSource( MYODBCUTIL_DATASOURCE_MODE_DSN_VIEW );
     MYODBCUTIL_DATASOURCE * pDataSource         = MYODBCUtilAllocDataSource( MYODBCUTIL_DATASOURCE_MODE_DSN_VIEW );
-    BOOL                    (*pFunc)( HWND, WORD, LPCSTR, LPCSTR );
+    ConfigDSNFunctor        pFunc;
 #if defined(WIN32)
     HINSTANCE               hLib                = 0;
 #else
@@ -843,7 +842,7 @@ int doConfigDataSource( WORD nRequest )
         {
             /* pull driver from attributes as we can not have it in there for ConfigDSN() */
             sprintf( szDriver, "%s", pDataSourceGiven->pszDRIVER );
-            free( pDataSourceGiven->pszDRIVER );
+            _global_free( pDataSourceGiven->pszDRIVER );
             pDataSourceGiven->pszDRIVER = NULL;
         }
         else
@@ -891,7 +890,7 @@ int doConfigDataSource( WORD nRequest )
     }
 
     /* lookup ConfigDSN */
-    pFunc = (BOOL (*)(HWND, WORD, LPCSTR, LPCSTR )) GetProcAddress( hLib, "ConfigDSN" );
+    pFunc = (ConfigDSNFunctor) GetProcAddress( hLib, "ConfigDSN" );
     if ( !pFunc )
     {
         doPrintLastErrorString();
@@ -917,7 +916,7 @@ int doConfigDataSource( WORD nRequest )
     }
 
     /* lookup ConfigDSN */
-    pFunc = (BOOL (*)(HWND, WORD, LPCSTR, LPCSTR )) lt_dlsym( hLib, "ConfigDSN" );
+    pFunc = (ConfigDSNFunctor) lt_dlsym( hLib, "ConfigDSN" );
     if ( !pFunc )
     {
         fprintf( stderr, "[%s][%d][ERROR] Could not find ConfigDSN in (%s). Error is %s\n", __FILE__, __LINE__, pDriver->pszSETUP, lt_dlerror() );
