@@ -239,6 +239,19 @@ void MYODBCSetupDataSourceDialog::slotOk()
             pDataSource->pszSOCKET = _global_strdup( ptab2->getSocket().toAscii() );
         if ( !ptab2->getInitialStatement().isEmpty() )
             pDataSource->pszSTMT = _global_strdup( ptab2->getInitialStatement().toAscii() );
+        if ( !ptab2->getCharset().isEmpty() )
+            pDataSource->pszCHARSET = _global_strdup( ptab2->getCharset().toAscii() );
+        if ( !ptab2->getSSLKey().isEmpty() )
+            pDataSource->pszSSLKEY = _global_strdup( ptab2->getSSLKey().toAscii() );
+        if ( !ptab2->getSSLCert().isEmpty() )
+            pDataSource->pszSSLCERT = _global_strdup( ptab2->getSSLCert().toAscii() );
+        if ( !ptab2->getSSLCA().isEmpty() )
+            pDataSource->pszSSLCA = _global_strdup( ptab2->getSSLCA().toAscii() );
+        if ( !ptab2->getSSLCAPath().isEmpty() )
+            pDataSource->pszSSLCAPATH = _global_strdup( ptab2->getSSLCAPath().toAscii() );
+        if ( !ptab2->getSSLCipher().isEmpty() )
+            pDataSource->pszSSLCIPHER = _global_strdup( ptab2->getSSLCipher().toAscii() );
+
 #else
         if ( !ptab1->getDataSourceName().isEmpty() )
             pDataSource->pszDSN = _global_strdup( ptab1->getDataSourceName().ascii() );
@@ -259,13 +272,25 @@ void MYODBCSetupDataSourceDialog::slotOk()
             pDataSource->pszSOCKET = _global_strdup( ptab2->getSocket().ascii() );
         if ( !ptab2->getInitialStatement().isEmpty() )
             pDataSource->pszSTMT = _global_strdup( ptab2->getInitialStatement().ascii() );
+        if ( !ptab2->getCharset().isEmpty() )
+            pDataSource->pszCHARSET = _global_strdup( ptab2->getCharset().ascii() );
+        if ( !ptab2->getSSLKey().isEmpty() )
+            pDataSource->pszSSLKEY = _global_strdup( ptab2->getSSLKey().ascii() );
+        if ( !ptab2->getSSLCert().isEmpty() )
+            pDataSource->pszSSLCERT = _global_strdup( ptab2->getSSLCert().ascii() );
+        if ( !ptab2->getSSLCA().isEmpty() )
+            pDataSource->pszSSLCA = _global_strdup( ptab2->getSSLCA().ascii() );
+        if ( !ptab2->getSSLCAPath().isEmpty() )
+            pDataSource->pszSSLCAPATH = _global_strdup( ptab2->getSSLCAPath().ascii() );
+        if ( !ptab2->getSSLCipher().isEmpty() )
+            pDataSource->pszSSLCIPHER = _global_strdup( ptab2->getSSLCipher().ascii() );
 #endif
 
         unsigned int nFlags = ptab3->getFlags();
 
         if ( nFlags > 0 )
         {
-            pDataSource->pszOPTION = (char*)malloc( 50 );
+            pDataSource->pszOPTION = (char*)_global_alloc(50);
             sprintf( pDataSource->pszOPTION, "%d", nFlags );
         }
     }
@@ -388,6 +413,12 @@ void MYODBCSetupDataSourceDialog::doInit()
     ptab1->setServer( pDataSource->pszSERVER );
     ptab2->setSocket( pDataSource->pszSOCKET );
     ptab2->setInitialStatement( pDataSource->pszSTMT );
+    ptab2->setCharset( pDataSource->pszCHARSET );
+    ptab2->setSSLKey( pDataSource->pszSSLKEY );
+    ptab2->setSSLCert( pDataSource->pszSSLCERT );
+    ptab2->setSSLCA( pDataSource->pszSSLCA );
+    ptab2->setSSLCAPath( pDataSource->pszSSLCAPATH );
+    ptab2->setSSLCipher( pDataSource->pszSSLCIPHER );
     ptab1->setUser( pDataSource->pszUSER );
 
     if ( pDataSource->pszOPTION )
@@ -417,6 +448,7 @@ void MYODBCSetupDataSourceDialog::doInit()
         ptab3->ptab3c->pcheckboxForceUseOfForwardOnlyCursors->setChecked( nOptions & (1 << 21) ? TRUE : FALSE );  
         ptab3->ptab3a->pcheckboxEnableReconnect->setChecked( nOptions & (1 << 22) ? TRUE : FALSE );
         ptab3->ptab3a->pcheckboxAutoIncrementIsNull->setChecked( nOptions & (1 << 23) ? TRUE : FALSE );
+        ptab3->ptab3c->pcheckboxMultiStatements->setChecked( nOptions & (1 << 26) ? TRUE : FALSE );
     }
 
     connect( ppushbuttonTest, SIGNAL(clicked()), SLOT(slotTest()) );
@@ -427,6 +459,7 @@ void MYODBCSetupDataSourceDialog::doInit()
 
 #ifndef __sparc
     connect( ptab1, SIGNAL(signalRequestDatabaseNames()), SLOT(slotLoadDatabaseNames()) );
+    connect( ptab2, SIGNAL(signalRequestCharsetNames()), SLOT(slotLoadCharsetNames()) );
 #endif
 
     connect( ptab1->plineeditDataSourceName, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
@@ -438,7 +471,13 @@ void MYODBCSetupDataSourceDialog::doInit()
     connect( ptab2->plineeditPort, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab2->plineeditSocket, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab2->plineeditInitialStatement, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
-    connect( ptab3->ptab3a->pcheckboxDontOptimizeColumnWidth, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->pcomboboxCharset, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->plineeditSSLKey, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->plineeditSSLCert, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->plineeditSSLCA, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->plineeditSSLCAPath, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab2->plineeditSSLCipher, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+	connect( ptab3->ptab3a->pcheckboxDontOptimizeColumnWidth, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3a->pcheckboxReturnMatchingRows, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3a->pcheckboxAllowBigResults, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3a->pcheckboxUseCompressedProtocol, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
@@ -460,6 +499,7 @@ void MYODBCSetupDataSourceDialog::doInit()
     connect( ptab3->ptab3c->pcheckboxReadOptionsFromMyCnf, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3c->pcheckboxDisableTransactions, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3c->pcheckboxForceUseOfForwardOnlyCursors, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
+    connect( ptab3->ptab3c->pcheckboxMultiStatements, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3d->pcheckboxTraceDriverCalls, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
     connect( ptab3->ptab3d->pcheckboxSaveQueries, SIGNAL(signalAssistText(const QString&)), ptextbrowserAssist, SLOT(setHtml(const QString&)) );
 
@@ -589,6 +629,96 @@ void MYODBCSetupDataSourceDialog::doApplyMode()
     }
 }
 
+
+QString MYODBCSetupDataSourceDialog::buildConnectString()
+{
+  QString     stringConnectIn;
+
+#ifdef Q_WS_MACX
+  /*
+   The iODBC that ships with Mac OS X (10.4) must be given a filename for
+   the driver library in SQLDriverConnect(), not just the driver name.  So
+   we have to look it up using SQLGetPrivateProfileString() if we haven't
+   already.
+  */
+  {
+    if (!pDataSource->pszDriverFileName)
+    {
+      /*
+        SQLGetPrivateProfileString has bugs on iODBC, so we have to check
+        both the SYSTEM and USER space explicitly.
+       */
+      UWORD configMode;
+      if (!SQLGetConfigMode(&configMode))
+        return FALSE;
+      if (!SQLSetConfigMode(ODBC_SYSTEM_DSN))
+        return FALSE;
+
+      char driver[PATH_MAX];
+      if (!SQLGetPrivateProfileString(pDataSource->pszDRIVER,
+                                      "DRIVER", pDataSource->pszDRIVER,
+                                      driver, sizeof(driver),
+                                      "ODBCINST.INI"))
+        return FALSE;
+
+      /* If we're creating a user DSN, make sure we really got a driver.  */
+      if (configMode != ODBC_SYSTEM_DSN &&
+          strcmp(driver, pDataSource->pszDRIVER) == 0)
+      {
+        if (configMode != ODBC_SYSTEM_DSN)
+        {
+          if (!SQLSetConfigMode(ODBC_USER_DSN))
+            return FALSE;
+          if (!SQLGetPrivateProfileString(pDataSource->pszDRIVER,
+                                          "DRIVER", pDataSource->pszDRIVER,
+                                          driver, sizeof(driver),
+                                          "ODBCINST.INI"))
+            return FALSE;
+        }
+      }
+
+      pDataSource->pszDriverFileName= _global_strdup(driver);
+
+      if (!SQLSetConfigMode(configMode))
+        return FALSE;
+    }
+
+    stringConnectIn= "DRIVER=" + QString(pDataSource->pszDriverFileName);
+  }
+#else
+  stringConnectIn= "DRIVER=" + QString(pDataSource->pszDRIVER);
+#endif
+
+  stringConnectIn+= ";UID=" + ptab1->getUser();
+  stringConnectIn+= ";PWD=" + ptab1->getPassword();
+  stringConnectIn+= ";SERVER=" + ptab1->getServer();
+  if (!ptab1->getDatabase().isEmpty())
+    stringConnectIn+= ";DATABASE=" + ptab1->getDatabase();
+  if (!ptab2->getPort().isEmpty())
+    stringConnectIn+= ";PORT=" + ptab2->getPort();
+  if (!ptab2->getSocket().isEmpty())
+    stringConnectIn+= ";SOCKET=" + ptab2->getSocket();
+//    if ( !ptab1->getOptions().isEmpty() )
+//        stringConnectIn += ";OPTION=" + ptab1->getOptions();
+  if (!ptab2->getInitialStatement().isEmpty())
+    stringConnectIn+= ";STMT=" + ptab2->getInitialStatement();
+  if (!ptab2->getCharset().isEmpty())
+    stringConnectIn+= ";CHARSET=" + ptab2->getCharset();
+  if (!ptab2->getSSLKey().isEmpty())
+	  stringConnectIn+= ";SSLKEY=" + ptab2->getSSLKey();
+  if (!ptab2->getSSLCert().isEmpty())
+	  stringConnectIn+= ";SSLERT=" + ptab2->getSSLCert();
+  if (!ptab2->getSSLCA().isEmpty())
+	  stringConnectIn+= ";SSLCA=" + ptab2->getSSLCA();
+  if (!ptab2->getSSLCAPath().isEmpty())
+	  stringConnectIn+= ";SSLCAPATH=" + ptab2->getSSLCAPath();
+  if (!ptab2->getSSLCipher().isEmpty())
+	  stringConnectIn+= ";SSLCIPHER=" + ptab2->getSSLCipher();
+
+  return stringConnectIn;
+}
+
+
 /*!
     \brief  Try connect/disconnect.
 
@@ -608,39 +738,7 @@ BOOL MYODBCSetupDataSourceDialog::doTestUsingDriverManager()
     SQLHENV     hEnv        = SQL_NULL_HENV;
     SQLHDBC     hDbc        = SQL_NULL_HDBC;
     SQLRETURN   nReturn;
-    QString     stringConnectIn;
-
-    /* build connect string in */
-#ifdef Q_WS_MACX
-    /*!
-        \note OSX
-
-              Yet another place where iODBC fails to act like the ODBC
-              specification. SQLDriverConnect() should accept the 
-              driver *description* and should allow for the curly
-              braces but fails on both accounts deciding instead to
-              interpret the whole value as a driver file name.
-
-              Its possible to do the xref here, for SQLDriverConnect,
-              but due to time constraints the file is hard-coded.
-    */ 
-    stringConnectIn  = "Driver=/usr/lib/libmyodbc3.dylib";
-#else
-    stringConnectIn  = "DRIVER=" + QString( pDataSource->pszDRIVER );
-#endif
-    stringConnectIn += ";UID=" + ptab1->getUser();
-    stringConnectIn += ";PWD=" + ptab1->getPassword();
-    stringConnectIn += ";SERVER=" + ptab1->getServer();
-    if ( !ptab1->getDatabase().isEmpty() )
-        stringConnectIn += ";DATABASE=" + ptab1->getDatabase();
-    if ( !ptab2->getPort().isEmpty() )
-        stringConnectIn += ";PORT=" + ptab2->getPort();
-    if ( !ptab2->getSocket().isEmpty() )
-        stringConnectIn += ";SOCKET=" + ptab2->getSocket();
-//    if ( !ptab1->getOptions().isEmpty() )
-//        stringConnectIn += ";OPTION=" + ptab1->getOptions();
-    if ( !ptab2->getInitialStatement().isEmpty() )
-        stringConnectIn += ";STMT=" + ptab2->getInitialStatement();
+    QString     stringConnectIn= buildConnectString();
 
     nReturn = SQLAllocHandle( SQL_HANDLE_ENV, NULL, &hEnv );
     if ( nReturn != SQL_SUCCESS )
@@ -699,47 +797,115 @@ BOOL MYODBCSetupDataSourceDialog::doTestUsingDriver()
     return TRUE;
 }
 
+BOOL MYODBCSetupDataSourceDialog::doLoadCharsetNamesUsingDriverManager()
+{
+    SQLHENV     hEnv        = SQL_NULL_HENV;
+    SQLHDBC     hDbc        = hDBC;
+    SQLHSTMT    hStmt;
+    SQLRETURN   nReturn;
+    QStringList stringlistDatabases;
+    SQLCHAR     szCatalog[MYODBC_DB_NAME_MAX]; 
+    SQLLEN      nCatalog;
+    QString     stringConnectIn= buildConnectString();
+
+    stringlistDatabases += " ";
+
+    if ( hDBC == SQL_NULL_HDBC )
+    {
+        nReturn = SQLAllocHandle( SQL_HANDLE_ENV, NULL, &hEnv );
+        if ( nReturn != SQL_SUCCESS )
+            slotShowDiagnostics( nReturn, SQL_HANDLE_ENV, NULL );
+        if ( !SQL_SUCCEEDED(nReturn) )
+            return FALSE;
+    
+        nReturn = SQLSetEnvAttr( hEnv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER)SQL_OV_ODBC3, 0 );
+        if ( nReturn != SQL_SUCCESS )
+            slotShowDiagnostics( nReturn, SQL_HANDLE_ENV, NULL );
+        if ( !SQL_SUCCEEDED(nReturn) )
+            goto slotLoadDatabaseNamesExit1;
+    
+        nReturn = SQLAllocHandle( SQL_HANDLE_DBC, hEnv, &hDbc );
+        if ( nReturn != SQL_SUCCESS )
+            slotShowDiagnostics( nReturn, SQL_HANDLE_ENV, hEnv );
+        if ( !SQL_SUCCEEDED(nReturn) )
+            goto slotLoadDatabaseNamesExit1;
+    }
+
+#if QT_VERSION >= 0x040000
+    nReturn = SQLDriverConnect( hDbc, NULL, (SQLCHAR*)stringConnectIn.toAscii().data(), SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT );
+#else
+    nReturn = SQLDriverConnect( hDbc, NULL, (SQLCHAR*)stringConnectIn.latin1(), SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT );
+#endif
+    if ( nReturn != SQL_SUCCESS )
+        slotShowDiagnostics( nReturn, SQL_HANDLE_DBC, hDbc );
+    if ( !SQL_SUCCEEDED(nReturn) )
+        goto slotLoadDatabaseNamesExit2;
+
+    nReturn = SQLAllocHandle( SQL_HANDLE_STMT, hDbc, &hStmt );
+    if ( nReturn != SQL_SUCCESS )
+        slotShowDiagnostics( nReturn, SQL_HANDLE_DBC, hDbc );
+    if ( !SQL_SUCCEEDED(nReturn) )
+        goto slotLoadDatabaseNamesExit2;
+
+    nReturn = SQLExecDirect( hStmt, (SQLCHAR *)"SHOW CHARACTER SET", SQL_NTS);
+    if ( nReturn != SQL_SUCCESS )
+        slotShowDiagnostics( nReturn, SQL_HANDLE_STMT, hStmt );
+    if ( !SQL_SUCCEEDED(nReturn) )
+        goto slotLoadDatabaseNamesExit3;
+
+    nReturn = SQLBindCol( hStmt, 1, SQL_C_CHAR, szCatalog, MYODBC_DB_NAME_MAX, &nCatalog );
+    while ( TRUE )
+    {
+        nReturn = SQLFetch( hStmt );
+        if ( nReturn == SQL_NO_DATA )
+            break;
+        else if ( nReturn != SQL_SUCCESS )
+            slotShowDiagnostics( nReturn, SQL_HANDLE_STMT, hStmt );
+        if ( SQL_SUCCEEDED(nReturn) )
+            stringlistDatabases += (const char*)szCatalog;
+        else
+            break;
+    }
+
+slotLoadDatabaseNamesExit3:
+    nReturn = SQLFreeHandle( SQL_HANDLE_STMT, hStmt );
+slotLoadDatabaseNamesExit2:
+    nReturn = SQLDisconnect( hDbc );
+    if ( hDBC == SQL_NULL_HDBC )
+        nReturn = SQLFreeHandle( SQL_HANDLE_DBC, hDbc );
+slotLoadDatabaseNamesExit1:
+    if ( hDBC == SQL_NULL_HDBC )
+        nReturn = SQLFreeHandle( SQL_HANDLE_ENV, hEnv );
+
+    ptab2->pcomboboxCharset->clear();
+#if QT_VERSION >= 0x040000
+    ptab2->pcomboboxCharset->addItems( stringlistDatabases );
+#else
+    ptab2->pcomboboxCharset->insertStringList( stringlistDatabases );
+#endif
+
+    return TRUE;
+}
+
+BOOL MYODBCSetupDataSourceDialog::doLoadCharsetNamesUsingDriver()
+{
+    QMessageBox::warning( this, "Connector/ODBC", "Loading character set list not implemented for SQLDriverConnect()", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton );
+    return TRUE;
+}
+
 BOOL MYODBCSetupDataSourceDialog::doLoadDatabaseNamesUsingDriverManager()
 {
     SQLHENV     hEnv        = SQL_NULL_HENV;
     SQLHDBC     hDbc        = hDBC;
     SQLHSTMT    hStmt;
     SQLRETURN   nReturn;
-    QString     stringConnectIn;
     QStringList stringlistDatabases;
     SQLCHAR     szCatalog[MYODBC_DB_NAME_MAX]; 
     SQLLEN      nCatalog;
+    QString     stringConnectIn= buildConnectString();
 
     stringlistDatabases += " ";
 
-    /* build connect string in */
-#ifdef Q_WS_MACX
-    /*!
-        \note OSX
-
-              Yet another place where iODBC fails to act like the ODBC
-              specification. SQLDriverConnect() should accept the 
-              driver *description* and should allow for the curly
-              braces but fails on both accounts deciding instead to
-              interpret the whole value as a driver file name.
-
-              Its possible to do the xref here, for SQLDriverConnect,
-              but due to time constraints the file is hard-coded.
-    */ 
-    stringConnectIn  = "Driver=/usr/lib/libmyodbc3.dylib";
-#else
-    stringConnectIn  = "DRIVER=" + QString( pDataSource->pszDRIVER );
-#endif
-    stringConnectIn += ";UID=" + ptab1->getUser();
-    stringConnectIn += ";PWD=" + ptab1->getPassword();
-    stringConnectIn += ";SERVER=" + ptab1->getServer();
-    if ( !ptab2->getPort().isEmpty() )
-        stringConnectIn += ";PORT=" + ptab2->getPort();
-    if ( !ptab2->getSocket().isEmpty() )
-        stringConnectIn += ";SOCKET=" + ptab2->getSocket();
-//    if ( !plineeditOptions->text().isEmpty() )
-//        stringConnectIn += ";OPTION=" + plineeditOptions->text();
-
     if ( hDBC == SQL_NULL_HDBC )
     {
         nReturn = SQLAllocHandle( SQL_HANDLE_ENV, NULL, &hEnv );
@@ -915,3 +1081,13 @@ void MYODBCSetupDataSourceDialog::slotLoadDatabaseNames()
 }
 
 
+void MYODBCSetupDataSourceDialog::slotLoadCharsetNames()
+{
+  if (hDBC)
+    doLoadCharsetNamesUsingDriver();
+  else
+    doLoadCharsetNamesUsingDriverManager();
+}
+
+
+
