@@ -209,42 +209,6 @@ void myerror( SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle, const char *szF
     }
 }
 
-/**
-  CONNECTION
-*/
-void myconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
-{
-    SQLRETURN rc;
-
-    rc = SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,henv);
-    myenv(*henv,rc);   
-
-    rc = SQLSetEnvAttr(*henv,SQL_ATTR_ODBC_VERSION,(SQLPOINTER)SQL_OV_ODBC3,0);
-    myenv(*henv,rc);   
-
-    rc = SQLAllocHandle(SQL_HANDLE_DBC,*henv, hdbc);
-    myenv(*henv,rc);    
-
-    printMessage(" Connecting to '%s' with user name '%s'...\n",mydsn,myuid);
-    rc = SQLConnect(*hdbc, mydsn, SQL_NTS, myuid, SQL_NTS,  mypwd, SQL_NTS);
-    mycon(*hdbc,rc);
-
-    rc = SQLSetConnectAttr(*hdbc,SQL_ATTR_AUTOCOMMIT,(SQLPOINTER)SQL_AUTOCOMMIT_ON,0);
-    mycon(*hdbc,rc);
-
-    rc = SQLAllocHandle(SQL_HANDLE_STMT,*hdbc,hstmt);
-    mycon(*hdbc,rc);
-
-    {
-        size_t i;
-        for (i=0; i< sizeof(init_db)/sizeof(init_db[0]); i++)
-            SQLExecDirect(*hstmt, (SQLCHAR *)init_db[i], SQL_NTS);
-    }
-    SQLFreeStmt(*hstmt, SQL_CLOSE);
-    SQLSetStmtAttr(*hstmt,SQL_ATTR_CURSOR_TYPE,(SQLPOINTER)SQL_CURSOR_STATIC,0);
-    SQLSetStmtOption(*hstmt,SQL_SIMULATE_CURSOR,SQL_SC_NON_UNIQUE);
-    SQLSetStmtOption(*hstmt, SQL_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN);
-}
 /**
   DRV CONNECTION
 */
@@ -252,6 +216,7 @@ void mydrvconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt,SQLCHAR *connIn)
 {
     SQLRETURN rc;
     SQLCHAR   connOut[MAX_NAME_LEN];
+    SQLSMALLINT StringLength;
 
     rc = SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,henv);
     myenv(*henv,rc);   
@@ -264,7 +229,7 @@ void mydrvconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt,SQLCHAR *connIn)
 
     printMessage(" Connecting to '%s' \n",connIn);
     rc = SQLDriverConnect(*hdbc,NULL,connIn,MAX_NAME_LEN,
-                          connOut,MAX_NAME_LEN,NULL,SQL_DRIVER_NOPROMPT);
+                          connOut,MAX_NAME_LEN,&StringLength,SQL_DRIVER_NOPROMPT);
     mycon(*hdbc,rc);
     printMessage( "output connection string: %s\n", connOut);
 
@@ -284,31 +249,6 @@ void mydrvconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt,SQLCHAR *connIn)
     SQLSetStmtOption(*hstmt,SQL_SIMULATE_CURSOR,SQL_SC_NON_UNIQUE);
     SQLSetStmtOption(*hstmt, SQL_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN);
 }
-/**
-  DISCONNECT
-*/
-void mydisconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
-{
-    SQLRETURN rc;
-
-    SQLExecDirect(*hstmt, (SQLCHAR *)init_db[0], SQL_NTS);
-    SQLFreeStmt(*hstmt, SQL_CLOSE);
-
-    rc = SQLEndTran(SQL_HANDLE_DBC, *hdbc, SQL_COMMIT);
-    mycon(*hdbc, rc);
-
-    rc = SQLFreeStmt(*hstmt, SQL_DROP);
-    mystmt(*hstmt,rc);
-
-    rc = SQLDisconnect(*hdbc);
-    mycon(*hdbc,rc);
-
-    rc = SQLFreeConnect(*hdbc);
-    mycon(*hdbc,rc);
-
-    rc = SQLFreeEnv(*henv);
-    myenv(*henv,rc);
-}
 
 /**
   Print resultset dashes
@@ -317,12 +257,12 @@ void mydisconnect(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
 static void my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
 {
     SQLRETURN  rc;
-    SQLINTEGER field_count, disp_size, nullable;
+    SQLLEN     disp_size, nullable;
     SQLCHAR    ColName[MAX_NAME_LEN+1];
-    SQLUSMALLINT i, j;
+    SQLUSMALLINT field_count, i, j;
     SQLSMALLINT  col_len;
 
-    field_count= (SQLINTEGER)nCol;
+    field_count= (SQLUSMALLINT)nCol;
     printMessage( "\t" );
     printMessage( "+" );
 
@@ -359,7 +299,7 @@ static void my_print_data(SQLHSTMT hstmt, SQLUSMALLINT index,
                           SQLCHAR *data, SQLINTEGER length)
 {
     SQLRETURN  rc;
-    SQLINTEGER disp_size, nullable;
+    SQLLEN     disp_size, nullable;
     SQLCHAR    ColName[MAX_NAME_LEN+1];
     SQLSMALLINT col_len;
 
@@ -543,17 +483,18 @@ SQLRETURN tmysql_prepare(SQLHSTMT hstmt, char *sql_stmt)
 SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT irow)
 {
     SQLINTEGER nData;
+    SQLLEN len;
 
-    SQLGetData(hstmt,irow,SQL_INTEGER,&nData,0,NULL);
-    printMessage(" my_fetch_int: %ld\n", (long int)nData);
-    return(nData);
+    SQLGetData(hstmt, irow, SQL_INTEGER, &nData, 0, &len);
+    printMessage(" my_fetch_int: %ld (%ld)\n", (long int)nData, len);
+    return (len != SQL_NULL_DATA) ? nData : 0;
 }
 /**
   return string data, by fetching it
 */
 const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT irow)
 {
-    SQLLEN nLen= 0;
+    SQLLEN nLen;
 
     SQLGetData(hstmt,irow,SQL_CHAR,szData,MAX_ROW_DATA_LEN+1,&nLen);
     printMessage( " my_fetch_str: %s(%ld)\n",szData,nLen);
@@ -591,26 +532,10 @@ bool driver_supports_setpos(SQLHDBC hdbc)
     return true;
 }
 
-/*
-  Check if driver supports SQLSetPos
-*/
-bool driver_supports_positioned_ops(SQLHDBC hdbc)
-{
-    SQLRETURN    rc;
-    SQLINTEGER status;
-
-    rc = SQLGetInfo(hdbc, SQL_POS_OPERATIONS, &status, 0, NULL);
-    mycon(hdbc,rc);
-
-    if ((status & SQL_POS_UPDATE) || (status & SQL_POS_DELETE))
-        return true;
-    return false;
-}
-
 /*
   Check for minimal MySQL version
 */
-bool mysql_min_version(SQLHDBC hdbc, SQLCHAR *min_version, unsigned int length)
+bool mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 {
     SQLCHAR server_version[MYSQL_NAME_LEN];
     SQLRETURN rc;
@@ -621,7 +546,7 @@ bool mysql_min_version(SQLHDBC hdbc, SQLCHAR *min_version, unsigned int length)
     rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
     mycon(hdbc, rc);
 
-    if (strncmp((char *)server_version, (char *)min_version, length) >= 0)
+    if (strncmp((char *)server_version, min_version, length) >= 0)
         return true;
 
     return false;
@@ -630,7 +555,7 @@ bool mysql_min_version(SQLHDBC hdbc, SQLCHAR *min_version, unsigned int length)
 /*
   Check for minimal Connector/ODBC version
 */
-bool driver_min_version(SQLHDBC hdbc, SQLCHAR *min_version, unsigned int length)
+bool driver_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 {
     SQLCHAR driver_version[MYSQL_NAME_LEN];
     SQLRETURN rc;
@@ -641,7 +566,7 @@ bool driver_min_version(SQLHDBC hdbc, SQLCHAR *min_version, unsigned int length)
     rc = SQLGetInfo(hdbc,SQL_DRIVER_VER,driver_version,MYSQL_NAME_LEN,NULL);
     mycon(hdbc, rc);
 
-    if (strncmp((char *)driver_version, (char *)min_version, length) >= 0)
+    if (strncmp((char *)driver_version, min_version, length) >= 0)
         return true;
 
     return false;
@@ -663,29 +588,4 @@ bool server_supports_trans(SQLHDBC hdbc)
     return false;
 }
 
-/*
-  Check if server supports GRANT
-*/
-bool server_supports_grant(SQLHSTMT hstmt)
-{
-    SQLRETURN   rc;
-
-    rc= SQLExecDirect(hstmt,
-                      (SQLCHAR *)"GRANT ALL ON *.* TO 'mysqltest_1'",
-                      SQL_NTS);
-    if (rc == SQL_ERROR)
-    {
-        SQLINTEGER native_err;
-
-        rc = SQLGetDiagField(SQL_HANDLE_STMT,hstmt,1,SQL_DIAG_NATIVE,&native_err,0,NULL);
-        mystmt(hstmt,rc);
-
-        if (native_err == 1047) /* command not found */
-            return false;
-    }
-    SQLExecDirect(hstmt,
-                  (SQLCHAR *)"DELETE FROM mysql.user WHERE User='mysqltest_1'",
-                  SQL_NTS);
-    return true;
-}
 #endif /* __TMYODBC__TEST__H */
