@@ -135,6 +135,8 @@ DECLARE_TEST(t_blob)
     mystmt(hstmt,rc);
     free(blobbuf);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS TBLOB");
+
   return OK;
 }
 
@@ -195,6 +197,8 @@ DECLARE_TEST(t_1piecewrite2)
     mycon(hdbc,rc);
     free(blobbuf);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS TBLOB");
+
   return OK;
 }
 
@@ -261,9 +265,12 @@ DECLARE_TEST(t_putdata)
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_putdata");
+
   return OK;
 }
 
+
 /* Test for a simple SQLPutData and SQLParamData handling for longtext */
 DECLARE_TEST(t_putdata1)
 {
@@ -329,6 +336,8 @@ DECLARE_TEST(t_putdata1)
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_putdata");
+
   return OK;
 }
 
@@ -414,6 +423,8 @@ DECLARE_TEST(t_putdata2)
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_putdata");
+
   return OK;
 }
 
@@ -422,11 +433,8 @@ DECLARE_TEST(t_putdata2)
 DECLARE_TEST(t_putdata3)
 {
   SQLRETURN   rc;
-  SQLINTEGER  id, id1, id2, id3, resId;
-  SQLINTEGER  resUTimeSec;
-  SQLINTEGER  resUTimeMSec;
-  SQLINTEGER  resDataLen;
-  SQLLEN      resData;
+  SQLINTEGER  id, id1, id2, id3;
+  SQLLEN      resId, resUTimeSec, resUTimeMSec, resDataLen, resData;
 
   char buffer[]= "MySQL - The worlds's most popular open source database";
   const int MAX_PART_SIZE = 5;
@@ -535,7 +543,7 @@ DECLARE_TEST(t_blob_bug)
 {
   SQLRETURN  rc;
   SQLCHAR    *data;
-  SQLINTEGER i;
+  SQLINTEGER i, val;
   SQLLEN     length;
   const SQLINTEGER max_blob_size=1024*100;
 
@@ -573,7 +581,7 @@ DECLARE_TEST(t_blob_bug)
     rc = SQLExecDirect(hstmt,"SELECT length(blb) FROM t_blob",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&length,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&val,0,NULL);
     mystmt(hstmt,rc);
 
     for (i= 1; i <= max_blob_size/1024; i++)
@@ -581,8 +589,8 @@ DECLARE_TEST(t_blob_bug)
       rc = SQLFetch(hstmt);
       mystmt(hstmt,rc);
 
-      fprintf(stdout,"row %d length: %d\n", i, length);
-      myassert(length == i * 1024);
+      fprintf(stdout,"row %d length: %d\n", i, val);
+      myassert(val == i * 1024);
     }
     rc = SQLFetch(hstmt);
     myassert(rc == SQL_NO_DATA);
@@ -592,6 +600,8 @@ DECLARE_TEST(t_blob_bug)
     SQLFreeStmt(hstmt,SQL_UNBIND);
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_blob");
+
   return OK;
 }
 
@@ -741,6 +751,42 @@ DECLARE_TEST(t_bug9781)
 }
 
 
+/*
+ * Bug #10562 - Large blobs fail in a cursor
+ */
+DECLARE_TEST(t_bug10562)
+{
+  SQLLEN bsize = 12 * 1024;
+  /* Test to just insert 12k blob */
+  SQLCHAR *blob = malloc(bsize);
+  SQLCHAR *blobcheck = malloc(bsize);
+  memset(blob, 'X', bsize);
+
+  ok_sql(hstmt, "drop table if exists t_bug10562");
+  ok_sql(hstmt, "create table t_bug10562 ( id int not null primary key, mb longblob )");
+  ok_sql(hstmt, "insert into t_bug10562 (mb) values ('zzzzzzzzzz')");
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "select id, mb from t_bug10562", SQL_NTS));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_BINARY, blob, bsize, &bsize));
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_UPDATE, SQL_LOCK_NO_CHANGE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Get the data back out to verify */
+  ok_sql(hstmt, "select mb from t_bug10562");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, blobcheck, bsize, NULL));
+  is(!memcmp(blob, blobcheck, bsize));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists t_bug10562");
+  free(blob);
+  free(blobcheck);
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(t_blob)
   ADD_TEST(t_1piecewrite2)
@@ -752,6 +798,7 @@ BEGIN_TESTS
   ADD_TEST(t_text_fetch)
   ADD_TEST(getdata_lenonly)
   ADD_TEST(t_bug9781)
+  ADD_TEST(t_bug10562)
 END_TESTS
 
 
