@@ -45,6 +45,8 @@ DECLARE_TEST(my_columns_null)
 
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS my_column_null");
+
   return OK;
 }
 
@@ -80,41 +82,6 @@ DECLARE_TEST(my_drop_table)
 }
 
 
-DECLARE_TEST(my_table_priv)
-{
-    SQLRETURN   rc;
-
-    rc = SQLTablePrivileges(hstmt,"te%",SQL_NTS,
-                            NULL,0,"my_%",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    my_print_non_format_result(hstmt);
-
-    SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-  return OK;
-}
-
-
-void check_sqlstate(SQLHSTMT hstmt,SQLCHAR *sqlstate)
-{
-    SQLCHAR     sql_state[6];
-    SQLINTEGER    err_code=0;
-    SQLCHAR         err_msg[SQL_MAX_MESSAGE_LENGTH]={0};
-    SQLSMALLINT   err_len=0;
-
-    memset(err_msg,'C',SQL_MAX_MESSAGE_LENGTH);
-    SQLGetDiagRec(SQL_HANDLE_STMT,hstmt,1,
-                  (SQLCHAR *)&sql_state,(SQLINTEGER *)&err_code,
-                  (SQLCHAR*)&err_msg, SQL_MAX_MESSAGE_LENGTH-1,
-                  (SQLSMALLINT *)&err_len);
-
-    printMessage("\n\t ERROR: %s\n",err_msg);
-    printMessage("\n SQLSTATE (expected:%s, obtained:%s)\n",sqlstate,sql_state);
-    myassert(strcmp(sql_state,sqlstate)==0);
-
-}
 #define TODBC_BIND_CHAR(n,buf) SQLBindCol(hstmt,n,SQL_C_CHAR,&buf,sizeof(buf),NULL);
 
 
@@ -123,6 +90,7 @@ DECLARE_TEST(my_table_dbs)
     SQLCHAR    database[100];
     SQLRETURN  rc;
     SQLINTEGER nrows;
+    SQLLEN lenOrNull;
 
     SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test1",   SQL_NTS);
     SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test2",   SQL_NTS);
@@ -166,31 +134,31 @@ DECLARE_TEST(my_table_dbs)
     memset(database,0,100);
     rc = SQLGetData(hstmt,1,SQL_C_CHAR,(SQLCHAR *)&database,100,NULL);
     mystmt(hstmt,rc);
-    printMessage("\n database: %s", database);
+    printMessage("\n catalog: %s", database);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,2,SQL_C_CHAR,(SQLCHAR *)&database,100,NULL);
+    rc = SQLGetData(hstmt,2,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
+    mystmt(hstmt,rc);
+    printMessage("\n schema: %s", database); 
+    myassert(lenOrNull == SQL_NULL_DATA);
+
+    memset(database,0,100);
+    rc = SQLGetData(hstmt,3,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
     mystmt(hstmt,rc);
     printMessage("\n table: %s", database); 
-    myassert(strcmp(database,"")==0);
+    myassert(lenOrNull == SQL_NULL_DATA);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,3,SQL_C_CHAR,(SQLCHAR *)&database,100,NULL);
+    rc = SQLGetData(hstmt,4,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
     mystmt(hstmt,rc);
-    printMessage("\n table: %s", database); 
-    myassert(strcmp(database,"")==0);
+    printMessage("\n type: %s", database); 
+    myassert(lenOrNull == SQL_NULL_DATA);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,4,SQL_C_CHAR,(SQLCHAR *)&database,100,NULL);
-    mystmt(hstmt,rc);
-    printMessage("\n table: %s", database); 
-    myassert(strcmp(database,"")==0);
-
-    memset(database,0,100);
-    rc = SQLGetData(hstmt,5,SQL_C_CHAR, (SQLCHAR*)&database,100,NULL);
+    rc = SQLGetData(hstmt,5,SQL_C_CHAR, (SQLCHAR*)&database,100,&lenOrNull);
     mystmt(hstmt,rc);
     printMessage("\n database remark: %s", database);
-    myassert(strcmp(database,"")==0);
+    myassert(lenOrNull == SQL_NULL_DATA);
 
     SQLFreeStmt(hstmt,SQL_UNBIND);
     SQLFreeStmt(hstmt,SQL_CLOSE);
@@ -233,6 +201,14 @@ DECLARE_TEST(my_table_dbs)
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
+    /* unknown table should be empty */
+    rc = SQLTables(hstmt,"my_all_db_test%",SQL_NTS,NULL,0,"xyz",SQL_NTS,NULL,0);
+    mystmt(hstmt,rc);
+
+    assert(0 == my_print_non_format_result(hstmt));
+    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
+    mystmt(hstmt,rc);
+
     rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test1",  SQL_NTS);
     mystmt(hstmt,rc);
     rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test2",  SQL_NTS);
@@ -277,349 +253,9 @@ void my_colpriv_init(SQLHDBC hdbc,SQLHSTMT hstmt)
 }
 
 
-bool my_tablepriv_init(SQLHDBC hdbc,SQLHSTMT hstmt)
-{
-    SQLRETURN rc;
-
-    SQLExecDirect( hstmt, "DROP TABLE test_tabprev1",SQL_NTS);
-    SQLExecDirect( hstmt, "DROP TABLE test_tabprev2",SQL_NTS);
-    SQLExecDirect( hstmt, "DROP TABLE test_tabprev3",SQL_NTS);
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-    rc = SQLExecDirect(hstmt, "CREATE TABLE test_tabprev1(f1 INT)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt, "CREATE TABLE test_tabprev2(f1 INT)", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt, "CREATE TABLE test_tabprev3(f1 INT)", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-    SQLExecDirect(hstmt, "DELETE FROM mysql.tables_priv where USER='my_tabpriv'",SQL_NTS);
-
-    SQLExecDirect(hstmt, "GRANT CONNECT TO my_tabpriv IDENTIFIED BY my_tabpriv",SQL_NTS);/* sybase */
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-    rc = SQLExecDirect(hstmt, "GRANT SELECT,INSERT ON test_tabprev1 TO my_tabpriv",SQL_NTS);
-    if (rc == SQL_ERROR)
-    {
-        /* probably GRANT is disabled or not enough privs */
-        return 1;
-    }
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect( hstmt, "GRANT ALL ON test_tabprev3 TO my_tabpriv",  SQL_NTS);
-    mystmt(hstmt,rc);
-
-    SQLExecDirect(  hstmt, "FLUSH PRIVILEGES",  SQL_NTS);
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-    return 0;
-
-}
-
-
-DECLARE_TEST(my_tablepriv)
-{
-    SQLRETURN   rc;
-
-    if (my_tablepriv_init(hdbc, hstmt))
-      return SKIP;
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             "mysql", SQL_NTS,/* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             NULL,SQL_NTS);       /* TableName  */
-    mystmt(hstmt,rc);
-
-    assert( 0 == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             NULL, 0,                 /* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             "test_tabprev1",SQL_NTS);  /* TableName  */
-    mystmt(hstmt,rc);
-
-    assert( 2 == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             NULL, 0,                 /* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             "test_tabprev2",SQL_NTS);  /* TableName  */
-    mystmt(hstmt,rc);
-
-    assert( 0 == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             NULL, 0,                 /* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             "test_tabprev3",SQL_NTS);  /* TableName  */
-    mystmt(hstmt,rc);
-
-    /* value changed due to two additional rows in the result related to VIEWs */
-    assert( 11 == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             NULL, 0,                 /* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             "test_%",SQL_NTS);  /* TableName  */
-    mystmt(hstmt,rc);
-
-    my_print_non_format_result(hstmt);
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-
-    printMessage("\n With All Types(CataLog,Schema and TableName)");
-
-    rc  = SQLTablePrivileges(hstmt,
-                             NULL, 0,                 /* CataLog	 */
-                             NULL,SQL_NTS,        /* SchemaName */
-                             "test_tabprev%",SQL_NTS);  /* TableName  */
-    mystmt(hstmt,rc);
-
-
-    /* value changed due to two additional rows in the result related to VIEWs */
-    assert( 13  == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLTablePrivileges(hstmt,"mysql",SQL_NTS,NULL,SQL_NTS,"tables_priv",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    my_print_non_format_result(hstmt);
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-  return OK;
-}
-
-
-DECLARE_TEST(my_tablepriv_data)
-{
-    SQLINTEGER i;
-    SQLRETURN  rc;
-    SQLCHAR       TableQualifier_buf[129];
-    SQLCHAR       TableOwner_buf[129];
-    SQLCHAR       TableName_buf[129];
-    SQLCHAR       Grantor_buf[129];
-    SQLCHAR       Grantee_buf[129];
-    SQLCHAR       Privilege_buf[129];
-    SQLCHAR       IsGrantable_buf[4];
-
-    SQLExecDirect(hstmt, "DROP USER todbc1_test",SQL_NTS);
-    SQLExecDirect(hstmt, "DROP TABLE todbc1_tpriv1",SQL_NTS);
-    SQLExecDirect(hstmt, "DROP TABLE todbc1_tpriv2",SQL_NTS);
-    SQLExecDirect(hstmt, "DELETE FROM mysql.tables_priv where Table_name='todbc1_tpriv1'",SQL_NTS);
-    SQLExecDirect(hstmt, "DELETE FROM mysql.tables_priv where User='todbc1_test'",SQL_NTS);
-    SQLTransact(NULL, hdbc, SQL_COMMIT); 
-    rc = SQLExecDirect(hstmt, "CREATE TABLE todbc1_tpriv1(a INTEGER)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt, "CREATE TABLE todbc1_tpriv2 (a INTEGER)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    SQLTransact(NULL, hdbc, SQL_COMMIT);
-
-    /*--- Test 1: DBA should have at least CREATE, DELETE, INSERT,
-          REFERENCES, SELECT and UPDATE privileges to the tables
-          (and in this order)
-      ---*/
-
-    rc = SQLTablePrivileges(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
-                            "todbc1_tpriv1", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLFetch(hstmt);
-    assert (rc == SQL_NO_DATA_FOUND);
-
-    /*--- Test 2: Grant SELECT to todbc1_test with GRANT option,
-          and INSERT and UPDATE without.
-      ---*/
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-    rc = SQLExecDirect(hstmt, "GRANT SELECT ON todbc1_tpriv1\
-               TO todbc1_test WITH GRANT OPTION",SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "GRANT INSERT, UPDATE ON todbc1_tpriv1\
-               TO todbc1_test", SQL_NTS);mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "GRANT USAGE ON todbc1_tpriv2\
-               TO todbc1_test", SQL_NTS);mystmt(hstmt,rc);
-    SQLTransact(NULL, hdbc, SQL_COMMIT);
-    rc = SQLTablePrivileges(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
-                            "todbc1_tpriv1", SQL_NTS);
-    mystmt(hstmt,rc);  
-
-    assert( 4  == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLTablePrivileges(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
-                            "todbc1_tpriv1", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    TODBC_BIND_CHAR(1, TableQualifier_buf);
-    TODBC_BIND_CHAR(2, TableOwner_buf);
-    TODBC_BIND_CHAR(3, TableName_buf);
-    TODBC_BIND_CHAR(4, Grantor_buf);
-    TODBC_BIND_CHAR(5, Grantee_buf);
-    TODBC_BIND_CHAR(6, Privilege_buf);
-    TODBC_BIND_CHAR(7, IsGrantable_buf);  
-
-    i = 0;
-    while (1)
-    {
-        rc = SQLFetch(hstmt);
-        if ((rc == SQL_SUCCESS) || (rc == SQL_SUCCESS_WITH_INFO))
-        {
-            char *p[] = { "Select", "Insert", "Update","Grant"};
-            printMessage ("\n row '%d'",i);
-            if (i < 4 && strcmp(Privilege_buf, p[i]) == 0)
-            {
-                if (strcmp(Privilege_buf, "Select") == 0 ||
-                    strcmp(Privilege_buf, "Delete") == 0 ||
-                    strcmp(Privilege_buf, "References") == 0)
-                {
-                    assert(strcmp(Grantee_buf, "todbc1_test") == 0);
-                    assert(strcmp(IsGrantable_buf, "YES") == 0);
-                }
-                else
-                {
-                    if (strcmp(Grantee_buf, "todbc1_test") == 0)
-                    {
-                        /*assert(strcmp(Grantor_buf, "venu@localhost") == 0);*/
-                    }
-                    else
-                    {
-                        assert(strcmp(Grantee_buf, "todbc1_test") == 0);
-                        assert(strcmp(IsGrantable_buf, "YES") == 0);
-                    }
-                }
-                i++;
-            }
-            else
-            {
-                assert(strcmp(Grantee_buf, "todbc1_test") != 0);
-            }
-            assert(strcmp(TableName_buf, "todbc1_tpriv1") == 0);
-        }
-        else break;
-    }
-    assert(i == 4);
-
-    /*--- test 3: Test 'LIKE' match patterns ---*/
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-    rc = SQLExecDirect(hstmt, "REVOKE INSERT, UPDATE, SELECT\
-               ON todbc1_tpriv1 FROM todbc1_test",SQL_NTS);
-    mystmt(hstmt,rc);
-    SQLTransact(NULL, hdbc, SQL_COMMIT);
-    rc = SQLTablePrivileges(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
-                            "todbc1\\_tpriv_", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    assert( 2  == my_print_non_format_result(hstmt));
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLTablePrivileges(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
-                            "todbc1\\_tpriv_", SQL_NTS);
-    mystmt(hstmt,rc);
-
-    TODBC_BIND_CHAR(1, TableQualifier_buf);
-    TODBC_BIND_CHAR(2, TableOwner_buf);
-    TODBC_BIND_CHAR(3, TableName_buf);
-    TODBC_BIND_CHAR(4, Grantor_buf);
-    TODBC_BIND_CHAR(5, Grantee_buf);
-    TODBC_BIND_CHAR(6, Privilege_buf);
-    TODBC_BIND_CHAR(7, IsGrantable_buf);  
-
-    i = 0;
-
-    while (1)
-    {
-        memset( TableName_buf, 0, sizeof(TableName_buf));
-        rc = SQLFetch(hstmt);
-        if ((rc == SQL_SUCCESS) || (rc == SQL_SUCCESS_WITH_INFO))
-        {
-            i++;
-            printMessage("\n Name:%s",TableName_buf);
-            assert(strncmp(TableName_buf, "todbc1_tpriv", 12) == 0);
-            if (strcmp(TableName_buf, "todbc1_tpriv1") == 0)
-            {
-                assert(strcmp(Grantee_buf, "todbc1_test") == 0);
-                assert(strcmp(IsGrantable_buf, "YES") == 0);
-
-            }
-            else if (strcmp(TableName_buf, "todbc1_tpriv2") == 0)
-            {
-                assert(strcmp(Grantee_buf, "todbc1_test") == 0);
-                assert(strcmp(IsGrantable_buf, "NO") == 0);
-            }
-        }
-        else break;
-    }
-    printMessage("\n i:%d",i);
-    assert(i == 2);
-
-    SQLFreeStmt(hstmt,SQL_CLOSE);
-
-  return OK;
-}
-
-
-DECLARE_TEST(my_column_priv)
-{
-    SQLRETURN   rc;
-
-    my_colpriv_init(hdbc, hstmt);
-
-    rc = SQLColumnPrivileges(hstmt,NULL,0,
-                             NULL,0,NULL,0,
-                             NULL,0);
-    mystmt(hstmt,rc);
-
-    my_print_non_format_result(hstmt);
-
-    SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
-    SQLFreeStmt(hstmt, SQL_CLOSE);
-
-  return OK;
-}
-
-
 DECLARE_TEST(my_colpriv)
 {
-  ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
-                                     "mysql", SQL_NTS, NULL, SQL_NTS,
-                                     NULL, SQL_NTS, NULL, SQL_NTS));
-
-  assert(0 == my_print_non_format_result(hstmt));
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  my_colpriv_init(hdbc, hstmt);
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
@@ -676,15 +312,18 @@ DECLARE_TEST(my_colpriv)
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
+  ok_sql(hstmt, "DROP TABLE test_colprev1, test_colprev2, test_colprev3");
   return OK;
 }
 
-
 DECLARE_TEST(t_sqlprocedures)
 {
   SQLRETURN rc;
   /** @todo check server version */
 
+  /* avoid errors in case binary log is activated */
+  ok_sql(hstmt, "SET GLOBAL log_bin_trust_function_creators = 1");
+
   ok_sql(hstmt, "DROP FUNCTION IF EXISTS t_sqlproc_func");
   ok_sql(hstmt,
          "CREATE FUNCTION t_sqlproc_func (a INT) RETURNS INT RETURN SQRT(a)");
@@ -767,6 +406,8 @@ DECLARE_TEST(t_catalog)
     }
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_catalog");
+
   return OK;
 }
 
@@ -831,20 +472,21 @@ DECLARE_TEST(t_columns)
 {
   SQLSMALLINT   NumPrecRadix, DataType, Nullable, DecimalDigits;
   SQLLEN        cbColumnSize, cbDecimalDigits, cbNumPrecRadix,
-                cbDatabaseName, cbDataType, cbNullable;
+                cbDataType, cbNullable;
+  SQLINTEGER    cbDatabaseName;
   SQLRETURN     rc;
   SQLUINTEGER   ColumnSize, i;
   SQLUINTEGER   ColumnCount= 7;
   SQLCHAR       ColumnName[MAX_NAME_LEN], DatabaseName[MAX_NAME_LEN];
   SQLINTEGER    Values[7][5][2]=
   {
-    { {5,2},  {6,4}, {0,2},  {10,2},  {1,2}},
+    { {5,2},  {5,4}, {0,2},  {10,2},  {1,2}},
     { {1,2},  {5,4},  {0,-1}, {10,-1}, {1,2}},
     { {12,2}, {20,4}, {0,-1}, {10,-1}, {0,2}},
     { {3,2},  {10,4}, {2,2},  {10,2},  {1,2}},
-    { {-6,2},  {4,4}, {0,2},  {10,2},  {0,2}},
-    { {4,2}, {11,4}, {0,2},  {10,2},  {0,2}},
-    { {-6,2}, {4,4}, {0,2},  {10,2},  {0,2}}
+    { {-6,2},  {3,4}, {0,2},  {10,2},  {0,2}},
+    { {4,2}, {10,4}, {0,2},  {10,2},  {0,2}},
+    { {-6,2}, {3,4}, {0,2},  {10,2},  {1,2}}
   };
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_columns");
@@ -870,10 +512,11 @@ DECLARE_TEST(t_columns)
   for (i= 0; i < ColumnCount; i++)
   {
     sprintf((char *)ColumnName, "col%d", (int)i);
+    printMessage("checking column `%s`", (char *)ColumnName);
 
     ok_stmt(hstmt, SQLColumns(hstmt,
                               DatabaseName, cbDatabaseName,
-                              SQL_NULL_HANDLE, 0,
+                              (SQLCHAR *)"", SQL_NTS,
                               (SQLCHAR *)"t_columns", SQL_NTS,
                               ColumnName, SQL_NTS));
 
@@ -952,8 +595,9 @@ DECLARE_TEST(t_tables_bug)
 {
   SQLRETURN   rc;
   SQLSMALLINT i, ColumnCount, pcbColName, pfSqlType, pibScale, pfNullable;
-  SQLUINTEGER pcbColDef;
+  SQLULEN     pcbColDef;
   SQLCHAR     szColName[MAX_NAME_LEN];
+  bool is51= mysql_min_version(hdbc, "5.1", 3);
 
    SQLFreeStmt(hstmt, SQL_CLOSE);
 
@@ -984,7 +628,9 @@ DECLARE_TEST(t_tables_bug)
      myassert(strcmp(t_tables_bug_data[i-1].szColName,szColName) == 0);
      myassert(t_tables_bug_data[i-1].pcbColName == pcbColName);
      myassert(t_tables_bug_data[i-1].pfSqlType == pfSqlType);
-     myassert(t_tables_bug_data[i-1].pcbColDef == pcbColDef);
+     /* This depends on NAME_LEN in mysql_com.h */
+     is(t_tables_bug_data[i-1].pcbColDef == pcbColDef ||
+        t_tables_bug_data[i-1].pcbColDef == pcbColDef / 3);
      myassert(t_tables_bug_data[i-1].pibScale == pibScale);
      myassert(t_tables_bug_data[i-1].pfNullable == pfNullable);
    }
@@ -1070,6 +716,8 @@ DECLARE_TEST(tmysql_showkeys)
     rc = SQLFreeStmt(hstmt,SQL_CLOSE);
     mystmt(hstmt,rc);
 
+  ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_spk");
+
   return OK;
 }
 
@@ -1089,10 +737,7 @@ DECLARE_TEST(t_sqltables)
     r  = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"'system table'",SQL_NTS);
     mystmt(hstmt,r);
 
-    if (driver_min_version(hdbc,"03.51.07",8))
-        myassert(myresult(hstmt) != 0);
-    else
-        myassert(0 == myresult(hstmt));
+    is_num(myresult(hstmt), 0);
 
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
@@ -1132,11 +777,7 @@ DECLARE_TEST(t_sqltables)
     r = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"%",SQL_NTS);
     mystmt(hstmt,r);
 
-    if (driver_min_version(hdbc,"03.51.07",8))
-        myassert( 2 == myresult(hstmt));
-    else
-        myassert( 1 == myresult(hstmt));
-
+    is_num(myresult(hstmt), 3);
 
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
@@ -1147,11 +788,20 @@ DECLARE_TEST(t_sqltables)
 
 /**
  Bug #4518: SQLForeignKeys returns too many foreign key
+ Bug #27723: SQLForeignKeys does not escape _ and % in the table name arguments
+
+ The original test case was extended to have a table that would inadvertantly
+ get included because of the poor escaping.
 */
 DECLARE_TEST(t_bug4518)
 {
+  SQLCHAR buff[255];
 
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug4518_c, t_bug4518_c2, "
+  /** @todo re-enable this test when I_S based SQLForeignKeys is done. */
+  if (mysql_min_version(hdbc, "5.1", 3))
+    skip("can't test foreign keys with 5.1 or later yet");
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug4518_c, t_bug4518_c2, t_bug4518ac, "
                 "                     t_bug4518_p");
   ok_sql(hstmt, "CREATE TABLE t_bug4518_p (id INT PRIMARY KEY) ENGINE=InnoDB");
   ok_sql(hstmt, "CREATE TABLE t_bug4518_c (id INT, parent_id INT,"
@@ -1166,28 +816,321 @@ DECLARE_TEST(t_bug4518)
                 "                              t_bug4518_p(id)"
                 "                            ON DELETE SET NULL)"
                 " ENGINE=InnoDB");
+  ok_sql(hstmt, "CREATE TABLE t_bug4518ac (id INT, parent_id INT,"
+                "                          FOREIGN KEY (parent_id)"
+                "                           REFERENCES"
+                "                             t_bug4518_p(id)"
+                "                           ON DELETE SET NULL)"
+                " ENGINE=InnoDB");
 
   ok_stmt(hstmt, SQLForeignKeys(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0,
                                 NULL, 0, (SQLCHAR *)"t_bug4518_c", SQL_NTS));
 
-  my_assert(1 == my_print_non_format_result(hstmt));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug4518_p", 11);
+  is_str(my_fetch_str(hstmt, buff, 4), "id", 2);
+  is_str(my_fetch_str(hstmt, buff, 7), "t_bug4518_c", 11);
+  is_str(my_fetch_str(hstmt, buff, 8), "parent_id", 9);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug4518_c, t_bug4518_c2, t_bug4518_p");
+  ok_sql(hstmt,
+         "DROP TABLE t_bug4518_c, t_bug4518_c2, t_bug4518ac, t_bug4518_p");
 
   return OK;
 }
 
 
+/**
+ Tests the non-error code paths in catalog.c that return an empty set to
+ make sure the resulting empty result sets at least indicate the right
+ number of columns.
+*/
+DECLARE_TEST(empty_set)
+{
+  SQLSMALLINT columns;
+
+  /* SQLTables(): no known table types. */
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS,
+                           (SQLCHAR *)"UNKNOWN", SQL_NTS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &columns));
+  is_num(columns, 5);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* SQLTables(): no tables found. */
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
+                           (SQLCHAR *)"no_such_table", SQL_NTS, NULL, SQL_NTS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &columns));
+  is_num(columns, 5);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+
+/**
+ Bug #23031: SQLTables returns inaccurate catalog information on views
+*/
+DECLARE_TEST(t_bug23031)
+{
+  SQLCHAR buff[255];
+
+  ok_sql(hstmt, "DROP VIEW IF EXISTS t_bug23031_v");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug23031_t");
+  ok_sql(hstmt, "CREATE TABLE t_bug23031_t (a INT) COMMENT 'Whee!'");
+  ok_sql(hstmt, "CREATE VIEW t_bug23031_v AS SELECT * FROM t_bug23031_t");
+
+  /* Get both the table and view. */
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
+                           (SQLCHAR *)"t_bug23031%", SQL_NTS, NULL, SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug23031_t", 12);
+  is_str(my_fetch_str(hstmt, buff, 4), "TABLE", 5);
+  is_str(my_fetch_str(hstmt, buff, 5), "Whee!", 5);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug23031_v", 12);
+  is_str(my_fetch_str(hstmt, buff, 4), "VIEW", 4);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Get just the table. */
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
+                           (SQLCHAR *)"t_bug23031%", SQL_NTS,
+                           (SQLCHAR *)"TABLE", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug23031_t", 12);
+  is_str(my_fetch_str(hstmt, buff, 4), "TABLE", 5);
+  is_str(my_fetch_str(hstmt, buff, 5), "Whee!", 5);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Get just the view. */
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
+                           (SQLCHAR *)"t_bug23031%", SQL_NTS,
+                           (SQLCHAR *)"'VIEW'", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug23031_v", 12);
+  is_str(my_fetch_str(hstmt, buff, 4), "VIEW", 4);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP VIEW IF EXISTS t_bug23031_v");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug23031_t");
+
+  return OK;
+}
+
+
+/**
+ Bug #15713: null pointer when use the table qualifier in SQLColumns()
+*/
+DECLARE_TEST(bug15713)
+{
+  HDBC hdbc1;
+  HSTMT hstmt1;
+  SQLCHAR   conn[256], conn_out[256];
+  SQLSMALLINT conn_out_len;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15713");
+  ok_sql(hstmt, "CREATE TABLE t_bug15713 (a INT)");
+
+  /* The connection strings must not include DATABASE. */
+  sprintf((char *)conn, "DSN=%s;UID=%s;PASSWORD=%s", mydsn, myuid, mypwd);
+  if (mysock != NULL)
+  {
+    strcat((char *)conn, ";SOCKET=");
+    strcat((char *)conn, (char *)mysock);
+  }
+
+  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
+
+  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
+                                 sizeof(conn_out), &conn_out_len,
+                                 SQL_DRIVER_NOPROMPT));
+  ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+
+  ok_stmt(hstmt1, SQLColumns(hstmt1, (SQLCHAR *)"test", SQL_NTS,
+                             NULL, 0, (SQLCHAR *)"t_bug15713", SQL_NTS,
+                             NULL, 0));
+
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_str(my_fetch_str(hstmt1, conn, 1), "test", 4);
+  is_str(my_fetch_str(hstmt1, conn, 3), "t_bug15713", 10);
+  is_str(my_fetch_str(hstmt1, conn, 4), "a", 1);
+
+  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
+  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15713");
+  return OK;
+}
+
+
+/**
+ Bug #28316: Fatal crash with Crystal Reports and MySQL Server
+*/
+DECLARE_TEST(t_bug28316)
+{
+  ok_stmt(hstmt, SQLProcedures(hstmt, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS));
+
+  return OK;
+}
+
+
+/**
+ Bug #8860: Generic SQLColumns not supported?
+*/
+DECLARE_TEST(bug8860)
+{
+  SQLCHAR buff[512];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug8860, `t_bug8860_a'b`");
+  ok_sql(hstmt, "CREATE TABLE t_bug8860 (a INT)");
+  ok_sql(hstmt, "CREATE TABLE `t_bug8860_a'b` (b INT)");
+
+  /*
+   Specifying nothing gets us columns from all of the tables in the
+   current database.
+  */
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0));
+
+  /* We should have at least two rows. There may be more. */
+  is(myrowcount(hstmt) >= 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Specifying "" as the table name gets us nothing. */
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, (SQLCHAR *)"", SQL_NTS,
+                            NULL, 0));
+
+  is_num(myrowcount(hstmt), 0);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Get the info from just one table.  */
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, "t_bug8860", SQL_NTS,
+                            NULL, 0));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug8860", 9);
+  is_str(my_fetch_str(hstmt, buff, 4), "a", 1);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Get the info from just one table with a funny name.  */
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, "t_bug8860_a'b", SQL_NTS,
+                            NULL, 0));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug8860_a'b", 13);
+  is_str(my_fetch_str(hstmt, buff, 4), "b", 1);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "DROP TABLE t_bug8860, `t_bug8860_a'b`");
+  return OK;
+}
+
+/**
+ Bug #26934: SQLTables behavior has changed
+*/
+DECLARE_TEST(t_bug26934)
+{
+  HENV henv1;
+  HDBC hdbc1;
+  HSTMT hstmt1;
+
+  alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  ok_sql(hstmt1, "SET @@wait_timeout = 1");
+  sleep(2);
+  expect_stmt(hstmt1, SQLTables(hstmt1, "%", 1, NULL, SQL_NTS, NULL, SQL_NTS,
+                                NULL, SQL_NTS), SQL_ERROR);
+  if (check_sqlstate(hstmt1, "08S01") != OK)
+    return FAIL;
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
+
+/**
+ Bug #29888: Crystal wizard throws error on including tables
+*/
+DECLARE_TEST(t_bug29888)
+{
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug29888");
+  ok_sql(hstmt, "CREATE TABLE t_bug29888 (a INT, b INT)");
+
+  ok_stmt(hstmt, SQLColumns(hstmt, mydb, SQL_NTS, NULL, SQL_NTS,
+                            (SQLCHAR *)"t_bug29888", SQL_NTS,
+                            (SQLCHAR *)"%", SQL_NTS));
+
+  is_num(myrowcount(hstmt), 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug29888");
+
+  return OK;
+}
+
+
+/**
+  Bug #14407: SQLColumns gives wrong information of not nulls
+*/
+DECLARE_TEST(t_bug14407)
+{
+  SQLCHAR col[10];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug14407");
+  ok_sql(hstmt,
+         "CREATE TABLE t_bug14407(a INT NOT NULL AUTO_INCREMENT PRIMARY KEY)");
+
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"t_bug14407", SQL_NTS, NULL, 0));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, col, 4), "a", 1);
+  is_num(my_fetch_int(hstmt, 11), SQL_NULLABLE);
+  is_str(my_fetch_str(hstmt, col, 18), "YES", 3);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug14407");
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_columns_null)
   ADD_TEST(my_drop_table)
-  ADD_TEST(my_table_priv)
   ADD_TEST(my_table_dbs)
-  ADD_TEST(my_tablepriv)
-  ADD_TEST(my_tablepriv_data)
-  ADD_TEST(my_column_priv)
   ADD_TEST(my_colpriv)
   ADD_TEST(t_sqlprocedures)
   ADD_TEST(t_catalog)
@@ -1198,6 +1141,14 @@ BEGIN_TESTS
   ADD_TEST(tmysql_showkeys)
   ADD_TEST(t_sqltables)
   ADD_TEST(t_bug4518)
+  ADD_TEST(empty_set)
+  ADD_TEST(t_bug23031)
+  ADD_TEST(bug15713)
+  ADD_TEST(t_bug28316)
+  ADD_TEST(bug8860)
+  ADD_TEST(t_bug26934)
+  ADD_TEST(t_bug29888)
+  ADD_TEST(t_bug14407)
 END_TESTS
 
 
