@@ -121,6 +121,10 @@ DECLARE_TEST(my_setpos_cursor)
   ok_sql(hstmt, "INSERT INTO my_demo_cursor VALUES (0,'MySQL0'),(1,'MySQL1'),"
          "(2,'MySQL2'),(3,'MySQL3'),(4,'MySQL4')");
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
   ok_sql(hstmt, "SELECT * FROM my_demo_cursor");
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &id, 0, NULL));
@@ -272,6 +276,8 @@ DECLARE_TEST(t_setpos_del_all)
          "(200,'MySQL2'),(300,'MySQL3'),(400,'MySQL4')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu", SQL_NTS));
 
@@ -359,7 +365,7 @@ DECLARE_TEST(t_setpos_position)
   SQLINTEGER nData;
   SQLLEN nlen;
   SQLCHAR szData[255];
-  SQLUINTEGER pcrow;
+  SQLULEN pcrow;
   SQLUSMALLINT rgfRowStatus;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_setpos_position");
@@ -380,18 +386,20 @@ DECLARE_TEST(t_setpos_position)
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &nData, 0, NULL));
   ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData),
-                            NULL));
+                            &nlen));
 
   ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 1, &pcrow,
                                   &rgfRowStatus));
 
   is_num(nData, 100);
+  is_num(nlen, 6);
   is_str(szData, "MySQL1", 6);
 
   ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
 
   nData= 1000;
   strcpy((char *)szData, "updated");
+  nlen= 7;
 
   expect_stmt(hstmt, SQLSetPos(hstmt, 3, SQL_UPDATE, SQL_LOCK_NO_CHANGE),
               SQL_ERROR);
@@ -853,7 +861,8 @@ DECLARE_TEST(t_acc_crash)
          "INSERT INTO t_acc_crash (b) VALUES ('venu'),('monty'),('mysql')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
   ok_stmt(hstmt, SQLSetStmtOption(hstmt, SQL_ROWSET_SIZE, 1));
 
   ok_sql(hstmt, "SELECT * FROM t_acc_crash ORDER BY a ASC");
@@ -908,6 +917,8 @@ DECLARE_TEST(tmysql_setpos_del)
          "(300,'MySQL6'),(300,'MySQL7')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu", SQL_NTS));
 
@@ -969,7 +980,7 @@ DECLARE_TEST(tmysql_setpos_del1)
   SQLINTEGER nData;
   SQLLEN nlen;
   SQLCHAR szData[255];
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_setpos_del1");
@@ -978,11 +989,10 @@ DECLARE_TEST(tmysql_setpos_del1)
          "(200,'MySQL2'),(300,'MySQL3'),(400,'MySQL4')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
   ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu", SQL_NTS));
 
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
   ok_sql(hstmt, "SELECT * FROM tmysql_setpos_del1");
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &nData, 0, NULL));
@@ -1030,7 +1040,7 @@ DECLARE_TEST(tmysql_setpos_upd)
     SQLINTEGER nData = 500;
     SQLLEN nlen;
     SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
+    SQLROWSETSIZE pcrow;
     SQLUSMALLINT rgfRowStatus;
 
     tmysql_exec(hstmt,"drop table tmysql_setpos");
@@ -1053,8 +1063,9 @@ DECLARE_TEST(tmysql_setpos_upd)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
     mystmt(hstmt,rc);
@@ -1138,7 +1149,7 @@ DECLARE_TEST(tmysql_setpos_add)
     SQLINTEGER nData= 500;
     SQLLEN nlen;
     SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
+    SQLROWSETSIZE pcrow;
     SQLUSMALLINT rgfRowStatus;
 
     tmysql_exec(hstmt,"drop table tmysql_setpos_add");
@@ -1229,72 +1240,58 @@ DECLARE_TEST(tmysql_setpos_add)
 
 DECLARE_TEST(tmysql_pos_delete)
 {
-    SQLRETURN rc;
-    SQLHSTMT hstmt1;
-    SQLUINTEGER pcrow;
-    SQLUSMALLINT rgfRowStatus;
+  SQLHSTMT hstmt1;
+  SQLLEN rows;
+  SQLCHAR buff[10];
 
-    rc = SQLAllocStmt(hdbc,&hstmt1);
-    mycon(hdbc,rc);
+  ok_con(hdbc, SQLAllocStmt(hdbc, &hstmt1));
 
-    tmysql_exec(hstmt,"drop table tmysql_pos_delete");
-    rc = tmysql_exec(hstmt,"create table tmysql_pos_delete(col1 int , col2 varchar(30))");
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_pos_delete");
+  ok_sql(hstmt, "CREATE TABLE tmysql_pos_delete (a INT, b VARCHAR(30))");
+  ok_sql(hstmt, "INSERT INTO tmysql_pos_delete VALUES (1,'venu'),(2,'MySQL')");
 
-    rc = tmysql_exec(hstmt,"insert into tmysql_pos_delete values(100,'venu')");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = tmysql_exec(hstmt,"insert into tmysql_pos_delete values(200,'MySQL')");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu_cur", SQL_NTS));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_sql(hstmt, "SELECT * FROM tmysql_pos_delete");
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 1, NULL, NULL));
 
-    rc = SQLSetCursorName(hstmt,"venu_cur",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
 
-    rc = tmysql_exec(hstmt,"select * from tmysql_pos_delete");
-    mystmt(hstmt,rc);
+  expect_sql(hstmt1,
+             "   DfffELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur",
+             SQL_ERROR);
 
-    rc = SQLExtendedFetch(hstmt,SQL_FETCH_NEXT,1,&pcrow,&rgfRowStatus);
-    mystmt(hstmt,rc);
+  expect_sql(hstmt1,
+             "   DELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur curs",
+             SQL_ERROR);
 
-    rc = SQLSetPos(hstmt,1,SQL_POSITION,SQL_LOCK_NO_CHANGE);
-    mystmt(hstmt,rc);
+  expect_sql(hstmt1,
+             "   DELETE FROM tmysql_pos_delete WHERE ONE CURRENT OF venu_cur",
+             SQL_ERROR);
 
-    rc = SQLExecDirect(hstmt1,"   DfffELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur",SQL_NTS);
-    mystmt_r(hstmt1,rc);
+  ok_sql(hstmt1, "   DELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur");
 
-    rc = SQLExecDirect(hstmt1,"   DELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur curs",SQL_NTS);
-    mystmt_r(hstmt1,rc);
+  ok_stmt(hstmt1, SQLRowCount(hstmt1, &rows));
+  is_num(rows, 1);
 
-    rc = SQLExecDirect(hstmt1,"   DELETE FROM tmysql_pos_delete WHERE ONE CURRENT OF venu_cur",SQL_NTS);
-    mystmt_r(hstmt1,rc);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
 
-    rc = SQLExecDirect(hstmt1,"   DELETE FROM tmysql_pos_delete WHERE CURRENT OF venu_cur",SQL_NTS);
-    mystmt(hstmt1,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    SQLNumResultCols(hstmt1,&rgfRowStatus);
+  ok_sql(hstmt, "SELECT * FROM tmysql_pos_delete");
+  ok_stmt(hstmt, SQLFetch(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    rc = SQLFreeStmt(hstmt1,SQL_CLOSE);
+  is_num(my_fetch_int(hstmt, 1), 2);
+  is_str(my_fetch_str(hstmt, buff, 2), "MySQL", 5);
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
-    rc = tmysql_exec(hstmt,"select * from tmysql_pos_delete");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    my_assert(1 == myresult(hstmt));
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLFreeStmt(hstmt1,SQL_DROP);
-    mystmt(hstmt1,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_pos_delete");
 
   return OK;
 }
@@ -1310,6 +1307,8 @@ DECLARE_TEST(t_pos_update)
   ok_sql(hstmt, "INSERT INTO t_pos_update VALUES (100,'venu'),(200,'MySQL')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu_cur", SQL_NTS));
 
@@ -1363,7 +1362,7 @@ DECLARE_TEST(t_pos_update)
 DECLARE_TEST(tmysql_pos_update_ex)
 {
   SQLHSTMT hstmt1;
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
   SQLLEN rows;
   SQLCHAR cursor[30], sql[255], data[]= "tmysql_pos_update_ex";
@@ -1373,6 +1372,8 @@ DECLARE_TEST(tmysql_pos_update_ex)
   ok_sql(hstmt, "INSERT INTO t_pos_updex VALUES (100,'venu'),(200,'MySQL')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_sql(hstmt, "SELECT * FROM t_pos_updex");
 
@@ -1424,7 +1425,7 @@ DECLARE_TEST(tmysql_pos_update_ex)
 DECLARE_TEST(tmysql_pos_update_ex1)
 {
   SQLHSTMT hstmt1;
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLLEN      rows;
   SQLSMALLINT rgfRowStatus;
   SQLCHAR cursor[30], sql[100], data[]= "tmysql_pos_update_ex1";
@@ -1433,6 +1434,10 @@ DECLARE_TEST(tmysql_pos_update_ex1)
   ok_sql(hstmt, "CREATE TABLE t_pos_updex1  (a INT, b VARCHAR(30))");
   ok_sql(hstmt, "INSERT INTO t_pos_updex1 VALUES (100,'venu'),(200,'MySQL')");
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
   ok_sql(hstmt, "SELECT * FROM t_pos_updex1");
 
   ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_ABSOLUTE, 2, &pcrow,
@@ -1478,72 +1483,10 @@ DECLARE_TEST(tmysql_pos_update_ex1)
 }
 
 
-DECLARE_TEST(tmysql_pos_update_ex2)
-{
-  SQLHSTMT hstmt1;
-  SQLROWSETSIZE pcrow;
-  SQLLEN nlen= SQL_NTS;
-  SQLUSMALLINT rgfRowStatus;
-  SQLCHAR cursor[30], sql[255], data[]= "updated";
-
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_pos_updex2");
-  ok_sql(hstmt, "CREATE TABLE t_pos_updex2 (a INT NOT NULL, b VARCHAR(30),"
-         "c INT NOT NULL, PRIMARY KEY(a,c))");
-  ok_sql(hstmt, "INSERT INTO t_pos_updex2 VALUES (10,'venu',1),(20,'MySQL',2)");
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  ok_sql(hstmt,  "SELECT a, b FROM t_pos_updex2");
-
-  ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_ABSOLUTE, 2, &pcrow,
-                                  &rgfRowStatus));
-  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
-
-  ok_stmt(hstmt, SQLGetCursorName(hstmt, cursor, sizeof(cursor), NULL));
-
-  ok_con(hdbc, SQLAllocStmt(hdbc, &hstmt1));
-
-  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
-                                   SQL_CHAR, 0, 0, data, sizeof(data), NULL));
-
-  sprintf((char *)sql,
-          "UPDATE t_pos_updex2 SET a = 999, b = ? WHERE CURRENT OF %s", cursor);
-
-  ok_stmt(hstmt1, SQLExecDirect(hstmt1, sql, SQL_NTS));
-
-  ok_stmt(hstmt1, SQLRowCount(hstmt1, &nlen));
-  is_num(nlen, 1);
-
-  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  ok_sql(hstmt, "SELECT * FROM t_pos_updex2");
-
-  ok_stmt(hstmt, SQLFetch(hstmt));
-  is_num(my_fetch_int(hstmt, 1), 10);
-  is_str(my_fetch_str(hstmt, sql, 2), "venu", 4);
-  is_num(my_fetch_int(hstmt, 3), 1);
-
-  ok_stmt(hstmt, SQLFetch(hstmt));
-  is_num(my_fetch_int(hstmt, 1), 999);
-  is_str(my_fetch_str(hstmt, sql, 2), "updated", 7);
-  is_num(my_fetch_int(hstmt, 3), 2);
-
-  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  ok_sql(hstmt, "DROP TABLE IF EXISTS t_pos_updex2");
-
-  return OK;
-}
-
-
 DECLARE_TEST(tmysql_pos_update_ex3)
 {
   SQLHSTMT hstmt1;
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
   SQLCHAR cursor[30], sql[255];
 
@@ -1553,6 +1496,8 @@ DECLARE_TEST(tmysql_pos_update_ex3)
   ok_sql(hstmt, "INSERT INTO t_pos_updex3 VALUES (100,'venu'),(200,'MySQL')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_sql(hstmt,  "SELECT a, b FROM t_pos_updex3");
 
@@ -1581,7 +1526,7 @@ DECLARE_TEST(tmysql_pos_update_ex3)
 
 DECLARE_TEST(tmysql_pos_update_ex4)
 {
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLLEN nlen= SQL_NTS;
   SQLCHAR data[]= "venu", szData[20];
   SQLUSMALLINT rgfRowStatus;
@@ -1592,6 +1537,8 @@ DECLARE_TEST(tmysql_pos_update_ex4)
   ok_sql(hstmt, "INSERT INTO t_pos_updex4 (a,b) VALUES ('Monty','Widenius')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_sql(hstmt, "SELECT * FROM t_pos_updex4");
 
@@ -1628,7 +1575,7 @@ DECLARE_TEST(tmysql_pos_update_ex4)
 DECLARE_TEST(tmysql_pos_dyncursor)
 {
   SQLHSTMT hstmt1;
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
   SQLCHAR buff[100];
   SQLLEN rows;
@@ -1638,6 +1585,8 @@ DECLARE_TEST(tmysql_pos_dyncursor)
   ok_sql(hstmt, "INSERT INTO tmysql_pos_dyncursor VALUES (1,'foo'),(2,'bar')");
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu_cur", SQL_NTS));
 
@@ -1686,7 +1635,7 @@ DECLARE_TEST(tmysql_mtab_setpos_del)
     SQLINTEGER nData= 500;
     SQLLEN nlen;
     SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
+    SQLROWSETSIZE pcrow;
     SQLUSMALLINT rgfRowStatus;
 
     tmysql_exec(hstmt,"drop table tmysql_t1");
@@ -1714,8 +1663,9 @@ DECLARE_TEST(tmysql_mtab_setpos_del)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
     mystmt(hstmt,rc);
@@ -1757,7 +1707,7 @@ DECLARE_TEST(tmysql_setpos_pkdel)
     SQLINTEGER nData= 500;
     SQLLEN nlen;
     SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
+    SQLROWSETSIZE pcrow;
     SQLUSMALLINT rgfRowStatus;
 
     tmysql_exec(hstmt,"drop table tmysql_setpos1");
@@ -1780,8 +1730,9 @@ DECLARE_TEST(tmysql_setpos_pkdel)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
     mystmt(hstmt,rc);
@@ -1836,7 +1787,7 @@ DECLARE_TEST(t_alias_setpos_pkdel)
   SQLINTEGER nData= 500;
   SQLLEN nlen;
   SQLCHAR szData[255]= {0};
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_alias_setpos_pkdel");
@@ -1855,6 +1806,8 @@ DECLARE_TEST(t_alias_setpos_pkdel)
   ok_con(hdbc, SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT));
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, "venu", SQL_NTS));
 
@@ -1897,7 +1850,7 @@ DECLARE_TEST(t_alias_setpos_del)
   SQLINTEGER nData= 500;
   SQLLEN nlen;
   SQLCHAR szData[255]= {0};
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_alias_setpos_del");
@@ -1914,6 +1867,8 @@ DECLARE_TEST(t_alias_setpos_del)
   ok_con(hdbc, SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT));
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_stmt(hstmt, SQLSetCursorName(hstmt, "venu", SQL_NTS));
 
@@ -1951,231 +1906,69 @@ DECLARE_TEST(t_alias_setpos_del)
 }
 
 
-DECLARE_TEST(tmysql_setpos_pkdel1)
-{
-    SQLRETURN rc;
-    SQLINTEGER nData= 500;
-    SQLLEN nlen;
-    SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
-    SQLUSMALLINT rgfRowStatus;
-
-    tmysql_exec(hstmt,"drop table tmysql_setpos1");
-
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
-
-    rc = tmysql_exec(hstmt,"create table tmysql_setpos1(col1 int primary key, col3 int,col2 varchar(30))");
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(100,10,'MySQL1')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(200,20,'MySQL2')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(300,20,'MySQL3')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(400,20,'MySQL4')");
-    mystmt(hstmt,rc);
-
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"select col2,col3 from tmysql_setpos1");
-    mystmt(hstmt,rc);
-
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&nData,100,NULL);
-    mystmt(hstmt,rc);
-
-    rc = SQLBindCol(hstmt,2,SQL_C_CHAR,szData,100,&nlen);
-    mystmt(hstmt,rc);
-
-    rc = SQLExtendedFetch(hstmt,SQL_FETCH_ABSOLUTE,4,&pcrow,&rgfRowStatus);
-    mystmt(hstmt,rc);
-
-    printMessage(" pcrow:%d\n",pcrow);
-
-    printMessage(" row1:%d,%s\n",nData,szData);
-
-    rc = SQLSetPos(hstmt,1,SQL_POSITION,SQL_LOCK_NO_CHANGE);
-    mystmt(hstmt,rc);
-
-    rc = SQLSetPos(hstmt,1,SQL_DELETE,SQL_LOCK_NO_CHANGE);
-    mystmt(hstmt,rc);
-
-    rc = SQLRowCount(hstmt,&nlen);
-    mystmt(hstmt,rc);
-
-    printMessage(" rows affected:%d\n",nlen);
-
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"select * from tmysql_setpos1");
-    mystmt(hstmt,rc);
-
-    my_assert( 3 == myresult(hstmt));
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-  return OK;
-}
-
-
 DECLARE_TEST(tmysql_setpos_pkdel2)
 {
-    SQLRETURN rc;
-    SQLINTEGER nData= 500;
-    SQLLEN nlen;
-    SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
-    SQLUSMALLINT rgfRowStatus;
+  SQLINTEGER nData= 500;
+  SQLLEN nlen;
+  SQLCHAR szData[255]= {0};
+  SQLROWSETSIZE pcrow;
+  SQLUSMALLINT rgfRowStatus;
 
-    tmysql_exec(hstmt,"drop table tmysql_setpos1");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_setpos_pkdel2");
+  ok_sql(hstmt, "CREATE TABLE tmysql_setpos_pkdel2 (a INT, b INT,"
+         "c VARCHAR(30) PRIMARY KEY)");
+  ok_sql(hstmt, "INSERT INTO tmysql_setpos_pkdel2 VALUES (100,10,'MySQL1'),"
+         "(200,20,'MySQL2'),(300,30,'MySQL3'),(400,40,'MySQL4')");
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = tmysql_exec(hstmt,"create table tmysql_setpos1(col1 int, col3 int,col2 varchar(30) primary key)");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(100,10,'MySQL1')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(200,20,'MySQL2')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(300,20,'MySQL3')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(400,20,'MySQL4')");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"venu", SQL_NTS));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_sql(hstmt, "SELECT b,c FROM tmysql_setpos_pkdel2");
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData),
+                            &nlen));
 
-    rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_ABSOLUTE, 4,
+                                  &pcrow, &rgfRowStatus));
+  is_num(pcrow, 1);
+  is_num(nData, 40);
+  is_str(szData, "MySQL4", 6);
 
-    rc = tmysql_exec(hstmt,"select col2,col3 from tmysql_setpos1");
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_DELETE, SQL_LOCK_NO_CHANGE));
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&nData,100,NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLRowCount(hstmt, &nlen));
+  is_num(nlen, 1);
 
-    rc = SQLBindCol(hstmt,2,SQL_C_CHAR,szData,100,&nlen);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLExtendedFetch(hstmt,SQL_FETCH_ABSOLUTE,4,&pcrow,&rgfRowStatus);
-    mystmt(hstmt,rc);
-    printMessage(" pcrow:%d\n",pcrow);
+  ok_sql(hstmt, "SELECT * FROM tmysql_setpos_pkdel2");
 
-    printMessage(" row1:%d,%s\n",nData,szData);
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 100);
+  is_num(my_fetch_int(hstmt, 2), 10);
+  is_str(my_fetch_str(hstmt, szData, 3), "MySQL1", 6);
 
-    rc = SQLSetPos(hstmt,1,SQL_DELETE,SQL_LOCK_NO_CHANGE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 200);
+  is_num(my_fetch_int(hstmt, 2), 20);
+  is_str(my_fetch_str(hstmt, szData, 3), "MySQL2", 6);
 
-    rc = SQLRowCount(hstmt,&nlen);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 300);
+  is_num(my_fetch_int(hstmt, 2), 30);
+  is_str(my_fetch_str(hstmt, szData, 3), "MySQL3", 6);
 
-    printMessage(" rows affected:%d\n",nlen);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"select * from tmysql_setpos1");
-    mystmt(hstmt,rc);
-
-    my_assert( 3 == myresult(hstmt));
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-  return OK;
-}
-
-
-DECLARE_TEST(tmysql_setpos_pkdel3)
-{
-    SQLRETURN rc;
-    SQLINTEGER nData= 500;
-    SQLLEN nlen;
-    SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
-    SQLUSMALLINT rgfRowStatus;
-
-    tmysql_exec(hstmt,"drop table tmysql_setpos1");
-
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
-
-    rc = tmysql_exec(hstmt,"create table tmysql_setpos1(col1 int, col3 int,col2 varchar(30) primary key)");
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(100,10,'MySQL1')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(200,20,'MySQL2')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(300,20,'MySQL3')");
-    mystmt(hstmt,rc);
-    rc = tmysql_exec(hstmt,"insert into tmysql_setpos1 values(400,20,'MySQL4')");
-    mystmt(hstmt,rc);
-
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"select col1 from tmysql_setpos1");
-    mystmt(hstmt,rc);
-
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&nData,100,NULL);
-    mystmt(hstmt,rc);
-
-    rc = SQLExtendedFetch(hstmt,SQL_FETCH_ABSOLUTE,4,&pcrow,&rgfRowStatus);
-    mystmt(hstmt,rc);
-
-    printMessage(" pcrow:%d\n",pcrow);
-
-    printMessage(" row1:%d,%s\n",nData,szData);
-
-    rc = SQLSetPos(hstmt,1,SQL_DELETE,SQL_LOCK_NO_CHANGE);
-    mystmt(hstmt,rc);
-
-    rc = SQLRowCount(hstmt,&nlen);
-    mystmt(hstmt,rc);
-
-    printMessage(" rows affected:%d\n",nlen);
-
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = tmysql_exec(hstmt,"select * from tmysql_setpos1");
-    mystmt(hstmt,rc);
-
-    my_assert( 3 == myresult(hstmt));
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_setpos_pkdel2");
 
   return OK;
 }
@@ -2186,7 +1979,8 @@ DECLARE_TEST(t_setpos_upd_bug1)
     SQLRETURN rc;
     SQLINTEGER id;
     SQLLEN len,id_len,f_len,l_len,ts_len;
-    SQLCHAR fname[21],lname[21],ts[17],szTable[256];
+    SQLCHAR fname[21],lname[21],szTable[256];
+    SQL_TIMESTAMP_STRUCT ts;
     SQLSMALLINT pccol;
     SQLUSMALLINT rgfRowStatus;
 
@@ -2208,8 +2002,9 @@ DECLARE_TEST(t_setpos_upd_bug1)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = tmysql_exec(hstmt,"select * from t_setpos_upd_bug1 order by id asc");
     mystmt(hstmt,rc);
@@ -2228,10 +2023,10 @@ DECLARE_TEST(t_setpos_upd_bug1)
     rc = SQLBindCol(hstmt,3,SQL_C_CHAR,lname,20,&l_len);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,4,SQL_C_TIMESTAMP,ts,21,&ts_len);
+    rc = SQLBindCol(hstmt,4,SQL_C_TIMESTAMP,&ts,21,&ts_len);
     mystmt(hstmt,rc);
 
-    rc = SQLColAttributes(hstmt,1,SQL_COLUMN_TABLE_NAME,szTable,sizeof(szTable),NULL,NULL);
+    rc = SQLColAttribute(hstmt,1,SQL_COLUMN_TABLE_NAME,szTable,sizeof(szTable),NULL,NULL);
     mystmt(hstmt,rc);
 
     rc = SQLExtendedFetch(hstmt,SQL_FETCH_FIRST,0,NULL,&rgfRowStatus);
@@ -2297,7 +2092,7 @@ DECLARE_TEST(my_setpos_upd_pk_order)
     SQLINTEGER nData= 500;
     SQLLEN nlen;
     SQLCHAR szData[255]={0};
-    SQLUINTEGER pcrow;
+    SQLROWSETSIZE pcrow;
     SQLUSMALLINT rgfRowStatus;
 
     tmysql_exec(hstmt,"drop table my_setpos_upd_pk_order");
@@ -2312,8 +2107,9 @@ DECLARE_TEST(my_setpos_upd_pk_order)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = SQLSetCursorName(hstmt,"venu",SQL_NTS);
     mystmt(hstmt,rc);
@@ -2381,7 +2177,7 @@ DECLARE_TEST(my_setpos_upd_pk_order1)
 {
   SQLINTEGER nData;
   SQLCHAR szData[255];
-  SQLUINTEGER pcrow;
+  SQLROWSETSIZE pcrow;
   SQLUSMALLINT rgfRowStatus;
   SQLLEN rows;
 
@@ -2544,15 +2340,9 @@ DECLARE_TEST(tmysql_pcbvalue)
 
   ok_con(hdbc, SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT));
 
-  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
-
-#if 0
-  /* MS SQL Server to work...*/
-  SQLSetStmtAttr(hstmt, SQL_ATTR_CONCURRENCY,
-                 (SQLPOINTER)SQL_CONCUR_ROWVER, 0);
-  SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
-                 (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN, 0);
-#endif
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
   ok_sql(hstmt,"SELECT * FROM tmysql_pcbvalue");
 
@@ -2606,6 +2396,191 @@ DECLARE_TEST(tmysql_pcbvalue)
 }
 
 
+/**
+ Bug #28255: Cursor operations on result sets containing only part of a key
+ are incorrect
+*/
+DECLARE_TEST(t_bug28255)
+{
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug28255");
+  ok_sql(hstmt, "CREATE TABLE t_bug28255 (a INT, b INT, PRIMARY KEY (a,b))");
+  ok_sql(hstmt, "INSERT INTO t_bug28255 VALUES (1,3),(1,4),(1,5)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"bug", SQL_NTS));
+
+  ok_sql(hstmt, "SELECT a FROM t_bug28255 WHERE b > 3");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
+  expect_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_DELETE, SQL_LOCK_NO_CHANGE),
+              SQL_ERROR);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "SELECT * FROM t_bug28255");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+  is_num(my_fetch_int(hstmt, 2), 3);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+  is_num(my_fetch_int(hstmt, 2), 4);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+  is_num(my_fetch_int(hstmt, 2), 5);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug28255");
+
+  return OK;
+}
+
+
+/**
+ Bug #10563: Update using multicolumn primary key with duplicate indexes fails
+*/
+DECLARE_TEST(bug10563)
+{
+  SQLLEN nlen;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug10563");
+  ok_sql(hstmt, "CREATE TABLE t_bug10563 (a INT, b INT, PRIMARY KEY (a,b), UNIQUE (b))");
+  ok_sql(hstmt, "INSERT INTO t_bug10563 VALUES (1,3),(1,4)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetCursorName(hstmt, (SQLCHAR *)"bug", SQL_NTS));
+
+  ok_sql(hstmt, "SELECT b FROM t_bug10563 WHERE b > 3");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 4);
+
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_DELETE, SQL_LOCK_NO_CHANGE));
+
+  ok_stmt(hstmt, SQLRowCount(hstmt, &nlen));
+  is_num(nlen, 1);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "SELECT * FROM t_bug10563");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1);
+  is_num(my_fetch_int(hstmt, 2), 3);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug10563");
+
+  return OK;
+}
+
+
+/*
+ * Bug 6741 - SQL_ATTR_ROW_BIND_OFFSET_PTR is not supported
+ * It was supported for use in some batch operations, but not
+ * standard cursor operations.
+ */
+#define BUG6741_VALS 5
+
+DECLARE_TEST(bug6741)
+{
+  int i;
+  SQLLEN offset;
+  struct {
+    SQLINTEGER xval;
+    SQLLEN ylen;
+  } results[BUG6741_VALS];
+
+  ok_sql(hstmt, "drop table if exists t_bug6741");
+  ok_sql(hstmt, "create table t_bug6741 (x int, y int)");
+
+  ok_sql(hstmt, "insert into t_bug6741 values (0,0),(1,NULL),(2,2),(3,NULL),(4,4)");
+  ok_sql(hstmt, "select x,y from t_bug6741 order by x");
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_BIND_OFFSET_PTR,
+          &offset, SQL_IS_UINTEGER));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &results[0].xval, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_LONG, NULL, 0, &results[0].ylen));
+
+  /* fetch all the data */
+  for(i = 0; i < BUG6741_VALS; ++i)
+  {
+    offset = i * sizeof(results[0]);
+    ok_stmt(hstmt, SQLFetch(hstmt));
+  }
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  /* verify it */
+  for(i = 0; i < BUG6741_VALS; ++i)
+  {
+    printf("xval[%d] = %d\n", i, results[i].xval);
+    printf("ylen[%d] = %ld\n", i, results[i].ylen);
+    is_num(results[i].xval, i);
+    if(i % 2)
+    {
+      is_num(results[i].ylen, SQL_NULL_DATA);
+    }
+    else
+    {
+      is_num(results[i].ylen, sizeof(SQLINTEGER));
+    }
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists t_bug6741");
+
+  return OK;
+}
+
+
+/** Test SQL_POSITION */
+DECLARE_TEST(t_chunk)
+{
+  SQLCHAR   txt[100];
+  SQLLEN    len;
+
+  if (!driver_supports_setpos(hdbc))
+    skip("driver doesn't support setpos");
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_chunk");
+  ok_sql(hstmt, "CREATE TABLE t_chunk (id int not null primary key,"
+         "description varchar(50), txt text)");
+  ok_sql(hstmt, "INSERT INTO t_chunk VALUES (1,'venu','Developer, MySQL AB'),"
+         "(2,'monty','Michael Monty Widenius - main MySQL developer'),"
+         "(3,'mysql','MySQL AB- Speed, Power and Precision')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "SELECT * from t_chunk");
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 1));
+
+  ok_stmt(hstmt, SQLSetPos(hstmt, 1, SQL_POSITION, SQL_LOCK_NO_CHANGE));
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_CHAR, txt, 100, &len));
+  is_str(txt, "Developer, MySQL AB", 19);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_chunk");
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_positioned_cursor)
   ADD_TEST(my_setpos_cursor)
@@ -2627,15 +2602,12 @@ BEGIN_TESTS
   ADD_TEST(t_pos_update)
   ADD_TEST(tmysql_pos_update_ex)
   ADD_TEST(tmysql_pos_update_ex1)
-  ADD_TEST(tmysql_pos_update_ex2)
   ADD_TEST(tmysql_pos_update_ex3)
   ADD_TEST(tmysql_pos_update_ex4)
   ADD_TEST(tmysql_pos_dyncursor)
   ADD_TEST(tmysql_mtab_setpos_del)
   ADD_TEST(tmysql_setpos_pkdel)
-  ADD_TEST(tmysql_setpos_pkdel1)
   ADD_TEST(tmysql_setpos_pkdel2)
-  ADD_TEST(tmysql_setpos_pkdel3)
   ADD_TEST(t_alias_setpos_pkdel)
   ADD_TEST(t_alias_setpos_del)
   ADD_TEST(t_setpos_upd_bug1)
@@ -2645,6 +2617,10 @@ BEGIN_TESTS
   ADD_TEST(tmy_cursor2)
   ADD_TEST(tmy_cursor3)
   ADD_TEST(tmysql_pcbvalue)
+  ADD_TEST(t_bug28255)
+  ADD_TEST(bug10563)
+  ADD_TEST(bug6741)
+  ADD_TEST(t_chunk)
 END_TESTS
 
 
