@@ -42,6 +42,9 @@ DECLARE_TEST(my_ts)
   ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_RESET_PARAMS));
   ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN, 0));
+
   /* insert using SQL_C_TIMESTAMP to SQL_TIMESTAMP */
   ts.year= 2002;
   ts.month= 1;
@@ -228,11 +231,9 @@ DECLARE_TEST(t_tstotime1)
 
 DECLARE_TEST(t_bug25846)
 {
-  SQLINTEGER  narg;
-
   SQLSMALLINT          column_count;
-  SQLINTEGER           my_time_cb;
-  SQLINTEGER           my_date_cb;
+  SQLLEN               my_time_cb;
+  SQLLEN               my_date_cb;
   SQL_TIMESTAMP_STRUCT my_time_ts;
   SQL_TIMESTAMP_STRUCT my_date_ts;
 
@@ -611,9 +612,8 @@ DECLARE_TEST(t_time1)
 */
 DECLARE_TEST(t_bug12520)
 {
-  SQLINTEGER my_time_cb;
   SQL_TIMESTAMP_STRUCT my_time_ts;
-  SQLLEN len;
+  SQLLEN len, my_time_cb;
   SQLCHAR datetime[50];
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug12520");
@@ -627,7 +627,7 @@ DECLARE_TEST(t_bug12520)
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, datetime, sizeof(datetime),
                             &len));
-  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_TIMESTAMP, &my_time_ts, NULL,
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_TIMESTAMP, &my_time_ts, 0,
                             &my_time_cb));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -643,6 +643,207 @@ DECLARE_TEST(t_bug12520)
   return OK;
 }
 
+/**
+ Bug #15773: Wrong between results
+*/
+DECLARE_TEST(t_bug15773)
+{
+  SQL_DATE_STRUCT a,b,c,d;
+  SQLLEN len1;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15773");
+  ok_sql(hstmt, "CREATE TABLE t_bug15773("
+         "`a` varchar(255) NOT NULL default '',"
+         "`b` datetime NOT NULL default '0000-00-00 00:00:00',"
+         "`c` datetime NOT NULL default '0000-00-00 00:00:00'"
+         ") ENGINE=InnoDB DEFAULT CHARSET=latin1");
+  ok_sql(hstmt, "INSERT INTO t_bug15773 VALUES ('a', '2005-12-24 00:00:00', '2008-05-12 00:00:00')");
+  ok_sql(hstmt, "INSERT INTO t_bug15773 VALUES ('b', '2004-01-01 00:00:00', '2005-01-01 00:00:00')");
+  ok_sql(hstmt, "INSERT INTO t_bug15773 VALUES ('c', '2004-12-12 00:00:00', '2005-12-12 00:00:00')");
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"SELECT * FROM t_bug15773"
+                           " WHERE (?) BETWEEN b AND c", SQL_NTS));
+
+  d.day= 15;
+  d.month= 12;
+  d.year = 2005;
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_DATE,
+                                  SQL_TYPE_DATE, 0, 0, &d, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, &a, 255, &len1));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_TYPE_DATE, &b, 0, &len1));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 3, SQL_C_TYPE_DATE, &c, 0, &len1));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15773");
+  return OK;
+}
+
+
+/**
+ Bug #9927: Updating datetime columns
+*/
+DECLARE_TEST(t_bug9927)
+{
+  SQLCHAR col[10];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug9927");
+  ok_sql(hstmt,
+         "CREATE TABLE t_bug9927 (a TIMESTAMP DEFAULT 0,"
+        "b TIMESTAMP ON UPDATE CURRENT_TIMESTAMP)");
+
+  ok_stmt(hstmt, SQLSpecialColumns(hstmt,SQL_ROWVER,  NULL, 0,
+                                   NULL, 0, (SQLCHAR *)"t_bug9927", SQL_NTS,
+                                   0, SQL_NO_NULLS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, col, 2), "b", 1);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug9927");
+
+  return OK;
+}
+
+
+/**
+ Bug #30081: Can't distinguish between auto-set TIMESTAMP and auto-updated
+ TIMESTAMP
+*/
+DECLARE_TEST(t_bug30081)
+{
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug30081");
+  ok_sql(hstmt,
+         "CREATE TABLE t_bug30081 (a TIMESTAMP DEFAULT 0,"
+        "b TIMESTAMP DEFAULT CURRENT_TIMESTAMP)");
+
+  ok_stmt(hstmt, SQLSpecialColumns(hstmt,SQL_ROWVER,  NULL, 0,
+                                   NULL, 0, (SQLCHAR *)"t_bug30081", SQL_NTS,
+                                   0, SQL_NO_NULLS));
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug30081");
+
+  return OK;
+}
+
+
+/**
+  Verify that we get correct data for SQL_DATA_TYPE and SQL_DATETIME_SUB
+  from SQLColumns(). Also check SQL_DESC_TYPE from SQLColAttribute().
+*/
+DECLARE_TEST(t_datecolumns)
+{
+  SQLCHAR col[10];
+  SQLINTEGER type;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_datecolumns");
+  ok_sql(hstmt,
+         "CREATE TABLE t_datecolumns(a TIMESTAMP, b DATETIME, c DATE, d TIME)");
+
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"t_datecolumns", SQL_NTS, NULL, 0));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, col, 4), "a", 1);
+  is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
+  is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIMESTAMP);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, col, 4), "b", 1);
+  is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
+  is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIMESTAMP);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, col, 4), "c", 1);
+  is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
+  is_num(my_fetch_int(hstmt, 15), SQL_TYPE_DATE);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, col, 4), "d", 1);
+  is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
+  is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIME);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "SELECT * FROM t_datecolumns");
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_TYPE, NULL, 0, NULL,
+                                 &type));
+  is_num(type, SQL_DATETIME);
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 2, SQL_DESC_TYPE, NULL, 0, NULL,
+                                 &type));
+  is_num(type, SQL_DATETIME);
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 3, SQL_DESC_TYPE, NULL, 0, NULL,
+                                 &type));
+  is_num(type, SQL_DATETIME);
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 4, SQL_DESC_TYPE, NULL, 0, NULL,
+                                 &type));
+  is_num(type, SQL_DATETIME);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_datecolumns");
+  return OK;
+}
+
+
+/**
+  Bug #14414: SQLColumn() does not return timestamp nullable attribute correctly
+*/
+DECLARE_TEST(t_bug14414)
+{
+  SQLCHAR col[10];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug14414");
+  ok_sql(hstmt, "CREATE TABLE t_bug14414(a TIMESTAMP, b TIMESTAMP NOT NULL,"
+        "c TIMESTAMP NULL)");
+
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"t_bug14414", SQL_NTS, NULL, 0));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, col, 4), "a", 1);
+  is_num(my_fetch_int(hstmt, 11), SQL_NULLABLE);
+  is_str(my_fetch_str(hstmt, col, 18), "YES", 3);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, col, 4), "b", 1);
+  is_num(my_fetch_int(hstmt, 11), SQL_NO_NULLS);
+  is_str(my_fetch_str(hstmt, col, 18), "NO", 3);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, col, 4), "c", 1);
+  is_num(my_fetch_int(hstmt, 11), SQL_NULLABLE);
+  is_str(my_fetch_str(hstmt, col, 18), "YES", 3);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug14414");
+  return OK;
+}
+
 
 BEGIN_TESTS
   ADD_TEST(my_ts)
@@ -652,6 +853,11 @@ BEGIN_TESTS
   ADD_TEST(t_time)
   ADD_TEST(t_time1)
   ADD_TEST(t_bug12520)
+  ADD_TEST(t_bug15773)
+  ADD_TEST(t_bug9927)
+  ADD_TODO(t_bug30081)
+  ADD_TEST(t_datecolumns)
+  ADD_TEST(t_bug14414)
 END_TESTS
 
 
