@@ -27,8 +27,8 @@ SQLRETURN rc;
 /* Basic prepared statements - binary protocol test */
 DECLARE_TEST(t_prep_basic)
 {
-    SQLINTEGER id, pcrow;
-    SQLLEN length1, length2;
+    SQLINTEGER id;
+    SQLLEN length1, length2, pcrow;
     char       name[20];
 
     SQLExecDirect(hstmt,"drop table t_prep_basic",SQL_NTS);
@@ -183,8 +183,7 @@ DECLARE_TEST(t_prep_buffer_length)
 /* For data truncation */
 DECLARE_TEST(t_prep_truncate)
 {
-    SQLINTEGER pcrow;
-    SQLLEN length, length1;
+    SQLLEN length, length1, pcrow;
     SQLCHAR    name[20], bin[10];
 
 
@@ -258,6 +257,10 @@ DECLARE_TEST(t_prep_scroll)
   ok_sql(hstmt, "CREATE TABLE t_prep_scroll (a TINYINT)");
   ok_sql(hstmt, "INSERT INTO t_prep_scroll VALUES (1),(2),(3),(4),(5)");
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
   ok_sql(hstmt, "SELECT * FROM t_prep_scroll");
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &data, 0, NULL));
@@ -527,7 +530,7 @@ DECLARE_TEST(t_prep_getdata1)
 DECLARE_TEST(t_prep_catalog)
 {
     SQLCHAR     table[20];
-    SQLINTEGER  length;
+    SQLLEN      length;
 
     SQLExecDirect(hstmt,"drop table t_prep_catalog",SQL_NTS);
 
@@ -565,7 +568,8 @@ DECLARE_TEST(t_prep_catalog)
 
     rc = SQLGetData(hstmt,3,SQL_C_CHAR,table,15,&length);
     mystmt(hstmt,rc);
-    myassert(strcmp(table,"t_prep_catalog") == 0 && length == 14);
+    is_num(length, 14);
+    is_str(table, "t_prep_catalog", 14);
 
     rc = SQLGetData(hstmt,4,SQL_C_CHAR,table,0,&length);
     mystmt(hstmt,rc);
@@ -678,7 +682,7 @@ DECLARE_TEST(t_prepare)
   SQLINTEGER nidata= 200, nodata;
   SQLLEN    nlen;
   char      szodata[20],szidata[20]="MySQL";
-  SQLROWCOUNT pccol;
+  SQLSMALLINT pccol;
 
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
@@ -871,7 +875,7 @@ DECLARE_TEST(tmysql_bindparam)
     SQLINTEGER nodata, nidata= 200;
     SQLLEN    nlen;
     SQLCHAR   szodata[20],szidata[20]="MySQL";
-    short     pccol;
+    SQLSMALLINT pccol;
 
     tmysql_exec(hstmt,"drop table tmysql_bindparam");
 
