@@ -25,139 +25,97 @@
 /* Testing SQL_FETCH_RELATIVE with row_set_size as 10 */
 DECLARE_TEST(t_relative)
 {
-    SQLRETURN rc;
-    SQLUINTEGER i, nrows;
-    SQLINTEGER  iarray[15];
-    long      index;
-    char      name[21];
+  SQLUINTEGER i, iarray[15];
+  SQLULEN nrows, index;
+  SQLCHAR name[21];
 
-    SQLExecDirect(hstmt,"drop table t_relative",SQL_NTS);
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
-    rc = SQLExecDirect(hstmt,"create table t_relative(id int,name char(20))",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative");
+  ok_sql(hstmt, "CREATE TABLE t_relative (id INT, name CHAR(20))");
 
-    rc = SQLPrepare(hstmt,"insert into t_relative values(?,?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"INSERT INTO t_relative VALUES (?,?)",
+                            SQL_NTS));
 
-    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
-                          SQL_INTEGER,0,0,&i,0,NULL);
-    mystmt(hstmt,rc);
-    rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT, SQL_C_CHAR,
-                          SQL_CHAR,20,0,name,20,NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_ULONG,
+                                  SQL_INTEGER, 0, 0, &i, 0, NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_CHAR, 20, 0, name, 20, NULL));
 
-    for ( i = 1; i <= 50; i++ )
-    {
-        sprintf(name,"my%d",i);
-        rc = SQLExecute(hstmt);
-        mystmt(hstmt,rc);
-    }
+  for (i= 1; i <= 50; i++)
+  {
+    sprintf((char *)name, "my%d", i);
+    ok_stmt(hstmt, SQLExecute(hstmt));
+  }
 
-    SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    SQLFreeStmt(hstmt,SQL_CLOSE);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLEndTran(SQL_HANDLE_DBC,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  /* set row size as 10 */
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)10, 0));
 
-    /* set row size as 10 */
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROW_ARRAY_SIZE,(SQLPOINTER)10,0);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &nrows, 0));
 
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "SELECT * FROM t_relative");
 
-    rc = SQLExecDirect(hstmt,"select * from t_relative",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_ULONG, &iarray, 0, NULL));
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 0)); /* 1-10 */
+  is_num(nrows, 10);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);/* 1-10 */
-    mystmt(hstmt,rc);
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index);
 
-    printMessage("1-10, total rows:%ld\n",nrows);
-    myassert(nrows == 10);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 0)); /* 10-20 */
+  is_num(nrows, 10);
 
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage("\n %d",iarray[index-1]);
-        myassert(iarray[index-1] == index);
-    }
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index + 10);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);/* 10-20 */
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_PREV, 0)); /* 1-10 */
+  is_num(nrows, 10);
 
-    printMessage("\n10-20, total rows:%d\n",nrows);
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index);
 
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d ",iarray[index-1]);
-        myassert(iarray[index-1] == index+10);
-    }
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1)); /* 2-11 */
+  is_num(nrows, 10);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_PREV,0);/* 1-10 */
-    mystmt(hstmt,rc);
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index + 1);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1)); /* 1-10 */
+  is_num(nrows, 10);
 
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d ",iarray[index-1]);
-        myassert(iarray[index-1] == index);
-    }
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* 2-11 */
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 0)); /* 1-10 */
+  is_num(nrows, 10);
 
-    printMessage("\n2-12, total rows:%d\n",nrows);
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index);
 
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d ",iarray[index-1]);
-        myassert(iarray[index-1] == index+1);
-    }
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND); /* BOF */
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* 1-10 */
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1)); /* 1-10 */
+  is_num(nrows, 10);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+  for (index= 1; index <= nrows; index++)
+    is_num(iarray[index - 1], index);
 
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d",iarray[index-1]);
-        myassert(iarray[index-1] == index);
-    }
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,0);/* 1-10 */
-    mystmt(hstmt,rc);
+  /* reset row size */
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)1, 0));
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
-
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d",iarray[index-1]);
-        myassert(iarray[index-1] == index);
-    }
-
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* BOF */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
-
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* 1-10 */
-    mystmt(hstmt,rc);
-
-    printMessage("\n1-10, total rows:%d\n",nrows);
-
-    for (index=1; index<=nrows; index++)
-    {
-        printMessage(" %d",iarray[index-1]);
-        myassert(iarray[index-1] == index);
-    }
-
-    SQLFreeStmt(hstmt,SQL_UNBIND);
-    SQLFreeStmt(hstmt,SQL_CLOSE);
-
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROW_ARRAY_SIZE,(SQLPOINTER)1,0);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative");
 
   return OK;
 }
@@ -166,204 +124,175 @@ DECLARE_TEST(t_relative)
 /* Testing SQL_FETCH_RELATIVE with row_set_size as 1 */
 DECLARE_TEST(t_relative1)
 {
-    SQLRETURN rc;
-    SQLINTEGER nrows;
-    SQLUINTEGER i;
-    const int max_rows=10;
+  SQLULEN nrows;
+  SQLUINTEGER i;
+  const SQLUINTEGER max_rows= 10;
 
-    SQLExecDirect(hstmt,"drop table t_relative1",SQL_NTS);
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
-    rc = SQLExecDirect(hstmt,"create table t_relative1(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative1");
+  ok_sql(hstmt, "CREATE TABLE t_relative1 (id INT)");
 
-    rc = SQLPrepare(hstmt,"insert into t_relative1 values(?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"INSERT INTO t_relative1 VALUES (?)",
+                            SQL_NTS));
 
-    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
-                          SQL_INTEGER,0,0,&i,0,NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_ULONG,
+                                  SQL_INTEGER, 0, 0, &i, 0, NULL));
 
-    for ( i = 1; i <= max_rows; i++ )
-    {
-        rc = SQLExecute(hstmt);
-        mystmt(hstmt,rc);
-    }
+  for (i= 1; i <= max_rows; i++)
+  {
+    ok_stmt(hstmt, SQLExecute(hstmt));
+  }
 
-    SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    SQLFreeStmt(hstmt,SQL_CLOSE);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLEndTran(SQL_HANDLE_DBC,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  /* set row_size as 1 */
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)1, 0));
 
-    /* set row_size as 1 */
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROW_ARRAY_SIZE,(SQLPOINTER)1,0);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &nrows, 0));
 
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "SELECT * FROM t_relative1");
 
-    rc = SQLExecDirect(hstmt,"select * from t_relative1",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &i, 0, NULL));
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&i,0,NULL);
-    mystmt(hstmt,rc);
+  /* row 1 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 0));
+  is_num(i, 1);
 
-    /* row 1 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);/* 1 */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* Before start */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND);
 
-    /* Before start */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* jump to last row */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, max_rows));
+  is_num(i, max_rows);
 
-    /* jump to last row */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,max_rows);/* last row */
-    mystmt(hstmt,rc);
-    my_assert(i==max_rows);
+  /* jump to last row+1 */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1),
+              SQL_NO_DATA_FOUND);
 
-    /* jump to last row+1 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* after last */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* goto first row */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
-    /* goto first row */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* 1 */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* before start */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND);
 
-    /* before start */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* goto fifth  row */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 5));
+  is_num(i, 5);
 
-    /* goto fifth  row */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,5);/* 5 */
-    mystmt(hstmt,rc);
-    my_assert(i==5);
+  /* goto after end */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, max_rows),
+              SQL_NO_DATA_FOUND);
 
-    /* goto after end */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,max_rows);/* after last */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* the scenarios from ODBC spec */
 
-    /*
-       the scenarios from ODBC spec
-    */
+  /* CASE 1 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
-    /* CASE 1 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* 1 */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* BeforeStart AND FetchOffset <= 0 */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -20),
+              SQL_NO_DATA_FOUND);
 
-    /* BeforeStart AND FetchOffset <= 0 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-20);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 0),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,0);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* case 1: Before start AND FetchOffset > 0 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1));
+  is_num(i, 1);
 
-    /* case 1: Before start AND FetchOffset > 0 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* 1 */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* CASE 2 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_LAST, 1));
+  is_num(i, max_rows);
 
-    /* CASE 2 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_LAST,1);/* last row */
-    mystmt(hstmt,rc);
-    my_assert(i==max_rows);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* after end */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* After end AND FetchOffset >= 0 */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 10),
+              SQL_NO_DATA_FOUND);
 
-    /* After end AND FetchOffset >= 0 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,10);/* after end */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 20),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,20);/* after end */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 1),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* after end */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 0),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,0);/* after end */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  /* After end AND FetchOffset < 0 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1));
+  is_num(i, max_rows);
 
-    /* After end AND FetchOffset < 0 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* last row */
-    mystmt(hstmt,rc);
-    my_assert(i==max_rows);
+  /* CASE 3 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
+  /* CurrRowsetStart = 1 AND FetchOffset < 0 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 0));
+  is_num(i, 1);
 
-    /* CASE 3 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* first row */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -1),
+              SQL_NO_DATA_FOUND);
 
-    /* CurrRowsetStart = 1 AND FetchOffset < 0 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,0);/* first row */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* CASE 4 */
+  /* CurrRowsetStart > 1 AND CurrRowsetStart + FetchOffset < 1 AND
+     | FetchOffset | > RowsetSize
+  */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 3));
+  is_num(i, 4);
 
-    /* CASE 4 */
-    /* CurrRowsetStart > 1 AND CurrRowsetStart + FetchOffset < 1 AND
-       | FetchOffset | > RowsetSize
-    */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* first row */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* the following call satisfies 4 > 1 AND (3-4) < 1 AND |-4| > 1 */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -4),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,3);/* fourth row */
-    mystmt(hstmt,rc);
-    my_assert(i==4);
+  /* CASE 5 */
+  /* 1 <= CurrRowsetStart + FetchOffset <= LastResultRow */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
-    /* the following call satisfies 4 > 1 AND (3-4) < 1 AND |-4| > 1 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-4);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 5));
+  is_num(i, 6);
 
-    /* CASE 5 */
-    /* 1 <= CurrRowsetStart + FetchOffset <= LastResultRow */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* first row */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* 1 <= 6-2 <= 10 */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -2));
+  is_num(i, 4);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,5);/* sixth row */
-    mystmt(hstmt,rc);
-    my_assert(i==6);
+  /* CASE 6 */
+  /*  CurrRowsetStart > 1 AND CurrRowsetStart + FetchOffset < 1 AND
+      | FetchOffset | <= RowsetSize
+   */
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 1));
+  is_num(i, 1);
 
-    /* 1 <= 6-2 <= 10 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-2);/* fourth row */
-    mystmt(hstmt,rc);
-    my_assert(i==4);
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, 3));
+  is_num(i, 4);
 
-    /* CASE 6 */
-    /*  CurrRowsetStart > 1 AND CurrRowsetStart + FetchOffset < 1 AND
-        | FetchOffset | <= RowsetSize
-     */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,1);/* first row */
-    mystmt(hstmt,rc);
-    my_assert(i==1);
+  /* 4 >1 AND 4-4 <1 AND |-4| <=10 */
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_RELATIVE, -4),
+              SQL_NO_DATA_FOUND);
 
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,3);/* fourth row */
-    mystmt(hstmt,rc);
-    my_assert(i==4);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    /* 4 >1 AND 4-4 <1 AND |-4| <=10 */
-    rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-4);/* before start */
-    mystmt_err(hstmt,rc==SQL_NO_DATA_FOUND,rc);
-
-
-    SQLFreeStmt(hstmt,SQL_UNBIND);
-    SQLFreeStmt(hstmt,SQL_CLOSE);
-
-    rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROW_ARRAY_SIZE,(SQLPOINTER)1,0);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative1");
 
   return OK;
 }
@@ -372,10 +301,13 @@ DECLARE_TEST(t_relative1)
 /* Testing SQL_FETCH_RELATIVE with row_set_size as 2 */
 DECLARE_TEST(t_relative2)
 {
-    SQLRETURN rc;
-    SQLINTEGER nrows,iarray[15];
-    SQLUINTEGER i;
-    const int max_rows=10;
+  SQLRETURN rc;
+  SQLULEN nrows;
+  SQLUINTEGER i, iarray[15];
+  const SQLUINTEGER max_rows=10;
+
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     SQLExecDirect(hstmt,"drop table t_relative2",SQL_NTS);
 
@@ -616,7 +548,7 @@ DECLARE_TEST(t_relative2)
 DECLARE_TEST(t_rows_fetched_ptr)
 {
     SQLRETURN    rc;
-    SQLINTEGER   rowsFetched, rowsSize;
+    SQLULEN rowsFetched, rowsSize;
     long         i;
 
     SQLExecDirect(hstmt,"drop table t_rows_fetched_ptr",SQL_NTS);
@@ -748,7 +680,7 @@ DECLARE_TEST(t_rows_fetched_ptr)
 DECLARE_TEST(t_rows_fetched_ptr1)
 {
   SQLRETURN   rc;
-  SQLLEN      rowsFetched, rowsSize;
+  SQLULEN     rowsFetched, rowsSize;
   SQLINTEGER  i;
 
     SQLExecDirect(hstmt,"drop table t_rows_fetched_ptr",SQL_NTS);
