@@ -26,7 +26,8 @@
 DECLARE_TEST(my_resultset)
 {
     SQLRETURN   rc;
-    SQLUINTEGER nRowCount=0, pcColDef;
+    SQLUINTEGER nRowCount=0;
+    SQLULEN     pcColDef;
     SQLCHAR     szColName[MAX_NAME_LEN];
     SQLCHAR     szData[MAX_ROW_DATA_LEN+1];
     SQLSMALLINT nIndex,ncol,pfSqlType, pcbScale, pfNullable;
@@ -257,14 +258,14 @@ static void desc_col_check(SQLHSTMT hstmt,
                            SQLUSMALLINT icol,
                            const char *name,
                            SQLSMALLINT sql_type,
-                           SQLUINTEGER col_def,
-                           SQLUINTEGER col_def1,
+                           SQLULEN col_def,
+                           SQLULEN col_def1,
                            SQLSMALLINT scale,
                            SQLSMALLINT nullable)
 {
   SQLRETURN   rc;
   SQLSMALLINT pcbColName, pfSqlType, pibScale, pfNullable;
-  SQLUINTEGER pcbColDef;
+  SQLULEN     pcbColDef;
   SQLCHAR     szColName[MAX_NAME_LEN];
 
   rc = SQLDescribeCol(hstmt, icol,
@@ -333,29 +334,31 @@ DECLARE_TEST(t_desc_col)
     fprintf(stdout,"total columns: %d\n", ColumnCount);
     my_assert(ColumnCount == 23);
 
-    desc_col_check(hstmt, 1,  "c1",  SQL_INTEGER,   10, 11, 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 2,  "c2",  SQL_BINARY,    2,  2,  0,  SQL_NO_NULLS);
+    desc_col_check(hstmt, 1,  "c1",  SQL_INTEGER,   10, 10, 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 2,  "c2",  SQL_BINARY,    4,  2,  0,  SQL_NO_NULLS);
     desc_col_check(hstmt, 3,  "c3",  SQL_CHAR,      1,  1,  0,  SQL_NULLABLE);
     desc_col_check(hstmt, 4,  "c4",  SQL_VARCHAR,   5,  5,  0,  SQL_NULLABLE);
     desc_col_check(hstmt, 5,  "c5",  SQL_DECIMAL,   10, 10, 3,  SQL_NO_NULLS);
     desc_col_check(hstmt, 6,  "c6",  SQL_TINYINT,   3,  4,  0,  SQL_NULLABLE);
     desc_col_check(hstmt, 7,  "c7",  SQL_SMALLINT,  5,  6,  0,  SQL_NULLABLE);
     desc_col_check(hstmt, 8,  "c8",  SQL_DECIMAL,   4,  4,  2,  SQL_NULLABLE);
-    desc_col_check(hstmt, 9,  "c9",  SQL_DOUBLE,    22, 24, 31, SQL_NULLABLE);
-    desc_col_check(hstmt, 10, "c10", SQL_REAL,      12, 24, 31, SQL_NULLABLE);
+    desc_col_check(hstmt, 9,  "c9",  SQL_DOUBLE,    15, 15, 0, SQL_NULLABLE);
+    desc_col_check(hstmt, 10, "c10", SQL_REAL,      7,  7,  0, SQL_NULLABLE);
     desc_col_check(hstmt, 11, "c11", SQL_BIGINT,    19, 19, 0,  SQL_NO_NULLS);
-    desc_col_check(hstmt, 12, "c12", SQL_VARBINARY, 12, 12, 0,  SQL_NULLABLE);
+
+    desc_col_check(hstmt, 12, "c12", SQL_VARBINARY, 24, 24, 0,  SQL_NULLABLE);
+
     desc_col_check(hstmt, 13, "c13", SQL_CHAR,      20, 20, 0,  SQL_NO_NULLS);
-    desc_col_check(hstmt, 14, "c14", SQL_REAL,      10, 24, 3,  SQL_NULLABLE);
+    desc_col_check(hstmt, 14, "c14", SQL_REAL,      7,  7,  0,  SQL_NULLABLE);
     desc_col_check(hstmt, 15, "c15", SQL_LONGVARCHAR, 255, 255, 0,  SQL_NULLABLE);
     desc_col_check(hstmt, 16, "c16", SQL_LONGVARCHAR, 65535, 65535, 0,  SQL_NULLABLE);
     desc_col_check(hstmt, 17, "c17", SQL_LONGVARCHAR, 16777215, 16777215, 0,  SQL_NULLABLE);
     desc_col_check(hstmt, 18, "c18", SQL_LONGVARCHAR, 4294967295 , 16777215 , 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 19, "c19", SQL_LONGVARBINARY, 255, 255, 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 20, "c20", SQL_LONGVARBINARY, 65535, 65535, 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 21, "c21", SQL_LONGVARBINARY, 16777215, 16777215, 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295 , 16777215 , 0,  SQL_NULLABLE);
-    desc_col_check(hstmt, 23, "c23", SQL_LONGVARBINARY, 255, 5, 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 19, "c19", SQL_LONGVARBINARY, 255 * 2, 255, 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 20, "c20", SQL_LONGVARBINARY, 65535 * 2, 65535, 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 21, "c21", SQL_LONGVARBINARY, 16777215 * 2, 16777215, 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295 * 2 , 16777215 , 0,  SQL_NULLABLE);
+    desc_col_check(hstmt, 23, "c23", SQL_LONGVARBINARY, 255 * 2, 5, 0,  SQL_NULLABLE);
 
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
@@ -1046,8 +1049,9 @@ DECLARE_TEST(t_empty_str_bug)
 
     my_assert( 1 == myresult(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = tmysql_exec(hstmt,"select * from t_empty_str_bug");
     mystmt(hstmt,rc);
@@ -1123,7 +1127,7 @@ DECLARE_TEST(t_desccol)
 
 
 void desccol(SQLHSTMT hstmt, SQLCHAR    *cname,  SQLSMALLINT clen,
-             SQLSMALLINT sqltype,SQLUINTEGER size,
+             SQLSMALLINT sqltype,SQLULEN size,
              SQLSMALLINT scale,SQLSMALLINT isNull)
 {
     SQLRETURN   rc =0;
@@ -1237,19 +1241,19 @@ DECLARE_TEST(t_desccolext)
     rc = SQLFreeStmt(hstmt,SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    desccol(hstmt,"t1",2,SQL_TINYINT,4,0,SQL_NULLABLE);
-    desccol(hstmt,"t2",2,SQL_TINYINT,10,0,SQL_NULLABLE);
+    desccol(hstmt,"t1",2,SQL_TINYINT,3,0,SQL_NULLABLE);
+    desccol(hstmt,"t2",2,SQL_TINYINT,3,0,SQL_NULLABLE);
     desccol(hstmt,"t3",2,SQL_TINYINT,3,0,SQL_NULLABLE);
 
-    desccol(hstmt,"s1",2,SQL_SMALLINT,6,0,SQL_NULLABLE);
-    desccol(hstmt,"s2",2,SQL_SMALLINT,10,0,SQL_NULLABLE);
+    desccol(hstmt,"s1",2,SQL_SMALLINT,5,0,SQL_NULLABLE);
+    desccol(hstmt,"s2",2,SQL_SMALLINT,5,0,SQL_NULLABLE);
     desccol(hstmt,"s3",2,SQL_SMALLINT,5,0,SQL_NULLABLE);
 
-    desccol(hstmt,"m1",2,SQL_INTEGER,9,0,SQL_NULLABLE);
-    desccol(hstmt,"m2",2,SQL_INTEGER,10,0,SQL_NULLABLE);
+    desccol(hstmt,"m1",2,SQL_INTEGER,8,0,SQL_NULLABLE);
+    desccol(hstmt,"m2",2,SQL_INTEGER,8,0,SQL_NULLABLE);
     desccol(hstmt,"m3",2,SQL_INTEGER,8,0,SQL_NULLABLE);
 
-    desccol(hstmt,"i1",2,SQL_INTEGER,11,0,SQL_NULLABLE);
+    desccol(hstmt,"i1",2,SQL_INTEGER,10,0,SQL_NULLABLE);
     desccol(hstmt,"i2",2,SQL_INTEGER,10,0,SQL_NO_NULLS);
     desccol(hstmt,"i3",2,SQL_INTEGER,10,0,SQL_NULLABLE);
     desccol(hstmt,"i4",2,SQL_INTEGER,10,0,SQL_NULLABLE);
@@ -1258,15 +1262,15 @@ DECLARE_TEST(t_desccolext)
     desccol(hstmt,"b2",2,SQL_BIGINT,19,0,SQL_NULLABLE);
     desccol(hstmt,"b3",2,SQL_BIGINT,20,0,SQL_NULLABLE);
 
-    desccol(hstmt,"f1",2,SQL_REAL,12,31,SQL_NULLABLE);
-    desccol(hstmt,"f2",2,SQL_REAL,12,31,SQL_NULLABLE);
-    desccol(hstmt,"f3",2,SQL_REAL,12,31,SQL_NULLABLE);
-    desccol(hstmt,"f4",2,SQL_REAL,10,4,SQL_NULLABLE);
+    desccol(hstmt,"f1",2,SQL_REAL,7,0,SQL_NULLABLE);
+    desccol(hstmt,"f2",2,SQL_REAL,7,0,SQL_NULLABLE);
+    desccol(hstmt,"f3",2,SQL_REAL,7,0,SQL_NULLABLE);
+    desccol(hstmt,"f4",2,SQL_REAL,7,0,SQL_NULLABLE);
 
-    desccol(hstmt,"d1",2,SQL_DOUBLE,22,31,SQL_NULLABLE);
-    desccol(hstmt,"d2",2,SQL_DOUBLE,30,3,SQL_NULLABLE);
-    desccol(hstmt,"d3",2,SQL_DOUBLE,22,31,SQL_NULLABLE);
-    desccol(hstmt,"d4",2,SQL_DOUBLE,30,3,SQL_NULLABLE);
+    desccol(hstmt,"d1",2,SQL_DOUBLE,15,0,SQL_NULLABLE);
+    desccol(hstmt,"d2",2,SQL_DOUBLE,15,0,SQL_NULLABLE);
+    desccol(hstmt,"d3",2,SQL_DOUBLE,15,0,SQL_NULLABLE);
+    desccol(hstmt,"d4",2,SQL_DOUBLE,15,0,SQL_NULLABLE);
 
     rc = SQLFreeStmt(hstmt,SQL_CLOSE);
     mystmt(hstmt,rc);
@@ -1338,36 +1342,11 @@ DECLARE_TEST(t_desccol1)
 }
 
 
-static int colattr(SQLHSTMT hstmt, SQLUSMALLINT cno, SQLUSMALLINT attribute,
-                   SQLCHAR *sptr, SQLSMALLINT slen, SQLINTEGER nptr)
-{
-  SQLCHAR     lsptr[40];
-  SQLSMALLINT lslen;
-  SQLLEN      lnptr;
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
-  ok_sql(hstmt, "SELECT * FROM t_colattr");
-
-  ok_stmt(hstmt, SQLColAttributes(hstmt, cno, attribute, lsptr, sizeof(lsptr),
-                                  &lslen, &lnptr));
-  if (sptr)
-  {
-    is_num(lslen, slen);
-    is_str(lsptr, sptr, lslen);
-  }
-  else
-  {
-    is_num(lnptr, nptr);
-  }
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  return OK;
-}
-
-
 DECLARE_TEST(t_colattributes)
 {
+  SQLLEN count;
+  SQLINTEGER isauto;
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_colattr");
 
   ok_sql(hstmt,
@@ -1428,10 +1407,16 @@ DECLARE_TEST(t_colattributes)
          "en ENUM('v1','v2'),"
          "st SET('1','2','3'))");
 
-  if (colattr(hstmt, 1, SQL_COLUMN_COUNT, NULL, 0, 55) != OK)
-    return FAIL;
-  if (colattr(hstmt, 1, SQL_COLUMN_AUTO_INCREMENT, NULL, 0, SQL_TRUE) != OK)
-    return FAIL;
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+  ok_sql(hstmt, "SELECT * FROM t_colattr");
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_COLUMN_COUNT, NULL, 0, NULL,
+                                 &count));
+  is(count == 55);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_COLUMN_AUTO_INCREMENT, NULL, 0,
+                                 NULL, &isauto));
+  is_num(isauto, SQL_TRUE);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_colattr");
 
@@ -1444,6 +1429,9 @@ DECLARE_TEST(t_exfetch)
     SQLRETURN rc;
     SQLUINTEGER i;
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     tmysql_exec(hstmt,"drop table t_exfetch");
 
     rc = tmysql_exec(hstmt,"create table t_exfetch(col1 int)");
@@ -1603,8 +1591,9 @@ DECLARE_TEST(tmysql_rowstatus)
     rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
     mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
 
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE  ,(SQLPOINTER)1 , 0);
     mystmt(hstmt, rc);
@@ -1705,6 +1694,127 @@ DECLARE_TEST(t_true_length)
 }
 
 
+/**
+ Bug #27544: Calling stored procedure causes "Out of sync" and "Lost
+connection" errors
+*/
+DECLARE_TEST(t_bug27544)
+{
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t1");
+  ok_sql(hstmt, "CREATE TABLE t1(a int)");
+  ok_sql(hstmt, "INSERT INTO t1 VALUES (1)");
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS p1");
+  ok_sql(hstmt, "CREATE PROCEDURE p1() BEGIN"
+                "   SELECT a FROM t1; "
+                "END;");
+
+  ok_sql(hstmt,"CALL p1()");
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE p1");
+  ok_sql(hstmt, "DROP TABLE t1");
+
+  return OK;
+}
+
+
+/**
+ Bug #6157: BUG in the alias use with ADO's Object
+*/
+DECLARE_TEST(bug6157)
+{
+  SQLCHAR name[30];
+  SQLSMALLINT len;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug6157");
+  ok_sql(hstmt, "CREATE TABLE t_bug6157 (a INT)");
+  ok_sql(hstmt, "INSERT INTO t_bug6157 VALUES (1)");
+
+  ok_sql(hstmt, "SELECT a AS b FROM t_bug6157 AS c");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_NAME,
+                                 name, sizeof(name), &len, NULL));
+  is_str(name, "b", 1);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_BASE_COLUMN_NAME,
+                                 name, sizeof(name), &len, NULL));
+  is_str(name, "a", 1);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_TABLE_NAME,
+                                 name, sizeof(name), &len, NULL));
+  is_str(name, "c", 1);
+
+  ok_stmt(hstmt, SQLColAttribute(hstmt, 1, SQL_DESC_BASE_TABLE_NAME,
+                                 name, sizeof(name), &len, NULL));
+  is_str(name, "t_bug6157", 9);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug6157");
+  return OK;
+}
+
+
+/**
+Bug #16817: ODBC doesn't return multiple resultsets
+*/
+DECLARE_TEST(t_bug16817)
+{
+  SQLCHAR name[30];
+  SQLSMALLINT len;
+  SQLSMALLINT ncol;
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS p_bug16817");
+  ok_sql(hstmt, "CREATE PROCEDURE p_bug16817 () "
+                "BEGIN "
+                "  SELECT 'Marten' FROM DUAL; "
+                "  SELECT 'Zack' FROM DUAL; "
+               "END");
+
+  ok_sql(hstmt, "CALL p_bug16817()");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, name, 1), "Marten", 6);
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, name, 1), "Zack", 4);
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 0);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE p_bug16817");
+  return OK;
+}
+
+
+DECLARE_TEST(t_binary_collation)
+{
+  SQLSMALLINT name_length, data_type, decimal_digits, nullable;
+  SQLCHAR column_name[SQL_MAX_COLUMN_NAME_LEN];
+  SQLULEN column_size;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_binary_collation");
+  ok_sql(hstmt, "CREATE TABLE t_binary_collation (id INT)");
+  ok_sql(hstmt, "SHOW CREATE TABLE t_binary_collation");
+
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, column_name, sizeof(column_name),
+                                &name_length, &data_type, &column_size,
+                                &decimal_digits, &nullable));
+  is_num(data_type, SQL_VARCHAR);
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_binary_collation");
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_resultset)
   ADD_TEST(t_convert_type)
@@ -1723,6 +1833,10 @@ BEGIN_TESTS
   ADD_TEST(t_exfetch)
   ADD_TEST(tmysql_rowstatus)
   ADD_TEST(t_true_length)
+  ADD_TEST(t_bug27544)
+  ADD_TEST(t_bug16817)
+  ADD_TEST(bug6157)
+  ADD_TEST(t_binary_collation)
 END_TESTS
 
 
