@@ -32,6 +32,10 @@ DECLARE_TEST(t_scroll)
   ok_sql(hstmt, "CREATE TABLE t_scroll (col1 INT)");
   ok_sql(hstmt, "INSERT INTO t_scroll VALUES (1),(2),(3),(4),(5)");
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
   ok_sql(hstmt, "SELECT * FROM t_scroll");
 
   ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_ULONG, &i, 0, NULL));
@@ -117,10 +121,14 @@ DECLARE_TEST(t_scroll)
 DECLARE_TEST(t_array_relative_10)
 {
     SQLRETURN rc;
-    SQLINTEGER nrows, iarray[15];
+    SQLINTEGER iarray[15];
+    SQLLEN   nrows;
     SQLUINTEGER i,index;
     char name[21];
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     SQLExecDirect(hstmt,"drop table t_array_relative_10",SQL_NTS);
 
     rc = SQLExecDirect(hstmt,"create table t_array_relative_10(id int,name char(20))",SQL_NTS);
@@ -153,19 +161,21 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROW_ARRAY_SIZE,(SQLPOINTER)10,0);
     mystmt(hstmt,rc);
 
+    /* According to http://support.microsoft.com/kb/298678, the storage
+       pointed to if SQL_ATTR_ROWS_FETCHED_PTR should be SQLLEN */
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
     rc = SQLExecDirect(hstmt,"select * from t_array_relative_10",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
 
     rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);/* 1-10 */
     mystmt(hstmt,rc);
 
-    printMessage("1-10, total rows:%d\n",nrows);
+    printMessage("1-10, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -176,7 +186,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);/* 10-20 */
     mystmt(hstmt,rc);
 
-    printMessage("\n10-20, total rows:%d\n",nrows);
+    printMessage("\n10-20, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -187,7 +197,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_PREV,0);/* 1-10 */
     mystmt(hstmt,rc);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+    printMessage("\n1-10, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -198,7 +208,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* 2-11 */
     mystmt(hstmt,rc);
 
-    printMessage("\n2-12, total rows:%d\n",nrows);
+    printMessage("\n2-12, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -209,7 +219,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,-1);/* 1-10 */
     mystmt(hstmt,rc);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+    printMessage("\n1-10, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -220,7 +230,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_FIRST,0);/* 1-10 */
     mystmt(hstmt,rc);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+    printMessage("\n1-10, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -234,7 +244,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLFetchScroll(hstmt,SQL_FETCH_RELATIVE,1);/* 1-10 */
     mystmt(hstmt,rc);
 
-    printMessage("\n1-10, total rows:%d\n",nrows);
+    printMessage("\n1-10, total rows:%ld\n",(long)nrows);
 
     for (index=1; index<=nrows; index++)
     {
@@ -256,10 +266,13 @@ DECLARE_TEST(t_array_relative_10)
 DECLARE_TEST(t_relative_1)
 {
     SQLRETURN rc;
-    SQLINTEGER nrows;
+    SQLLEN nrows;
     SQLUINTEGER i;
     const int max_rows=10;
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     SQLExecDirect(hstmt,"drop table t_relative_1",SQL_NTS);
 
     rc = SQLExecDirect(hstmt,"create table t_relative_1(id int)",SQL_NTS);
@@ -462,10 +475,14 @@ DECLARE_TEST(t_relative_1)
 DECLARE_TEST(t_array_relative_2)
 {
     SQLRETURN rc;
-    SQLUINTEGER i, nrows;
+    SQLUINTEGER i;
+    SQLLEN nrows;
     SQLINTEGER iarray[15];
     const int max_rows=10;
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     SQLExecDirect(hstmt,"drop table t_array_relative_2",SQL_NTS);
 
     rc = SQLExecDirect(hstmt,"create table t_array_relative_2(id int)",SQL_NTS);
@@ -500,7 +517,7 @@ DECLARE_TEST(t_array_relative_2)
     rc = SQLExecDirect(hstmt,"select * from t_array_relative_2",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
 
     /* row 1 */
@@ -707,7 +724,7 @@ DECLARE_TEST(t_array_relative_2)
     rc = SQLExecDirect(hstmt,"select * from t_array_relative_2",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
 
     rc = SQLFetchScroll(hstmt,SQL_FETCH_ABSOLUTE,2);/* 2 */    
@@ -770,10 +787,13 @@ DECLARE_TEST(t_array_relative_2)
 DECLARE_TEST(t_absolute_1)
 {
     SQLRETURN rc;
-    SQLINTEGER nrows;
+    SQLLEN nrows;
     SQLUINTEGER i;
     const int max_rows=10;
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     SQLExecDirect(hstmt,"drop table t_absolute_1",SQL_NTS);
 
     rc = SQLExecDirect(hstmt,"create table t_absolute_1(id int)",SQL_NTS);
@@ -951,10 +971,14 @@ DECLARE_TEST(t_absolute_1)
 DECLARE_TEST(t_absolute_2)
 {
     SQLRETURN rc;
-    SQLINTEGER nrows, iarray[15];
+    SQLLEN nrows;
+    SQLINTEGER iarray[15];
     const int max_rows=10;
     SQLUINTEGER i;
 
+    ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
     SQLExecDirect(hstmt,"drop table t_absolute_2",SQL_NTS);
 
     rc = SQLExecDirect(hstmt,"create table t_absolute_2(id int)",SQL_NTS);
@@ -989,7 +1013,7 @@ DECLARE_TEST(t_absolute_2)
     rc = SQLExecDirect(hstmt,"select * from t_absolute_2",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
 
     /* row 1 */
@@ -1156,7 +1180,7 @@ DECLARE_TEST(t_absolute_2)
     rc = SQLExecDirect(hstmt,"select * from t_absolute_2",SQL_NTS);
     mystmt(hstmt,rc);
 
-    rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
+    rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
 
     rc = SQLFetchScroll(hstmt,SQL_FETCH_ABSOLUTE,2);/* 2 */    
